<?php

return [
    'Names' => [
        'af' => 'afrikaans',
        'af_NA' => 'afrikaans (Namibia)',
        'af_ZA' => 'afrikaans (Sydafrika)',
        'ak' => 'akan',
        'ak_GH' => 'akan (Ghana)',
        'am' => 'amharisk',
        'am_ET' => 'amharisk (Etiopien)',
        'ar' => 'arabisk',
        'ar_001' => 'arabisk (Verden)',
        'ar_AE' => 'arabisk (De Forenede Arabiske Emirater)',
        'ar_BH' => 'arabisk (Bahrain)',
        'ar_DJ' => 'arabisk (Djibouti)',
        'ar_DZ' => 'arabisk (Algeriet)',
        'ar_EG' => 'arabisk (Egypten)',
        'ar_EH' => 'arabisk (Vestsahara)',
        'ar_ER' => 'arabisk (Eritrea)',
        'ar_IL' => 'arabisk (Israel)',
        'ar_IQ' => 'arabisk (Irak)',
        'ar_JO' => 'arabisk (Jordan)',
        'ar_KM' => 'arabisk (Comorerne)',
        'ar_KW' => 'arabisk (Kuwait)',
        'ar_LB' => 'arabisk (Libanon)',
        'ar_LY' => 'arabisk (Libyen)',
        'ar_MA' => 'arabisk (Marokko)',
        'ar_MR' => 'arabisk (Mauretanien)',
        'ar_OM' => 'arabisk (Oman)',
        'ar_PS' => 'arabisk (De palæstinensiske områder)',
        'ar_QA' => 'arabisk (Qatar)',
        'ar_SA' => 'arabisk (Saudi-Arabien)',
        'ar_SD' => 'arabisk (Sudan)',
        'ar_SO' => 'arabisk (Somalia)',
        'ar_SS' => 'arabisk (Sydsudan)',
        'ar_SY' => 'arabisk (Syrien)',
        'ar_TD' => 'arabisk (Tchad)',
        'ar_TN' => 'arabisk (Tunesien)',
        'ar_YE' => 'arabisk (Yemen)',
        'as' => 'assamesisk',
        'as_IN' => 'assamesisk (Indien)',
        'az' => 'aserbajdsjansk',
        'az_AZ' => 'aserbajdsjansk (Aserbajdsjan)',
        'az_Cyrl' => 'aserbajdsjansk (kyrillisk)',
        'az_Cyrl_AZ' => 'aserbajdsjansk (kyrillisk, Aserbajdsjan)',
        'az_Latn' => 'aserbajdsjansk (latinsk)',
        'az_Latn_AZ' => 'aserbajdsjansk (latinsk, Aserbajdsjan)',
        'ba' => 'bashkir',
        'ba_RU' => 'bashkir (Rusland)',
        'be' => 'belarusisk',
        'be_BY' => 'belarusisk (Belarus)',
        'bg' => 'bulgarsk',
        'bg_BG' => 'bulgarsk (Bulgarien)',
        'bm' => 'bambara',
        'bm_ML' => 'bambara (Mali)',
        'bn' => 'bengali',
        'bn_BD' => 'bengali (Bangladesh)',
        'bn_IN' => 'bengali (Indien)',
        'bo' => 'tibetansk',
        'bo_CN' => 'tibetansk (Kina)',
        'bo_IN' => 'tibetansk (Indien)',
        'br' => 'bretonsk',
        'br_FR' => 'bretonsk (Frankrig)',
        'bs' => 'bosnisk',
        'bs_BA' => 'bosnisk (Bosnien-Hercegovina)',
        'bs_Cyrl' => 'bosnisk (kyrillisk)',
        'bs_Cyrl_BA' => 'bosnisk (kyrillisk, Bosnien-Hercegovina)',
        'bs_Latn' => 'bosnisk (latinsk)',
        'bs_Latn_BA' => 'bosnisk (latinsk, Bosnien-Hercegovina)',
        'ca' => 'catalansk',
        'ca_AD' => 'catalansk (Andorra)',
        'ca_ES' => 'catalansk (Spanien)',
        'ca_FR' => 'catalansk (Frankrig)',
        'ca_IT' => 'catalansk (Italien)',
        'ce' => 'tjetjensk',
        'ce_RU' => 'tjetjensk (Rusland)',
        'cs' => 'tjekkisk',
        'cs_CZ' => 'tjekkisk (Tjekkiet)',
        'cv' => 'tjuvasjisk',
        'cv_RU' => 'tjuvasjisk (Rusland)',
        'cy' => 'walisisk',
        'cy_GB' => 'walisisk (Storbritannien)',
        'da' => 'dansk',
        'da_DK' => 'dansk (Danmark)',
        'da_GL' => 'dansk (Grønland)',
        'de' => 'tysk',
        'de_AT' => 'tysk (Østrig)',
        'de_BE' => 'tysk (Belgien)',
        'de_CH' => 'tysk (Schweiz)',
        'de_DE' => 'tysk (Tyskland)',
        'de_IT' => 'tysk (Italien)',
        'de_LI' => 'tysk (Liechtenstein)',
        'de_LU' => 'tysk (Luxembourg)',
        'dz' => 'dzongkha',
        'dz_BT' => 'dzongkha (Bhutan)',
        'ee' => 'ewe',
        'ee_GH' => 'ewe (Ghana)',
        'ee_TG' => 'ewe (Togo)',
        'el' => 'græsk',
        'el_CY' => 'græsk (Cypern)',
        'el_GR' => 'græsk (Grækenland)',
        'en' => 'engelsk',
        'en_001' => 'engelsk (Verden)',
        'en_150' => 'engelsk (Europa)',
        'en_AE' => 'engelsk (De Forenede Arabiske Emirater)',
        'en_AG' => 'engelsk (Antigua og Barbuda)',
        'en_AI' => 'engelsk (Anguilla)',
        'en_AS' => 'engelsk (Amerikansk Samoa)',
        'en_AT' => 'engelsk (Østrig)',
        'en_AU' => 'engelsk (Australien)',
        'en_BB' => 'engelsk (Barbados)',
        'en_BE' => 'engelsk (Belgien)',
        'en_BI' => 'engelsk (Burundi)',
        'en_BM' => 'engelsk (Bermuda)',
        'en_BS' => 'engelsk (Bahamas)',
        'en_BW' => 'engelsk (Botswana)',
        'en_BZ' => 'engelsk (Belize)',
        'en_CA' => 'engelsk (Canada)',
        'en_CC' => 'engelsk (Cocosøerne)',
        'en_CH' => 'engelsk (Schweiz)',
        'en_CK' => 'engelsk (Cookøerne)',
        'en_CM' => 'engelsk (Cameroun)',
        'en_CX' => 'engelsk (Juleøen)',
        'en_CY' => 'engelsk (Cypern)',
        'en_CZ' => 'engelsk (Tjekkiet)',
        'en_DE' => 'engelsk (Tyskland)',
        'en_DK' => 'engelsk (Danmark)',
        'en_DM' => 'engelsk (Dominica)',
        'en_EE' => 'engelsk (Estland)',
        'en_ER' => 'engelsk (Eritrea)',
        'en_ES' => 'engelsk (Spanien)',
        'en_FI' => 'engelsk (Finland)',
        'en_FJ' => 'engelsk (Fiji)',
        'en_FK' => 'engelsk (Falklandsøerne)',
        'en_FM' => 'engelsk (Mikronesien)',
        'en_FR' => 'engelsk (Frankrig)',
        'en_GB' => 'engelsk (Storbritannien)',
        'en_GD' => 'engelsk (Grenada)',
        'en_GE' => 'engelsk (Georgien)',
        'en_GG' => 'engelsk (Guernsey)',
        'en_GH' => 'engelsk (Ghana)',
        'en_GI' => 'engelsk (Gibraltar)',
        'en_GM' => 'engelsk (Gambia)',
        'en_GS' => 'engelsk (South Georgia og De Sydlige Sandwichøer)',
        'en_GU' => 'engelsk (Guam)',
        'en_GY' => 'engelsk (Guyana)',
        'en_HK' => 'engelsk (SAR Hongkong)',
        'en_HU' => 'engelsk (Ungarn)',
        'en_ID' => 'engelsk (Indonesien)',
        'en_IE' => 'engelsk (Irland)',
        'en_IL' => 'engelsk (Israel)',
        'en_IM' => 'engelsk (Isle of Man)',
        'en_IN' => 'engelsk (Indien)',
        'en_IO' => 'engelsk (Det Britiske Territorium i Det Indiske Ocean)',
        'en_IT' => 'engelsk (Italien)',
        'en_JE' => 'engelsk (Jersey)',
        'en_JM' => 'engelsk (Jamaica)',
        'en_JP' => 'engelsk (Japan)',
        'en_KE' => 'engelsk (Kenya)',
        'en_KI' => 'engelsk (Kiribati)',
        'en_KN' => 'engelsk (Saint Kitts og Nevis)',
        'en_KY' => 'engelsk (Caymanøerne)',
        'en_LC' => 'engelsk (Saint Lucia)',
        'en_LR' => 'engelsk (Liberia)',
        'en_LS' => 'engelsk (Lesotho)',
        'en_LT' => 'engelsk (Litauen)',
        'en_LV' => 'engelsk (Letland)',
        'en_MG' => 'engelsk (Madagaskar)',
        'en_MH' => 'engelsk (Marshalløerne)',
        'en_MO' => 'engelsk (SAR Macao)',
        'en_MP' => 'engelsk (Nordmarianerne)',
        'en_MS' => 'engelsk (Montserrat)',
        'en_MT' => 'engelsk (Malta)',
        'en_MU' => 'engelsk (Mauritius)',
        'en_MV' => 'engelsk (Maldiverne)',
        'en_MW' => 'engelsk (Malawi)',
        'en_MY' => 'engelsk (Malaysia)',
        'en_NA' => 'engelsk (Namibia)',
        'en_NF' => 'engelsk (Norfolk Island)',
        'en_NG' => 'engelsk (Nigeria)',
        'en_NL' => 'engelsk (Nederlandene)',
        'en_NO' => 'engelsk (Norge)',
        'en_NR' => 'engelsk (Nauru)',
        'en_NU' => 'engelsk (Niue)',
        'en_NZ' => 'engelsk (New Zealand)',
        'en_PG' => 'engelsk (Papua Ny Guinea)',
        'en_PH' => 'engelsk (Filippinerne)',
        'en_PK' => 'engelsk (Pakistan)',
        'en_PL' => 'engelsk (Polen)',
        'en_PN' => 'engelsk (Pitcairn)',
        'en_PR' => 'engelsk (Puerto Rico)',
        'en_PT' => 'engelsk (Portugal)',
        'en_PW' => 'engelsk (Palau)',
        'en_RO' => 'engelsk (Rumænien)',
        'en_RW' => 'engelsk (Rwanda)',
        'en_SB' => 'engelsk (Salomonøerne)',
        'en_SC' => 'engelsk (Seychellerne)',
        'en_SD' => 'engelsk (Sudan)',
        'en_SE' => 'engelsk (Sverige)',
        'en_SG' => 'engelsk (Singapore)',
        'en_SH' => 'engelsk (St. Helena)',
        'en_SI' => 'engelsk (Slovenien)',
        'en_SK' => 'engelsk (Slovakiet)',
        'en_SL' => 'engelsk (Sierra Leone)',
        'en_SS' => 'engelsk (Sydsudan)',
        'en_SX' => 'engelsk (Sint Maarten)',
        'en_SZ' => 'engelsk (Eswatini)',
        'en_TC' => 'engelsk (Turks- og Caicosøerne)',
        'en_TK' => 'engelsk (Tokelau)',
        'en_TO' => 'engelsk (Tonga)',
        'en_TT' => 'engelsk (Trinidad og Tobago)',
        'en_TV' => 'engelsk (Tuvalu)',
        'en_TZ' => 'engelsk (Tanzania)',
        'en_UA' => 'engelsk (Ukraine)',
        'en_UG' => 'engelsk (Uganda)',
        'en_UM' => 'engelsk (Amerikanske oversøiske øer)',
        'en_US' => 'engelsk (USA)',
        'en_VC' => 'engelsk (Saint Vincent og Grenadinerne)',
        'en_VG' => 'engelsk (De Britiske Jomfruøer)',
        'en_VI' => 'engelsk (De Amerikanske Jomfruøer)',
        'en_VU' => 'engelsk (Vanuatu)',
        'en_WS' => 'engelsk (Samoa)',
        'en_ZA' => 'engelsk (Sydafrika)',
        'en_ZM' => 'engelsk (Zambia)',
        'en_ZW' => 'engelsk (Zimbabwe)',
        'eo' => 'esperanto',
        'eo_001' => 'esperanto (Verden)',
        'es' => 'spansk',
        'es_419' => 'spansk (Latinamerika)',
        'es_AR' => 'spansk (Argentina)',
        'es_BO' => 'spansk (Bolivia)',
        'es_BR' => 'spansk (Brasilien)',
        'es_BZ' => 'spansk (Belize)',
        'es_CL' => 'spansk (Chile)',
        'es_CO' => 'spansk (Colombia)',
        'es_CR' => 'spansk (Costa Rica)',
        'es_CU' => 'spansk (Cuba)',
        'es_DO' => 'spansk (Den Dominikanske Republik)',
        'es_EC' => 'spansk (Ecuador)',
        'es_ES' => 'spansk (Spanien)',
        'es_GQ' => 'spansk (Ækvatorialguinea)',
        'es_GT' => 'spansk (Guatemala)',
        'es_HN' => 'spansk (Honduras)',
        'es_MX' => 'spansk (Mexico)',
        'es_NI' => 'spansk (Nicaragua)',
        'es_PA' => 'spansk (Panama)',
        'es_PE' => 'spansk (Peru)',
        'es_PH' => 'spansk (Filippinerne)',
        'es_PR' => 'spansk (Puerto Rico)',
        'es_PY' => 'spansk (Paraguay)',
        'es_SV' => 'spansk (El Salvador)',
        'es_US' => 'spansk (USA)',
        'es_UY' => 'spansk (Uruguay)',
        'es_VE' => 'spansk (Venezuela)',
        'et' => 'estisk',
        'et_EE' => 'estisk (Estland)',
        'eu' => 'baskisk',
        'eu_ES' => 'baskisk (Spanien)',
        'fa' => 'persisk',
        'fa_AF' => 'persisk (Afghanistan)',
        'fa_IR' => 'persisk (Iran)',
        'ff' => 'fulah',
        'ff_Adlm' => 'fulah (adlam)',
        'ff_Adlm_BF' => 'fulah (adlam, Burkina Faso)',
        'ff_Adlm_CM' => 'fulah (adlam, Cameroun)',
        'ff_Adlm_GH' => 'fulah (adlam, Ghana)',
        'ff_Adlm_GM' => 'fulah (adlam, Gambia)',
        'ff_Adlm_GN' => 'fulah (adlam, Guinea)',
        'ff_Adlm_GW' => 'fulah (adlam, Guinea-Bissau)',
        'ff_Adlm_LR' => 'fulah (adlam, Liberia)',
        'ff_Adlm_MR' => 'fulah (adlam, Mauretanien)',
        'ff_Adlm_NE' => 'fulah (adlam, Niger)',
        'ff_Adlm_NG' => 'fulah (adlam, Nigeria)',
        'ff_Adlm_SL' => 'fulah (adlam, Sierra Leone)',
        'ff_Adlm_SN' => 'fulah (adlam, Senegal)',
        'ff_CM' => 'fulah (Cameroun)',
        'ff_GN' => 'fulah (Guinea)',
        'ff_Latn' => 'fulah (latinsk)',
        'ff_Latn_BF' => 'fulah (latinsk, Burkina Faso)',
        'ff_Latn_CM' => 'fulah (latinsk, Cameroun)',
        'ff_Latn_GH' => 'fulah (latinsk, Ghana)',
        'ff_Latn_GM' => 'fulah (latinsk, Gambia)',
        'ff_Latn_GN' => 'fulah (latinsk, Guinea)',
        'ff_Latn_GW' => 'fulah (latinsk, Guinea-Bissau)',
        'ff_Latn_LR' => 'fulah (latinsk, Liberia)',
        'ff_Latn_MR' => 'fulah (latinsk, Mauretanien)',
        'ff_Latn_NE' => 'fulah (latinsk, Niger)',
        'ff_Latn_NG' => 'fulah (latinsk, Nigeria)',
        'ff_Latn_SL' => 'fulah (latinsk, Sierra Leone)',
        'ff_Latn_SN' => 'fulah (latinsk, Senegal)',
        'ff_MR' => 'fulah (Mauretanien)',
        'ff_SN' => 'fulah (Senegal)',
        'fi' => 'finsk',
        'fi_FI' => 'finsk (Finland)',
        'fo' => 'færøsk',
        'fo_DK' => 'færøsk (Danmark)',
        'fo_FO' => 'færøsk (Færøerne)',
        'fr' => 'fransk',
        'fr_BE' => 'fransk (Belgien)',
        'fr_BF' => 'fransk (Burkina Faso)',
        'fr_BI' => 'fransk (Burundi)',
        'fr_BJ' => 'fransk (Benin)',
        'fr_BL' => 'fransk (Saint Barthélemy)',
        'fr_CA' => 'fransk (Canada)',
        'fr_CD' => 'fransk (Congo-Kinshasa)',
        'fr_CF' => 'fransk (Den Centralafrikanske Republik)',
        'fr_CG' => 'fransk (Congo-Brazzaville)',
        'fr_CH' => 'fransk (Schweiz)',
        'fr_CI' => 'fransk (Elfenbenskysten)',
        'fr_CM' => 'fransk (Cameroun)',
        'fr_DJ' => 'fransk (Djibouti)',
        'fr_DZ' => 'fransk (Algeriet)',
        'fr_FR' => 'fransk (Frankrig)',
        'fr_GA' => 'fransk (Gabon)',
        'fr_GF' => 'fransk (Fransk Guyana)',
        'fr_GN' => 'fransk (Guinea)',
        'fr_GP' => 'fransk (Guadeloupe)',
        'fr_GQ' => 'fransk (Ækvatorialguinea)',
        'fr_HT' => 'fransk (Haiti)',
        'fr_KM' => 'fransk (Comorerne)',
        'fr_LU' => 'fransk (Luxembourg)',
        'fr_MA' => 'fransk (Marokko)',
        'fr_MC' => 'fransk (Monaco)',
        'fr_MF' => 'fransk (Saint Martin)',
        'fr_MG' => 'fransk (Madagaskar)',
        'fr_ML' => 'fransk (Mali)',
        'fr_MQ' => 'fransk (Martinique)',
        'fr_MR' => 'fransk (Mauretanien)',
        'fr_MU' => 'fransk (Mauritius)',
        'fr_NC' => 'fransk (Ny Kaledonien)',
        'fr_NE' => 'fransk (Niger)',
        'fr_PF' => 'fransk (Fransk Polynesien)',
        'fr_PM' => 'fransk (Saint Pierre og Miquelon)',
        'fr_RE' => 'fransk (Réunion)',
        'fr_RW' => 'fransk (Rwanda)',
        'fr_SC' => 'fransk (Seychellerne)',
        'fr_SN' => 'fransk (Senegal)',
        'fr_SY' => 'fransk (Syrien)',
        'fr_TD' => 'fransk (Tchad)',
        'fr_TG' => 'fransk (Togo)',
        'fr_TN' => 'fransk (Tunesien)',
        'fr_VU' => 'fransk (Vanuatu)',
        'fr_WF' => 'fransk (Wallis og Futuna)',
        'fr_YT' => 'fransk (Mayotte)',
        'fy' => 'vestfrisisk',
        'fy_NL' => 'vestfrisisk (Nederlandene)',
        'ga' => 'irsk',
        'ga_GB' => 'irsk (Storbritannien)',
        'ga_IE' => 'irsk (Irland)',
        'gd' => 'skotsk gælisk',
        'gd_GB' => 'skotsk gælisk (Storbritannien)',
        'gl' => 'galicisk',
        'gl_ES' => 'galicisk (Spanien)',
        'gu' => 'gujarati',
        'gu_IN' => 'gujarati (Indien)',
        'gv' => 'manx',
        'gv_IM' => 'manx (Isle of Man)',
        'ha' => 'hausa',
        'ha_GH' => 'hausa (Ghana)',
        'ha_NE' => 'hausa (Niger)',
        'ha_NG' => 'hausa (Nigeria)',
        'he' => 'hebraisk',
        'he_IL' => 'hebraisk (Israel)',
        'hi' => 'hindi',
        'hi_IN' => 'hindi (Indien)',
        'hi_Latn' => 'hindi (latinsk)',
        'hi_Latn_IN' => 'hindi (latinsk, Indien)',
        'hr' => 'kroatisk',
        'hr_BA' => 'kroatisk (Bosnien-Hercegovina)',
        'hr_HR' => 'kroatisk (Kroatien)',
        'hu' => 'ungarsk',
        'hu_HU' => 'ungarsk (Ungarn)',
        'hy' => 'armensk',
        'hy_AM' => 'armensk (Armenien)',
        'ia' => 'interlingua',
        'ia_001' => 'interlingua (Verden)',
        'id' => 'indonesisk',
        'id_ID' => 'indonesisk (Indonesien)',
        'ie' => 'interlingue',
        'ie_EE' => 'interlingue (Estland)',
        'ig' => 'igbo',
        'ig_NG' => 'igbo (Nigeria)',
        'ii' => 'sichuan yi',
        'ii_CN' => 'sichuan yi (Kina)',
        'is' => 'islandsk',
        'is_IS' => 'islandsk (Island)',
        'it' => 'italiensk',
        'it_CH' => 'italiensk (Schweiz)',
        'it_IT' => 'italiensk (Italien)',
        'it_SM' => 'italiensk (San Marino)',
        'it_VA' => 'italiensk (Vatikanstaten)',
        'ja' => 'japansk',
        'ja_JP' => 'japansk (Japan)',
        'jv' => 'javanesisk',
        'jv_ID' => 'javanesisk (Indonesien)',
        'ka' => 'georgisk',
        'ka_GE' => 'georgisk (Georgien)',
        'ki' => 'kikuyu',
        'ki_KE' => 'kikuyu (Kenya)',
        'kk' => 'kasakhisk',
        'kk_Arab' => 'kasakhisk (arabisk)',
        'kk_Arab_CN' => 'kasakhisk (arabisk, Kina)',
        'kk_Cyrl' => 'kasakhisk (kyrillisk)',
        'kk_Cyrl_KZ' => 'kasakhisk (kyrillisk, Kasakhstan)',
        'kk_KZ' => 'kasakhisk (Kasakhstan)',
        'kl' => 'grønlandsk',
        'kl_GL' => 'grønlandsk (Grønland)',
        'km' => 'khmer',
        'km_KH' => 'khmer (Cambodja)',
        'kn' => 'kannada',
        'kn_IN' => 'kannada (Indien)',
        'ko' => 'koreansk',
        'ko_CN' => 'koreansk (Kina)',
        'ko_KP' => 'koreansk (Nordkorea)',
        'ko_KR' => 'koreansk (Sydkorea)',
        'ks' => 'kashmiri',
        'ks_Arab' => 'kashmiri (arabisk)',
        'ks_Arab_IN' => 'kashmiri (arabisk, Indien)',
        'ks_Deva' => 'kashmiri (devanagari)',
        'ks_Deva_IN' => 'kashmiri (devanagari, Indien)',
        'ks_IN' => 'kashmiri (Indien)',
        'ku' => 'kurdisk',
        'ku_Latn' => 'kurdisk (latinsk)',
        'ku_Latn_IQ' => 'kurdisk (latinsk, Irak)',
        'ku_Latn_SY' => 'kurdisk (latinsk, Syrien)',
        'ku_Latn_TR' => 'kurdisk (latinsk, Tyrkiet)',
        'ku_SY' => 'kurdisk (Syrien)',
        'ku_TR' => 'kurdisk (Tyrkiet)',
        'kw' => 'cornisk',
        'kw_GB' => 'cornisk (Storbritannien)',
        'ky' => 'kirgisisk',
        'ky_KG' => 'kirgisisk (Kirgisistan)',
        'lb' => 'luxembourgsk',
        'lb_LU' => 'luxembourgsk (Luxembourg)',
        'lg' => 'ganda',
        'lg_UG' => 'ganda (Uganda)',
        'ln' => 'lingala',
        'ln_AO' => 'lingala (Angola)',
        'ln_CD' => 'lingala (Congo-Kinshasa)',
        'ln_CF' => 'lingala (Den Centralafrikanske Republik)',
        'ln_CG' => 'lingala (Congo-Brazzaville)',
        'lo' => 'lao',
        'lo_LA' => 'lao (Laos)',
        'lt' => 'litauisk',
        'lt_LT' => 'litauisk (Litauen)',
        'lu' => 'luba-Katanga',
        'lu_CD' => 'luba-Katanga (Congo-Kinshasa)',
        'lv' => 'lettisk',
        'lv_LV' => 'lettisk (Letland)',
        'mg' => 'malagassisk',
        'mg_MG' => 'malagassisk (Madagaskar)',
        'mi' => 'maori',
        'mi_NZ' => 'maori (New Zealand)',
        'mk' => 'makedonsk',
        'mk_MK' => 'makedonsk (Nordmakedonien)',
        'ml' => 'malayalam',
        'ml_IN' => 'malayalam (Indien)',
        'mn' => 'mongolsk',
        'mn_MN' => 'mongolsk (Mongoliet)',
        'mr' => 'marathi',
        'mr_IN' => 'marathi (Indien)',
        'ms' => 'malajisk',
        'ms_BN' => 'malajisk (Brunei)',
        'ms_ID' => 'malajisk (Indonesien)',
        'ms_MY' => 'malajisk (Malaysia)',
        'ms_SG' => 'malajisk (Singapore)',
        'mt' => 'maltesisk',
        'mt_MT' => 'maltesisk (Malta)',
        'my' => 'burmesisk',
        'my_MM' => 'burmesisk (Myanmar [Burma])',
        'nb' => 'bokmål',
        'nb_NO' => 'bokmål (Norge)',
        'nb_SJ' => 'bokmål (Svalbard og Jan Mayen)',
        'nd' => 'nordndebele',
        'nd_ZW' => 'nordndebele (Zimbabwe)',
        'ne' => 'nepalesisk',
        'ne_IN' => 'nepalesisk (Indien)',
        'ne_NP' => 'nepalesisk (Nepal)',
        'nl' => 'nederlandsk',
        'nl_AW' => 'nederlandsk (Aruba)',
        'nl_BE' => 'nederlandsk (Belgien)',
        'nl_BQ' => 'nederlandsk (De tidligere Nederlandske Antiller)',
        'nl_CW' => 'nederlandsk (Curaçao)',
        'nl_NL' => 'nederlandsk (Nederlandene)',
        'nl_SR' => 'nederlandsk (Surinam)',
        'nl_SX' => 'nederlandsk (Sint Maarten)',
        'nn' => 'nynorsk',
        'nn_NO' => 'nynorsk (Norge)',
        'no' => 'norsk',
        'no_NO' => 'norsk (Norge)',
        'oc' => 'occitansk',
        'oc_ES' => 'occitansk (Spanien)',
        'oc_FR' => 'occitansk (Frankrig)',
        'om' => 'oromo',
        'om_ET' => 'oromo (Etiopien)',
        'om_KE' => 'oromo (Kenya)',
        'or' => 'oriya',
        'or_IN' => 'oriya (Indien)',
        'os' => 'ossetisk',
        'os_GE' => 'ossetisk (Georgien)',
        'os_RU' => 'ossetisk (Rusland)',
        'pa' => 'punjabi',
        'pa_Arab' => 'punjabi (arabisk)',
        'pa_Arab_PK' => 'punjabi (arabisk, Pakistan)',
        'pa_Guru' => 'punjabi (gurmukhi)',
        'pa_Guru_IN' => 'punjabi (gurmukhi, Indien)',
        'pa_IN' => 'punjabi (Indien)',
        'pa_PK' => 'punjabi (Pakistan)',
        'pl' => 'polsk',
        'pl_PL' => 'polsk (Polen)',
        'ps' => 'pashto',
        'ps_AF' => 'pashto (Afghanistan)',
        'ps_PK' => 'pashto (Pakistan)',
        'pt' => 'portugisisk',
        'pt_AO' => 'portugisisk (Angola)',
        'pt_BR' => 'portugisisk (Brasilien)',
        'pt_CH' => 'portugisisk (Schweiz)',
        'pt_CV' => 'portugisisk (Kap Verde)',
        'pt_GQ' => 'portugisisk (Ækvatorialguinea)',
        'pt_GW' => 'portugisisk (Guinea-Bissau)',
        'pt_LU' => 'portugisisk (Luxembourg)',
        'pt_MO' => 'portugisisk (SAR Macao)',
        'pt_MZ' => 'portugisisk (Mozambique)',
        'pt_PT' => 'portugisisk (Portugal)',
        'pt_ST' => 'portugisisk (São Tomé og Príncipe)',
        'pt_TL' => 'portugisisk (Timor-Leste)',
        'qu' => 'quechua',
        'qu_BO' => 'quechua (Bolivia)',
        'qu_EC' => 'quechua (Ecuador)',
        'qu_PE' => 'quechua (Peru)',
        'rm' => 'rætoromansk',
        'rm_CH' => 'rætoromansk (Schweiz)',
        'rn' => 'rundi',
        'rn_BI' => 'rundi (Burundi)',
        'ro' => 'rumænsk',
        'ro_MD' => 'rumænsk (Moldova)',
        'ro_RO' => 'rumænsk (Rumænien)',
        'ru' => 'russisk',
        'ru_BY' => 'russisk (Belarus)',
        'ru_KG' => 'russisk (Kirgisistan)',
        'ru_KZ' => 'russisk (Kasakhstan)',
        'ru_MD' => 'russisk (Moldova)',
        'ru_RU' => 'russisk (Rusland)',
        'ru_UA' => 'russisk (Ukraine)',
        'rw' => 'kinyarwanda',
        'rw_RW' => 'kinyarwanda (Rwanda)',
        'sa' => 'sanskrit',
        'sa_IN' => 'sanskrit (Indien)',
        'sc' => 'sardinsk',
        'sc_IT' => 'sardinsk (Italien)',
        'sd' => 'sindhi',
        'sd_Arab' => 'sindhi (arabisk)',
        'sd_Arab_PK' => 'sindhi (arabisk, Pakistan)',
        'sd_Deva' => 'sindhi (devanagari)',
        'sd_Deva_IN' => 'sindhi (devanagari, Indien)',
        'sd_IN' => 'sindhi (Indien)',
        'sd_PK' => 'sindhi (Pakistan)',
        'se' => 'nordsamisk',
        'se_FI' => 'nordsamisk (Finland)',
        'se_NO' => 'nordsamisk (Norge)',
        'se_SE' => 'nordsamisk (Sverige)',
        'sg' => 'sango',
        'sg_CF' => 'sango (Den Centralafrikanske Republik)',
        'sh' => 'serbokroatisk',
        'sh_BA' => 'serbokroatisk (Bosnien-Hercegovina)',
        'si' => 'singalesisk',
        'si_LK' => 'singalesisk (Sri Lanka)',
        'sk' => 'slovakisk',
        'sk_SK' => 'slovakisk (Slovakiet)',
        'sl' => 'slovensk',
        'sl_SI' => 'slovensk (Slovenien)',
        'sn' => 'shona',
        'sn_ZW' => 'shona (Zimbabwe)',
        'so' => 'somali',
        'so_DJ' => 'somali (Djibouti)',
        'so_ET' => 'somali (Etiopien)',
        'so_KE' => 'somali (Kenya)',
        'so_SO' => 'somali (Somalia)',
        'sq' => 'albansk',
        'sq_AL' => 'albansk (Albanien)',
        'sq_MK' => 'albansk (Nordmakedonien)',
        'sr' => 'serbisk',
        'sr_BA' => 'serbisk (Bosnien-Hercegovina)',
        'sr_Cyrl' => 'serbisk (kyrillisk)',
        'sr_Cyrl_BA' => 'serbisk (kyrillisk, Bosnien-Hercegovina)',
        'sr_Cyrl_ME' => 'serbisk (kyrillisk, Montenegro)',
        'sr_Cyrl_RS' => 'serbisk (kyrillisk, Serbien)',
        'sr_Latn' => 'serbisk (latinsk)',
        'sr_Latn_BA' => 'serbisk (latinsk, Bosnien-Hercegovina)',
        'sr_Latn_ME' => 'serbisk (latinsk, Montenegro)',
        'sr_Latn_RS' => 'serbisk (latinsk, Serbien)',
        'sr_ME' => 'serbisk (Montenegro)',
        'sr_RS' => 'serbisk (Serbien)',
        'st' => 'sydsotho',
        'st_LS' => 'sydsotho (Lesotho)',
        'st_ZA' => 'sydsotho (Sydafrika)',
        'su' => 'sundanesisk',
        'su_ID' => 'sundanesisk (Indonesien)',
        'su_Latn' => 'sundanesisk (latinsk)',
        'su_Latn_ID' => 'sundanesisk (latinsk, Indonesien)',
        'sv' => 'svensk',
        'sv_AX' => 'svensk (Åland)',
        'sv_FI' => 'svensk (Finland)',
        'sv_SE' => 'svensk (Sverige)',
        'sw' => 'swahili',
        'sw_CD' => 'swahili (Congo-Kinshasa)',
        'sw_KE' => 'swahili (Kenya)',
        'sw_TZ' => 'swahili (Tanzania)',
        'sw_UG' => 'swahili (Uganda)',
        'ta' => 'tamil',
        'ta_IN' => 'tamil (Indien)',
        'ta_LK' => 'tamil (Sri Lanka)',
        'ta_MY' => 'tamil (Malaysia)',
        'ta_SG' => 'tamil (Singapore)',
        'te' => 'telugu',
        'te_IN' => 'telugu (Indien)',
        'tg' => 'tadsjikisk',
        'tg_TJ' => 'tadsjikisk (Tadsjikistan)',
        'th' => 'thai',
        'th_TH' => 'thai (Thailand)',
        'ti' => 'tigrinya',
        'ti_ER' => 'tigrinya (Eritrea)',
        'ti_ET' => 'tigrinya (Etiopien)',
        'tk' => 'turkmensk',
        'tk_TM' => 'turkmensk (Turkmenistan)',
        'tl' => 'tagalog',
        'tl_PH' => 'tagalog (Filippinerne)',
        'tn' => 'tswana',
        'tn_BW' => 'tswana (Botswana)',
        'tn_ZA' => 'tswana (Sydafrika)',
        'to' => 'tongansk',
        'to_TO' => 'tongansk (Tonga)',
        'tr' => 'tyrkisk',
        'tr_CY' => 'tyrkisk (Cypern)',
        'tr_TR' => 'tyrkisk (Tyrkiet)',
        'tt' => 'tatarisk',
        'tt_RU' => 'tatarisk (Rusland)',
        'ug' => 'uygurisk',
        'ug_CN' => 'uygurisk (Kina)',
        'uk' => 'ukrainsk',
        'uk_UA' => 'ukrainsk (Ukraine)',
        'ur' => 'urdu',
        'ur_IN' => 'urdu (Indien)',
        'ur_PK' => 'urdu (Pakistan)',
        'uz' => 'usbekisk',
        'uz_AF' => 'usbekisk (Afghanistan)',
        'uz_Arab' => 'usbekisk (arabisk)',
        'uz_Arab_AF' => 'usbekisk (arabisk, Afghanistan)',
        'uz_Cyrl' => 'usbekisk (kyrillisk)',
        'uz_Cyrl_UZ' => 'usbekisk (kyrillisk, Usbekistan)',
        'uz_Latn' => 'usbekisk (latinsk)',
        'uz_Latn_UZ' => 'usbekisk (latinsk, Usbekistan)',
        'uz_UZ' => 'usbekisk (Usbekistan)',
        'vi' => 'vietnamesisk',
        'vi_VN' => 'vietnamesisk (Vietnam)',
        'wo' => 'wolof',
        'wo_SN' => 'wolof (Senegal)',
        'xh' => 'xhosa',
        'xh_ZA' => 'xhosa (Sydafrika)',
        'yi' => 'jiddisch',
        'yi_UA' => 'jiddisch (Ukraine)',
        'yo' => 'yoruba',
        'yo_BJ' => 'yoruba (Benin)',
        'yo_NG' => 'yoruba (Nigeria)',
        'za' => 'zhuang',
        'za_CN' => 'zhuang (Kina)',
        'zh' => 'kinesisk',
        'zh_CN' => 'kinesisk (Kina)',
        'zh_HK' => 'kinesisk (SAR Hongkong)',
        'zh_Hans' => 'kinesisk (forenklet)',
        'zh_Hans_CN' => 'kinesisk (forenklet, Kina)',
        'zh_Hans_HK' => 'kinesisk (forenklet, SAR Hongkong)',
        'zh_Hans_MO' => 'kinesisk (forenklet, SAR Macao)',
        'zh_Hans_MY' => 'kinesisk (forenklet, Malaysia)',
        'zh_Hans_SG' => 'kinesisk (forenklet, Singapore)',
        'zh_Hant' => 'kinesisk (traditionelt)',
        'zh_Hant_HK' => 'kinesisk (traditionelt, SAR Hongkong)',
        'zh_Hant_MO' => 'kinesisk (traditionelt, SAR Macao)',
        'zh_Hant_MY' => 'kinesisk (traditionelt, Malaysia)',
        'zh_Hant_TW' => 'kinesisk (traditionelt, Taiwan)',
        'zh_MO' => 'kinesisk (SAR Macao)',
        'zh_SG' => 'kinesisk (Singapore)',
        'zh_TW' => 'kinesisk (Taiwan)',
        'zu' => 'zulu',
        'zu_ZA' => 'zulu (Sydafrika)',
    ],
];
