<?php

declare(strict_types=1);

namespace SimpleSAML\XMLSchema\Type;

/**
 * @package simplesaml/xml-common
 */
class IDRefValue extends NCNameValue
{
    public const string SCHEMA_TYPE = 'IDREF';
}
