<?php

declare(strict_types=1);

namespace SimpleSAML\XPath\Exception;

/**
 * Class AxisNotAllowedException
 *
 * This exception is thrown when a disallowed axis is used in an XPath-expression
 *
 * @package simplesamlphp/xml-common
 */
class AxisNotAllowedException extends RuntimeException
{
}
