<?php

declare(strict_types=1);

namespace SimpleSAML\XPath\Exception;

/**
 * Class FunctionNotAllowedException
 *
 * This exception is thrown when a disallowed function is used in an XPath-expression
 *
 * @package simplesamlphp/xml-common
 */
class FunctionNotAllowedException extends RuntimeException
{
}
