<?php

declare(strict_types=1);

namespace SimpleSAML\XMLSchema\XML\Interface;

/**
 * interface class to be implemented by all the classes that represent an element from the xs:nestedParticle group
 *
 * @package simplesamlphp/xml-common
 */
interface NestedParticleInterface extends ParticleInterface
{
}
