<?php

declare(strict_types=1);

namespace SimpleSAML\XMLSchema;

/**
 * Various XML constants.
 *
 * @package simplesamlphp/xml-common
 */
class Constants
{
    /**
     * The namespace for XML schema.
     */
    public const string NS_XS = 'http://www.w3.org/2001/XMLSchema';

    /**
     * The namespace for XML schema instance.
     */
    public const string NS_XSI = 'http://www.w3.org/2001/XMLSchema-instance';
}
