<?php return array(
    'root' => array(
        'name' => 'simplesamlphp/simplesamlphp',
        'pretty_version' => 'v2.5.3.1',
        'version' => '2.5.3.1',
        'reference' => null,
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'gettext/gettext' => array(
            'pretty_version' => 'v5.7.3',
            'version' => '5.7.3.0',
            'reference' => '95820f020e4f2f05e0bbaa5603e4c6ec3edc50f1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gettext/gettext',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gettext/languages' => array(
            'pretty_version' => '2.12.2',
            'version' => '2.12.2.0',
            'reference' => '079d6f4842cbcbf5673a70d8e93169a684e7aadd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gettext/languages',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gettext/php-scanner' => array(
            'pretty_version' => 'v2.0.1',
            'version' => '2.0.1.0',
            'reference' => '43f53b6e0d5f9ac5ea165d8202c18267d6bd5dfd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gettext/php-scanner',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gettext/translator' => array(
            'pretty_version' => 'v1.2.3',
            'version' => '1.2.3.0',
            'reference' => 'e6d96e03d8c1afb3ef497799921ad0adfc9f4f96',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gettext/translator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.13.0',
            'version' => '2.13.0.0',
            'reference' => 'dad89620b7a6edb60c15858442eb2e408b45d8f4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nikic/php-parser' => array(
            'pretty_version' => 'v5.8.0',
            'version' => '5.8.0.0',
            'reference' => '044a6a392ff8ad0d61f14370a5fbbd0a0107152f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nikic/php-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nyholm/psr7' => array(
            'pretty_version' => '1.8.2',
            'version' => '1.8.2.0',
            'reference' => 'a71f2b11690f4b24d099d6b16690a90ae14fc6f3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nyholm/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/async-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-http/client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-http/message-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'phpmailer/phpmailer' => array(
            'pretty_version' => 'v7.1.1',
            'version' => '7.1.1.0',
            'reference' => '1bc1716a507a65e039d4ac9d9adebbbd0d346e15',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpmailer/phpmailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/cache' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => 'aa5030cfa5405eccfdcb1083ce040c2cb8d253bf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/cache-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.0|3.0',
            ),
        ),
        'psr/clock' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'e41a24703d4560fd0acb709162f73b8adfc3aa0d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/clock',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'c71ecc56dfe541dbd90c5360474fbc405f8d5963',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.1|2.0',
            ),
        ),
        'psr/event-dispatcher' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'dbefd12671e8a14ec7f180cab83036ed26714bb0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '2.0',
            'version' => '2.0.0.0',
            'reference' => '402d35bcb92c70c026d1a6a9883f06b2ead23d71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0|3.0',
            ),
        ),
        'psr/simple-cache-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0|3.0',
            ),
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'robrichards/xmlseclibs' => array(
            'pretty_version' => '3.1.5',
            'version' => '3.1.5.0',
            'reference' => '03062be78178cbb5e8f605cd255dc32a14981f92',
            'type' => 'library',
            'install_path' => __DIR__ . '/../robrichards/xmlseclibs',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/assert' => array(
            'pretty_version' => 'v2.0.2',
            'version' => '2.0.2.0',
            'reference' => '9e08dc5525a2259d75e6f6d0ae6754a7b4c13470',
            'type' => 'library',
            'install_path' => __DIR__ . '/../simplesamlphp/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/composer-module-installer' => array(
            'pretty_version' => 'v1.7.1',
            'version' => '1.7.1.0',
            'reference' => '348f02344e851655587474e389fe4234c9399404',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../simplesamlphp/composer-module-installer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/composer-xmlprovider-installer' => array(
            'pretty_version' => 'v1.3.1',
            'version' => '1.3.1.0',
            'reference' => '7eed3afcd294fd754f012f177c266c5ca71c3c58',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../simplesamlphp/composer-xmlprovider-installer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/saml2' => array(
            'pretty_version' => 'v6.2.4',
            'version' => '6.2.4.0',
            'reference' => '9a21edb96de1f3ece3b40ccbe3b7de761917a06f',
            'type' => 'simplesamlphp-xmlprovider',
            'install_path' => __DIR__ . '/../simplesamlphp/saml2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/saml2-legacy' => array(
            'pretty_version' => 'v4.20.3',
            'version' => '4.20.3.0',
            'reference' => 'b43d5d10ea1180f478ddcd677ff13bb324cfa866',
            'type' => 'library',
            'install_path' => __DIR__ . '/../simplesamlphp/saml2-legacy',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/simplesamlphp' => array(
            'pretty_version' => 'v2.5.3.1',
            'version' => '2.5.3.1',
            'reference' => null,
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/simplesamlphp-assets-base' => array(
            'pretty_version' => 'v2.5.15',
            'version' => '2.5.15.0',
            'reference' => '6c65cfe90415e2922a9a2ace2bbb72761525fa35',
            'type' => 'simplesamlphp-module',
            'install_path' => __DIR__ . '/../../public/assets/base',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/xml-common' => array(
            'pretty_version' => 'v2.8.2',
            'version' => '2.8.2.0',
            'reference' => 'c9b66277ac7fb89187fcb500c144bb01d5d18676',
            'type' => 'simplesamlphp-xmlprovider',
            'install_path' => __DIR__ . '/../simplesamlphp/xml-common',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/xml-security' => array(
            'pretty_version' => 'v2.3.1',
            'version' => '2.3.1.0',
            'reference' => 'ded07df15a53ee87a179e8bff79870ffea31e716',
            'type' => 'simplesamlphp-xmlprovider',
            'install_path' => __DIR__ . '/../simplesamlphp/xml-security',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simplesamlphp/xml-soap' => array(
            'pretty_version' => 'v2.3.0',
            'version' => '2.3.0.0',
            'reference' => '014a0846d3b3a1a29f8d39c5bf75a827de216501',
            'type' => 'simplesamlphp-xmlprovider',
            'install_path' => __DIR__ . '/../simplesamlphp/xml-soap',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/cache' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => '23a2c5298ca72c4d26b04611ed966c86762ffd49',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/cache-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => '9789738bc19af1106dc54d6afba9a0b467516cf2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/cache-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/cache-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.1|2.0|3.0',
            ),
        ),
        'symfony/config' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'e896f5e874e6983d0a098f554ca82a08a924c298',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/config',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/console' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'f4c69c9aed03abf933b294257d618bdd9b30a06d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/console',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/dependency-injection' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => '7f59a843c1fbcceafecdf6863d3e38ea6ecd5061',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/dependency-injection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => 'f3202fa1b5097b0af062dc978b32ecf63404e31d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/error-handler' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => 'd49f6a19f326db41ae7103bdc38e3eb35a791261',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/error-handler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => '336e7f3b9e95aba04f93ea9143920c2186abfbb9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => 'c7de7a00ffb67842132da02ea92988a39ccd9f4e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.0|3.0',
            ),
        ),
        'symfony/expression-language' => array(
            'pretty_version' => 'v7.4.14',
            'version' => '7.4.14.0',
            'reference' => 'bd5763f92959201816ecc31defdf352d2ea473be',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/expression-language',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/filesystem' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => 'ff16a16bf87fdf264638b8f6995b3515975e3c79',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/filesystem',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/finder' => array(
            'pretty_version' => 'v7.4.14',
            'version' => '7.4.14.0',
            'reference' => '13b38720174286f55d1761152b575a8d1436fc25',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/finder',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/framework-bundle' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'fa9c81699911e0eb9986306d5a5694f470f5aaff',
            'type' => 'symfony-bundle',
            'install_path' => __DIR__ . '/../symfony/framework-bundle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'c513ed0ba5d1784a6b55fc84190dbe4451b12f41',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => '41fc42d276aeff21192465331ebbab7d83a743c0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-client-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '3.0',
            ),
        ),
        'symfony/http-foundation' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'b676451bb638e99a7d34d8a2be90406822e301eb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-foundation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-kernel' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'f5e728670fa2218ae8be8ea91f2b44b7d6e5304c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-kernel',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/intl' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'a8ff4606c5f5c7ec166d565cea1fb1864c2f9850',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/intl',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/password-hasher' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => '18a7d92126c95962f7efbcc9e421ba710a366847',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/password-hasher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => '141046a8f9477948ff284fa65be2095baafb94f2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-grapheme' => array(
            'pretty_version' => 'v1.41.0',
            'version' => '1.41.0.0',
            'reference' => 'bb899c1db0aa8127dc3afe8cda4a67eb24915f8d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-grapheme',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-icu' => array(
            'pretty_version' => 'v1.38.0',
            'version' => '1.38.0.0',
            'reference' => '445c90e341fccda10311019cf82ff73bb7343945',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-icu',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.38.0',
            'version' => '1.38.0.0',
            'reference' => '2d446c214bdbe5b71bde5011b060a05fece3ae6b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.38.2',
            'version' => '1.38.2.0',
            'reference' => 'd3d318bad5e7a1bfbd026009c8bfb8d8f99ae6b6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php80' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => 'dfb55726c3a76ea3b6459fcfda1ec2d80a682411',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php80',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php83' => array(
            'pretty_version' => 'v1.41.0',
            'version' => '1.41.0.0',
            'reference' => '5ea99087fb99c273a9b9236ed4c31e78b16103c6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php83',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php85' => array(
            'pretty_version' => 'v1.41.0',
            'version' => '1.41.0.0',
            'reference' => '255fab485aaa1006ed411040c42aecd7b5302d7a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php85',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/routing' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => '80c0a93d3f8e7499f716204a1fb38ead942a7a2b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/routing',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/service-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => 'c0a284bab1ed8aa0417e3d69250ab437739563a0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/service-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/service-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.1|2.0|3.0',
            ),
        ),
        'symfony/string' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => 'e394af32256bf9e7bf80849d95e589167c10097b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => '501e0ff4553c744209ca1a68790d8a4541563710',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-contracts' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => 'ccb206b98faccc511ebae8e5fad50f2dc0b30621',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.3|3.0',
            ),
        ),
        'symfony/twig-bridge' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'adc759a2a142f25200a440acb3d12465558067a5',
            'type' => 'symfony-bridge',
            'install_path' => __DIR__ . '/../symfony/twig-bridge',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/var-dumper' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => '04ba4add636a95ff437af3a5a9499bb1d6c6d4bd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/var-dumper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/var-exporter' => array(
            'pretty_version' => 'v7.4.16',
            'version' => '7.4.16.0',
            'reference' => 'ca31404415670aa3834809005b529df1b84f0790',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/var-exporter',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v7.4.15',
            'version' => '7.4.15.0',
            'reference' => 'e101850ded5d2c0d44bf32abb8996404afec2dec',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'twig/intl-extra' => array(
            'pretty_version' => 'v3.26.0',
            'version' => '3.26.0.0',
            'reference' => '98f5ad5bff13230fcd2d834d9e79b50adf3ccda9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../twig/intl-extra',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'twig/twig' => array(
            'pretty_version' => 'v3.28.0',
            'version' => '3.28.0.0',
            'reference' => '597c12ed286fb9d1701a36684ce6e0cbe28ebc8b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../twig/twig',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '2.4.1',
            'version' => '2.4.1.0',
            'reference' => '2ccb7c2e821038c03a3e6e1700c570c158c55f70',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
