<?php

declare(strict_types=1);

return [
    'http://www.w3.org/2001/XMLSchema' => [
        'all' => '\SimpleSAML\XMLSchema\XML\All',
        'annotation' => '\SimpleSAML\XMLSchema\XML\Annotation',
        'any' => '\SimpleSAML\XMLSchema\XML\Any',
        'anyAttribute' => '\SimpleSAML\XMLSchema\XML\AnyAttribute',
        'appinfo' => '\SimpleSAML\XMLSchema\XML\Appinfo',
        'attribute' => '\SimpleSAML\XMLSchema\XML\TopLevelAttribute',
        'attributeGroup' => '\SimpleSAML\XMLSchema\XML\NamedAttributeGroup',
        'choice' => '\SimpleSAML\XMLSchema\XML\Choice',
        'complexContent' => '\SimpleSAML\XMLSchema\XML\ComplexContent',
        'complexType' => '\SimpleSAML\XMLSchema\XML\TopLevelComplexType',
        'documentation' => '\SimpleSAML\XMLSchema\XML\Documentation',
        'element' => '\SimpleSAML\XMLSchema\XML\TopLevelElement',
        'enumeration' => '\SimpleSAML\XMLSchema\XML\Enumeration',
        'field' => '\SimpleSAML\XMLSchema\XML\Field',
        'fractionDigits' => '\SimpleSAML\XMLSchema\XML\FractionDigits',
        'group' => '\SimpleSAML\XMLSchema\XML\NamedGroup',
        'import' => '\SimpleSAML\XMLSchema\XML\Import',
        'include' => '\SimpleSAML\XMLSchema\XML\XsInclude',
        'key' => '\SimpleSAML\XMLSchema\XML\Key',
        'keyref' => '\SimpleSAML\XMLSchema\XML\Keyref',
        'length' => '\SimpleSAML\XMLSchema\XML\Length',
        'list' => '\SimpleSAML\XMLSchema\XML\XsList',
        'maxExclusive' => '\SimpleSAML\XMLSchema\XML\MaxExclusive',
        'maxInclusive' => '\SimpleSAML\XMLSchema\XML\MaxInclusive',
        'maxLength' => '\SimpleSAML\XMLSchema\XML\MaxLength',
        'minExclusive' => '\SimpleSAML\XMLSchema\XML\MinExclusive',
        'minInclusive' => '\SimpleSAML\XMLSchema\XML\MinInclusive',
        'minLength' => '\SimpleSAML\XMLSchema\XML\MinLength',
        'notation' => '\SimpleSAML\XMLSchema\XML\Notation',
        'pattern' => '\SimpleSAML\XMLSchema\XML\Pattern',
        'redefine' => '\SimpleSAML\XMLSchema\XML\Redefine',
        'restriction' => '\SimpleSAML\XMLSchema\XML\Restriction',
        'schema' => '\SimpleSAML\XMLSchema\XML\Schema',
        'selector' => '\SimpleSAML\XMLSchema\XML\Selector',
        'sequence' => '\SimpleSAML\XMLSchema\XML\Sequence',
        'simpleContent' => '\SimpleSAML\XMLSchema\XML\SimpleContent',
        'simpleType' => '\SimpleSAML\XMLSchema\XML\TopLevelSimpleType',
        'totalDigits' => '\SimpleSAML\XMLSchema\XML\TotalDigits',
        'union' => '\SimpleSAML\XMLSchema\XML\Union',
        'unique' => '\SimpleSAML\XMLSchema\XML\Unique',
        'whiteSpace' => '\SimpleSAML\XMLSchema\XML\WhiteSpace',
    ],
];
