<?php

declare(strict_types=1);

namespace SimpleSAML\XMLSchema\Type\Schema;

/**
 * @package simplesaml/xml-common
 */
class MaxOccursValue extends AbstractAllNNIValue
{
    public const string SCHEMA_TYPE = 'maxOccurs';
}
