<?php

declare(strict_types=1);

namespace SimpleSAML\SOAP11\Exception;

use RuntimeException as BuiltinRuntimeException;

/**
 * Named exception
 */
class RuntimeException extends BuiltinRuntimeException
{
}
