<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Attribute' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/Attribute.php',
    'Collator' => $vendorDir . '/symfony/polyfill-intl-icu/Resources/stubs/Collator.php',
    'Composer\\InstalledVersions' => $vendorDir . '/composer/InstalledVersions.php',
    'DateError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateError.php',
    'DateException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateException.php',
    'DateInvalidOperationException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidOperationException.php',
    'DateInvalidTimeZoneException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidTimeZoneException.php',
    'DateMalformedIntervalStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedIntervalStringException.php',
    'DateMalformedPeriodStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedPeriodStringException.php',
    'DateMalformedStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedStringException.php',
    'DateObjectError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateObjectError.php',
    'DateRangeError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateRangeError.php',
    'DelayedTargetValidation' => $vendorDir . '/symfony/polyfill-php85/Resources/stubs/DelayedTargetValidation.php',
    'Filter\\FilterException' => $vendorDir . '/symfony/polyfill-php85/Resources/stubs/Filter/FilterException.php',
    'Filter\\FilterFailedException' => $vendorDir . '/symfony/polyfill-php85/Resources/stubs/Filter/FilterFailedException.php',
    'Gettext\\Comments' => $vendorDir . '/gettext/gettext/src/Comments.php',
    'Gettext\\Flags' => $vendorDir . '/gettext/gettext/src/Flags.php',
    'Gettext\\Formatter' => $vendorDir . '/gettext/translator/src/Formatter.php',
    'Gettext\\FormatterInterface' => $vendorDir . '/gettext/translator/src/FormatterInterface.php',
    'Gettext\\Generator\\ArrayGenerator' => $vendorDir . '/gettext/translator/src/Generator/ArrayGenerator.php',
    'Gettext\\Generator\\Generator' => $vendorDir . '/gettext/gettext/src/Generator/Generator.php',
    'Gettext\\Generator\\GeneratorInterface' => $vendorDir . '/gettext/gettext/src/Generator/GeneratorInterface.php',
    'Gettext\\Generator\\MoGenerator' => $vendorDir . '/gettext/gettext/src/Generator/MoGenerator.php',
    'Gettext\\Generator\\PoGenerator' => $vendorDir . '/gettext/gettext/src/Generator/PoGenerator.php',
    'Gettext\\GettextTranslator' => $vendorDir . '/gettext/translator/src/GettextTranslator.php',
    'Gettext\\Headers' => $vendorDir . '/gettext/gettext/src/Headers.php',
    'Gettext\\Languages\\Category' => $vendorDir . '/gettext/languages/src/Category.php',
    'Gettext\\Languages\\CldrData' => $vendorDir . '/gettext/languages/src/CldrData.php',
    'Gettext\\Languages\\Exporter\\Exporter' => $vendorDir . '/gettext/languages/src/Exporter/Exporter.php',
    'Gettext\\Languages\\Exporter\\Html' => $vendorDir . '/gettext/languages/src/Exporter/Html.php',
    'Gettext\\Languages\\Exporter\\Json' => $vendorDir . '/gettext/languages/src/Exporter/Json.php',
    'Gettext\\Languages\\Exporter\\Php' => $vendorDir . '/gettext/languages/src/Exporter/Php.php',
    'Gettext\\Languages\\Exporter\\Po' => $vendorDir . '/gettext/languages/src/Exporter/Po.php',
    'Gettext\\Languages\\Exporter\\Prettyjson' => $vendorDir . '/gettext/languages/src/Exporter/Prettyjson.php',
    'Gettext\\Languages\\Exporter\\Ruby' => $vendorDir . '/gettext/languages/src/Exporter/Ruby.php',
    'Gettext\\Languages\\Exporter\\Xml' => $vendorDir . '/gettext/languages/src/Exporter/Xml.php',
    'Gettext\\Languages\\FormulaConverter' => $vendorDir . '/gettext/languages/src/FormulaConverter.php',
    'Gettext\\Languages\\Language' => $vendorDir . '/gettext/languages/src/Language.php',
    'Gettext\\Loader\\ArrayLoader' => $vendorDir . '/gettext/translator/src/Loader/ArrayLoader.php',
    'Gettext\\Loader\\Loader' => $vendorDir . '/gettext/gettext/src/Loader/Loader.php',
    'Gettext\\Loader\\LoaderInterface' => $vendorDir . '/gettext/gettext/src/Loader/LoaderInterface.php',
    'Gettext\\Loader\\MoLoader' => $vendorDir . '/gettext/gettext/src/Loader/MoLoader.php',
    'Gettext\\Loader\\PoLoader' => $vendorDir . '/gettext/gettext/src/Loader/PoLoader.php',
    'Gettext\\Loader\\StrictPoLoader' => $vendorDir . '/gettext/gettext/src/Loader/StrictPoLoader.php',
    'Gettext\\Merge' => $vendorDir . '/gettext/gettext/src/Merge.php',
    'Gettext\\References' => $vendorDir . '/gettext/gettext/src/References.php',
    'Gettext\\Scanner\\CodeScanner' => $vendorDir . '/gettext/gettext/src/Scanner/CodeScanner.php',
    'Gettext\\Scanner\\FunctionsHandlersTrait' => $vendorDir . '/gettext/gettext/src/Scanner/FunctionsHandlersTrait.php',
    'Gettext\\Scanner\\FunctionsScannerInterface' => $vendorDir . '/gettext/gettext/src/Scanner/FunctionsScannerInterface.php',
    'Gettext\\Scanner\\ParsedFunction' => $vendorDir . '/gettext/gettext/src/Scanner/ParsedFunction.php',
    'Gettext\\Scanner\\PhpFunctionsScanner' => $vendorDir . '/gettext/php-scanner/src/PhpFunctionsScanner.php',
    'Gettext\\Scanner\\PhpNodeVisitor' => $vendorDir . '/gettext/php-scanner/src/PhpNodeVisitor.php',
    'Gettext\\Scanner\\PhpScanner' => $vendorDir . '/gettext/php-scanner/src/PhpScanner.php',
    'Gettext\\Scanner\\Scanner' => $vendorDir . '/gettext/gettext/src/Scanner/Scanner.php',
    'Gettext\\Scanner\\ScannerInterface' => $vendorDir . '/gettext/gettext/src/Scanner/ScannerInterface.php',
    'Gettext\\Translation' => $vendorDir . '/gettext/gettext/src/Translation.php',
    'Gettext\\Translations' => $vendorDir . '/gettext/gettext/src/Translations.php',
    'Gettext\\Translator' => $vendorDir . '/gettext/translator/src/Translator.php',
    'Gettext\\TranslatorFunctions' => $vendorDir . '/gettext/translator/src/TranslatorFunctions.php',
    'Gettext\\TranslatorInterface' => $vendorDir . '/gettext/translator/src/TranslatorInterface.php',
    'GuzzleHttp\\Psr7\\AppendStream' => $vendorDir . '/guzzlehttp/psr7/src/AppendStream.php',
    'GuzzleHttp\\Psr7\\BufferStream' => $vendorDir . '/guzzlehttp/psr7/src/BufferStream.php',
    'GuzzleHttp\\Psr7\\CachingStream' => $vendorDir . '/guzzlehttp/psr7/src/CachingStream.php',
    'GuzzleHttp\\Psr7\\DroppingStream' => $vendorDir . '/guzzlehttp/psr7/src/DroppingStream.php',
    'GuzzleHttp\\Psr7\\Exception\\MalformedUriException' => $vendorDir . '/guzzlehttp/psr7/src/Exception/MalformedUriException.php',
    'GuzzleHttp\\Psr7\\FnStream' => $vendorDir . '/guzzlehttp/psr7/src/FnStream.php',
    'GuzzleHttp\\Psr7\\Header' => $vendorDir . '/guzzlehttp/psr7/src/Header.php',
    'GuzzleHttp\\Psr7\\HttpFactory' => $vendorDir . '/guzzlehttp/psr7/src/HttpFactory.php',
    'GuzzleHttp\\Psr7\\InflateStream' => $vendorDir . '/guzzlehttp/psr7/src/InflateStream.php',
    'GuzzleHttp\\Psr7\\LazyOpenStream' => $vendorDir . '/guzzlehttp/psr7/src/LazyOpenStream.php',
    'GuzzleHttp\\Psr7\\LimitStream' => $vendorDir . '/guzzlehttp/psr7/src/LimitStream.php',
    'GuzzleHttp\\Psr7\\Message' => $vendorDir . '/guzzlehttp/psr7/src/Message.php',
    'GuzzleHttp\\Psr7\\MessageTrait' => $vendorDir . '/guzzlehttp/psr7/src/MessageTrait.php',
    'GuzzleHttp\\Psr7\\MimeType' => $vendorDir . '/guzzlehttp/psr7/src/MimeType.php',
    'GuzzleHttp\\Psr7\\MultipartStream' => $vendorDir . '/guzzlehttp/psr7/src/MultipartStream.php',
    'GuzzleHttp\\Psr7\\NoSeekStream' => $vendorDir . '/guzzlehttp/psr7/src/NoSeekStream.php',
    'GuzzleHttp\\Psr7\\PumpStream' => $vendorDir . '/guzzlehttp/psr7/src/PumpStream.php',
    'GuzzleHttp\\Psr7\\Query' => $vendorDir . '/guzzlehttp/psr7/src/Query.php',
    'GuzzleHttp\\Psr7\\Request' => $vendorDir . '/guzzlehttp/psr7/src/Request.php',
    'GuzzleHttp\\Psr7\\Response' => $vendorDir . '/guzzlehttp/psr7/src/Response.php',
    'GuzzleHttp\\Psr7\\Rfc3986' => $vendorDir . '/guzzlehttp/psr7/src/Rfc3986.php',
    'GuzzleHttp\\Psr7\\Rfc7230' => $vendorDir . '/guzzlehttp/psr7/src/Rfc7230.php',
    'GuzzleHttp\\Psr7\\ServerRequest' => $vendorDir . '/guzzlehttp/psr7/src/ServerRequest.php',
    'GuzzleHttp\\Psr7\\Stream' => $vendorDir . '/guzzlehttp/psr7/src/Stream.php',
    'GuzzleHttp\\Psr7\\StreamDecoratorTrait' => $vendorDir . '/guzzlehttp/psr7/src/StreamDecoratorTrait.php',
    'GuzzleHttp\\Psr7\\StreamWrapper' => $vendorDir . '/guzzlehttp/psr7/src/StreamWrapper.php',
    'GuzzleHttp\\Psr7\\UploadedFile' => $vendorDir . '/guzzlehttp/psr7/src/UploadedFile.php',
    'GuzzleHttp\\Psr7\\Uri' => $vendorDir . '/guzzlehttp/psr7/src/Uri.php',
    'GuzzleHttp\\Psr7\\UriComparator' => $vendorDir . '/guzzlehttp/psr7/src/UriComparator.php',
    'GuzzleHttp\\Psr7\\UriNormalizer' => $vendorDir . '/guzzlehttp/psr7/src/UriNormalizer.php',
    'GuzzleHttp\\Psr7\\UriResolver' => $vendorDir . '/guzzlehttp/psr7/src/UriResolver.php',
    'GuzzleHttp\\Psr7\\Utils' => $vendorDir . '/guzzlehttp/psr7/src/Utils.php',
    'IntlDateFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/Resources/stubs/IntlDateFormatter.php',
    'IntlListFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/Resources/stubs/IntlListFormatter.php',
    'Locale' => $vendorDir . '/symfony/polyfill-intl-icu/Resources/stubs/Locale.php',
    'NoDiscard' => $vendorDir . '/symfony/polyfill-php85/Resources/stubs/NoDiscard.php',
    'Normalizer' => $vendorDir . '/symfony/polyfill-intl-normalizer/Resources/stubs/Normalizer.php',
    'NumberFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/Resources/stubs/NumberFormatter.php',
    'Nyholm\\Psr7\\Factory\\HttplugFactory' => $vendorDir . '/nyholm/psr7/src/Factory/HttplugFactory.php',
    'Nyholm\\Psr7\\Factory\\Psr17Factory' => $vendorDir . '/nyholm/psr7/src/Factory/Psr17Factory.php',
    'Nyholm\\Psr7\\MessageTrait' => $vendorDir . '/nyholm/psr7/src/MessageTrait.php',
    'Nyholm\\Psr7\\Request' => $vendorDir . '/nyholm/psr7/src/Request.php',
    'Nyholm\\Psr7\\RequestTrait' => $vendorDir . '/nyholm/psr7/src/RequestTrait.php',
    'Nyholm\\Psr7\\Response' => $vendorDir . '/nyholm/psr7/src/Response.php',
    'Nyholm\\Psr7\\ServerRequest' => $vendorDir . '/nyholm/psr7/src/ServerRequest.php',
    'Nyholm\\Psr7\\Stream' => $vendorDir . '/nyholm/psr7/src/Stream.php',
    'Nyholm\\Psr7\\StreamTrait' => $vendorDir . '/nyholm/psr7/src/StreamTrait.php',
    'Nyholm\\Psr7\\UploadedFile' => $vendorDir . '/nyholm/psr7/src/UploadedFile.php',
    'Nyholm\\Psr7\\Uri' => $vendorDir . '/nyholm/psr7/src/Uri.php',
    'Override' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/Override.php',
    'PHPMailer\\PHPMailer\\DSNConfigurator' => $vendorDir . '/phpmailer/phpmailer/src/DSNConfigurator.php',
    'PHPMailer\\PHPMailer\\Exception' => $vendorDir . '/phpmailer/phpmailer/src/Exception.php',
    'PHPMailer\\PHPMailer\\OAuth' => $vendorDir . '/phpmailer/phpmailer/src/OAuth.php',
    'PHPMailer\\PHPMailer\\OAuthTokenProvider' => $vendorDir . '/phpmailer/phpmailer/src/OAuthTokenProvider.php',
    'PHPMailer\\PHPMailer\\PHPMailer' => $vendorDir . '/phpmailer/phpmailer/src/PHPMailer.php',
    'PHPMailer\\PHPMailer\\POP3' => $vendorDir . '/phpmailer/phpmailer/src/POP3.php',
    'PHPMailer\\PHPMailer\\SMTP' => $vendorDir . '/phpmailer/phpmailer/src/SMTP.php',
    'PhpParser\\Builder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder.php',
    'PhpParser\\BuilderFactory' => $vendorDir . '/nikic/php-parser/lib/PhpParser/BuilderFactory.php',
    'PhpParser\\BuilderHelpers' => $vendorDir . '/nikic/php-parser/lib/PhpParser/BuilderHelpers.php',
    'PhpParser\\Builder\\ClassConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/ClassConst.php',
    'PhpParser\\Builder\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Class_.php',
    'PhpParser\\Builder\\Declaration' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Declaration.php',
    'PhpParser\\Builder\\EnumCase' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/EnumCase.php',
    'PhpParser\\Builder\\Enum_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Enum_.php',
    'PhpParser\\Builder\\FunctionLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/FunctionLike.php',
    'PhpParser\\Builder\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Function_.php',
    'PhpParser\\Builder\\Interface_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Interface_.php',
    'PhpParser\\Builder\\Method' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Method.php',
    'PhpParser\\Builder\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Namespace_.php',
    'PhpParser\\Builder\\Param' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Param.php',
    'PhpParser\\Builder\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Property.php',
    'PhpParser\\Builder\\TraitUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/TraitUse.php',
    'PhpParser\\Builder\\TraitUseAdaptation' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/TraitUseAdaptation.php',
    'PhpParser\\Builder\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Trait_.php',
    'PhpParser\\Builder\\Use_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Use_.php',
    'PhpParser\\Comment' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Comment.php',
    'PhpParser\\Comment\\Doc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Comment/Doc.php',
    'PhpParser\\ConstExprEvaluationException' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ConstExprEvaluationException.php',
    'PhpParser\\ConstExprEvaluator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ConstExprEvaluator.php',
    'PhpParser\\Error' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Error.php',
    'PhpParser\\ErrorHandler' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler.php',
    'PhpParser\\ErrorHandler\\Collecting' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler/Collecting.php',
    'PhpParser\\ErrorHandler\\Throwing' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler/Throwing.php',
    'PhpParser\\Internal\\DiffElem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/DiffElem.php',
    'PhpParser\\Internal\\Differ' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/Differ.php',
    'PhpParser\\Internal\\PrintableNewAnonClassNode' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/PrintableNewAnonClassNode.php',
    'PhpParser\\Internal\\TokenPolyfill' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/TokenPolyfill.php',
    'PhpParser\\Internal\\TokenStream' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/TokenStream.php',
    'PhpParser\\JsonDecoder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/JsonDecoder.php',
    'PhpParser\\Lexer' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer.php',
    'PhpParser\\Lexer\\Emulative' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/Emulative.php',
    'PhpParser\\Lexer\\TokenEmulator\\AsymmetricVisibilityTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/AsymmetricVisibilityTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\AttributeEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/AttributeEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\EnumTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/EnumTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ExplicitOctalEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ExplicitOctalEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\FnTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/FnTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\KeywordEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/KeywordEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\MatchTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/MatchTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\NullsafeTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/NullsafeTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\PipeOperatorEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/PipeOperatorEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\PropertyTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/PropertyTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReadonlyFunctionTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReadonlyFunctionTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReadonlyTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReadonlyTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReverseEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReverseEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\TokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/TokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\VoidCastEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/VoidCastEmulator.php',
    'PhpParser\\Modifiers' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Modifiers.php',
    'PhpParser\\NameContext' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NameContext.php',
    'PhpParser\\Node' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node.php',
    'PhpParser\\NodeAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeAbstract.php',
    'PhpParser\\NodeDumper' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeDumper.php',
    'PhpParser\\NodeFinder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeFinder.php',
    'PhpParser\\NodeTraverser' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeTraverser.php',
    'PhpParser\\NodeTraverserInterface' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeTraverserInterface.php',
    'PhpParser\\NodeVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor.php',
    'PhpParser\\NodeVisitorAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitorAbstract.php',
    'PhpParser\\NodeVisitor\\CloningVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/CloningVisitor.php',
    'PhpParser\\NodeVisitor\\CommentAnnotatingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/CommentAnnotatingVisitor.php',
    'PhpParser\\NodeVisitor\\FindingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/FindingVisitor.php',
    'PhpParser\\NodeVisitor\\FirstFindingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/FirstFindingVisitor.php',
    'PhpParser\\NodeVisitor\\NameResolver' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/NameResolver.php',
    'PhpParser\\NodeVisitor\\NodeConnectingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/NodeConnectingVisitor.php',
    'PhpParser\\NodeVisitor\\ParentConnectingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/ParentConnectingVisitor.php',
    'PhpParser\\Node\\Arg' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Arg.php',
    'PhpParser\\Node\\ArrayItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ArrayItem.php',
    'PhpParser\\Node\\Attribute' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Attribute.php',
    'PhpParser\\Node\\AttributeGroup' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/AttributeGroup.php',
    'PhpParser\\Node\\ClosureUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ClosureUse.php',
    'PhpParser\\Node\\ComplexType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ComplexType.php',
    'PhpParser\\Node\\Const_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Const_.php',
    'PhpParser\\Node\\DeclareItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/DeclareItem.php',
    'PhpParser\\Node\\Expr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr.php',
    'PhpParser\\Node\\Expr\\ArrayDimFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ArrayDimFetch.php',
    'PhpParser\\Node\\Expr\\ArrayItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ArrayItem.php',
    'PhpParser\\Node\\Expr\\Array_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Array_.php',
    'PhpParser\\Node\\Expr\\ArrowFunction' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ArrowFunction.php',
    'PhpParser\\Node\\Expr\\Assign' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Assign.php',
    'PhpParser\\Node\\Expr\\AssignOp' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseAnd.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseOr.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseXor.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Coalesce' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Coalesce.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Concat' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Concat.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Div' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Div.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Minus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Minus.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Mod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Mod.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Mul' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Mul.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Plus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Plus.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Pow' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Pow.php',
    'PhpParser\\Node\\Expr\\AssignOp\\ShiftLeft' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/ShiftLeft.php',
    'PhpParser\\Node\\Expr\\AssignOp\\ShiftRight' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/ShiftRight.php',
    'PhpParser\\Node\\Expr\\AssignRef' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignRef.php',
    'PhpParser\\Node\\Expr\\BinaryOp' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseXor.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BooleanAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BooleanAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BooleanOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BooleanOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Coalesce' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Coalesce.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Concat' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Concat.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Div' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Div.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Equal' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Equal.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Greater' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Greater.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\GreaterOrEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/GreaterOrEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Identical' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Identical.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalXor.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Minus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Minus.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Mod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Mod.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Mul' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Mul.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\NotEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/NotEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\NotIdentical' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/NotIdentical.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Pipe' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Pipe.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Plus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Plus.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Pow' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Pow.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\ShiftLeft' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/ShiftLeft.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\ShiftRight' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/ShiftRight.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Smaller' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Smaller.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\SmallerOrEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/SmallerOrEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Spaceship' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Spaceship.php',
    'PhpParser\\Node\\Expr\\BitwiseNot' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BitwiseNot.php',
    'PhpParser\\Node\\Expr\\BooleanNot' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BooleanNot.php',
    'PhpParser\\Node\\Expr\\CallLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/CallLike.php',
    'PhpParser\\Node\\Expr\\Cast' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast.php',
    'PhpParser\\Node\\Expr\\Cast\\Array_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Array_.php',
    'PhpParser\\Node\\Expr\\Cast\\Bool_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Bool_.php',
    'PhpParser\\Node\\Expr\\Cast\\Double' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Double.php',
    'PhpParser\\Node\\Expr\\Cast\\Int_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Int_.php',
    'PhpParser\\Node\\Expr\\Cast\\Object_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Object_.php',
    'PhpParser\\Node\\Expr\\Cast\\String_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/String_.php',
    'PhpParser\\Node\\Expr\\Cast\\Unset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Unset_.php',
    'PhpParser\\Node\\Expr\\Cast\\Void_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Void_.php',
    'PhpParser\\Node\\Expr\\ClassConstFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ClassConstFetch.php',
    'PhpParser\\Node\\Expr\\Clone_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Clone_.php',
    'PhpParser\\Node\\Expr\\Closure' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Closure.php',
    'PhpParser\\Node\\Expr\\ClosureUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ClosureUse.php',
    'PhpParser\\Node\\Expr\\ConstFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ConstFetch.php',
    'PhpParser\\Node\\Expr\\Empty_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Empty_.php',
    'PhpParser\\Node\\Expr\\Error' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Error.php',
    'PhpParser\\Node\\Expr\\ErrorSuppress' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ErrorSuppress.php',
    'PhpParser\\Node\\Expr\\Eval_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Eval_.php',
    'PhpParser\\Node\\Expr\\Exit_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Exit_.php',
    'PhpParser\\Node\\Expr\\FuncCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/FuncCall.php',
    'PhpParser\\Node\\Expr\\Include_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Include_.php',
    'PhpParser\\Node\\Expr\\Instanceof_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Instanceof_.php',
    'PhpParser\\Node\\Expr\\Isset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Isset_.php',
    'PhpParser\\Node\\Expr\\List_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/List_.php',
    'PhpParser\\Node\\Expr\\Match_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Match_.php',
    'PhpParser\\Node\\Expr\\MethodCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/MethodCall.php',
    'PhpParser\\Node\\Expr\\New_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/New_.php',
    'PhpParser\\Node\\Expr\\NullsafeMethodCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/NullsafeMethodCall.php',
    'PhpParser\\Node\\Expr\\NullsafePropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/NullsafePropertyFetch.php',
    'PhpParser\\Node\\Expr\\PostDec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PostDec.php',
    'PhpParser\\Node\\Expr\\PostInc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PostInc.php',
    'PhpParser\\Node\\Expr\\PreDec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PreDec.php',
    'PhpParser\\Node\\Expr\\PreInc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PreInc.php',
    'PhpParser\\Node\\Expr\\Print_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Print_.php',
    'PhpParser\\Node\\Expr\\PropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PropertyFetch.php',
    'PhpParser\\Node\\Expr\\ShellExec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ShellExec.php',
    'PhpParser\\Node\\Expr\\StaticCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/StaticCall.php',
    'PhpParser\\Node\\Expr\\StaticPropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/StaticPropertyFetch.php',
    'PhpParser\\Node\\Expr\\Ternary' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Ternary.php',
    'PhpParser\\Node\\Expr\\Throw_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Throw_.php',
    'PhpParser\\Node\\Expr\\UnaryMinus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/UnaryMinus.php',
    'PhpParser\\Node\\Expr\\UnaryPlus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/UnaryPlus.php',
    'PhpParser\\Node\\Expr\\Variable' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Variable.php',
    'PhpParser\\Node\\Expr\\YieldFrom' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/YieldFrom.php',
    'PhpParser\\Node\\Expr\\Yield_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Yield_.php',
    'PhpParser\\Node\\FunctionLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/FunctionLike.php',
    'PhpParser\\Node\\Identifier' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Identifier.php',
    'PhpParser\\Node\\InterpolatedStringPart' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/InterpolatedStringPart.php',
    'PhpParser\\Node\\IntersectionType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/IntersectionType.php',
    'PhpParser\\Node\\MatchArm' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/MatchArm.php',
    'PhpParser\\Node\\Name' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name.php',
    'PhpParser\\Node\\Name\\FullyQualified' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name/FullyQualified.php',
    'PhpParser\\Node\\Name\\Relative' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name/Relative.php',
    'PhpParser\\Node\\NullableType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/NullableType.php',
    'PhpParser\\Node\\Param' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Param.php',
    'PhpParser\\Node\\PropertyHook' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/PropertyHook.php',
    'PhpParser\\Node\\PropertyItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/PropertyItem.php',
    'PhpParser\\Node\\Scalar' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar.php',
    'PhpParser\\Node\\Scalar\\DNumber' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/DNumber.php',
    'PhpParser\\Node\\Scalar\\Encapsed' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/Encapsed.php',
    'PhpParser\\Node\\Scalar\\EncapsedStringPart' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/EncapsedStringPart.php',
    'PhpParser\\Node\\Scalar\\Float_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/Float_.php',
    'PhpParser\\Node\\Scalar\\Int_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/Int_.php',
    'PhpParser\\Node\\Scalar\\InterpolatedString' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/InterpolatedString.php',
    'PhpParser\\Node\\Scalar\\LNumber' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/LNumber.php',
    'PhpParser\\Node\\Scalar\\MagicConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Class_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Dir' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Dir.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\File' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/File.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Function_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Line' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Line.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Method' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Method.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Namespace_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Property.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Trait_.php',
    'PhpParser\\Node\\Scalar\\String_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/String_.php',
    'PhpParser\\Node\\StaticVar' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/StaticVar.php',
    'PhpParser\\Node\\Stmt' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt.php',
    'PhpParser\\Node\\Stmt\\Block' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Block.php',
    'PhpParser\\Node\\Stmt\\Break_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Break_.php',
    'PhpParser\\Node\\Stmt\\Case_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Case_.php',
    'PhpParser\\Node\\Stmt\\Catch_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Catch_.php',
    'PhpParser\\Node\\Stmt\\ClassConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassConst.php',
    'PhpParser\\Node\\Stmt\\ClassLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassLike.php',
    'PhpParser\\Node\\Stmt\\ClassMethod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassMethod.php',
    'PhpParser\\Node\\Stmt\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Class_.php',
    'PhpParser\\Node\\Stmt\\Const_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Const_.php',
    'PhpParser\\Node\\Stmt\\Continue_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Continue_.php',
    'PhpParser\\Node\\Stmt\\DeclareDeclare' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/DeclareDeclare.php',
    'PhpParser\\Node\\Stmt\\Declare_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Declare_.php',
    'PhpParser\\Node\\Stmt\\Do_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Do_.php',
    'PhpParser\\Node\\Stmt\\Echo_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Echo_.php',
    'PhpParser\\Node\\Stmt\\ElseIf_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ElseIf_.php',
    'PhpParser\\Node\\Stmt\\Else_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Else_.php',
    'PhpParser\\Node\\Stmt\\EnumCase' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/EnumCase.php',
    'PhpParser\\Node\\Stmt\\Enum_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Enum_.php',
    'PhpParser\\Node\\Stmt\\Expression' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Expression.php',
    'PhpParser\\Node\\Stmt\\Finally_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Finally_.php',
    'PhpParser\\Node\\Stmt\\For_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/For_.php',
    'PhpParser\\Node\\Stmt\\Foreach_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Foreach_.php',
    'PhpParser\\Node\\Stmt\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Function_.php',
    'PhpParser\\Node\\Stmt\\Global_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Global_.php',
    'PhpParser\\Node\\Stmt\\Goto_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Goto_.php',
    'PhpParser\\Node\\Stmt\\GroupUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/GroupUse.php',
    'PhpParser\\Node\\Stmt\\HaltCompiler' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/HaltCompiler.php',
    'PhpParser\\Node\\Stmt\\If_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/If_.php',
    'PhpParser\\Node\\Stmt\\InlineHTML' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/InlineHTML.php',
    'PhpParser\\Node\\Stmt\\Interface_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Interface_.php',
    'PhpParser\\Node\\Stmt\\Label' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Label.php',
    'PhpParser\\Node\\Stmt\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Namespace_.php',
    'PhpParser\\Node\\Stmt\\Nop' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Nop.php',
    'PhpParser\\Node\\Stmt\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Property.php',
    'PhpParser\\Node\\Stmt\\PropertyProperty' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/PropertyProperty.php',
    'PhpParser\\Node\\Stmt\\Return_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Return_.php',
    'PhpParser\\Node\\Stmt\\StaticVar' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/StaticVar.php',
    'PhpParser\\Node\\Stmt\\Static_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Static_.php',
    'PhpParser\\Node\\Stmt\\Switch_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Switch_.php',
    'PhpParser\\Node\\Stmt\\TraitUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUse.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation\\Alias' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation/Alias.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation\\Precedence' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation/Precedence.php',
    'PhpParser\\Node\\Stmt\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Trait_.php',
    'PhpParser\\Node\\Stmt\\TryCatch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TryCatch.php',
    'PhpParser\\Node\\Stmt\\Unset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Unset_.php',
    'PhpParser\\Node\\Stmt\\UseUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/UseUse.php',
    'PhpParser\\Node\\Stmt\\Use_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Use_.php',
    'PhpParser\\Node\\Stmt\\While_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/While_.php',
    'PhpParser\\Node\\UnionType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/UnionType.php',
    'PhpParser\\Node\\UseItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/UseItem.php',
    'PhpParser\\Node\\VarLikeIdentifier' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/VarLikeIdentifier.php',
    'PhpParser\\Node\\VariadicPlaceholder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/VariadicPlaceholder.php',
    'PhpParser\\Parser' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser.php',
    'PhpParser\\ParserAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ParserAbstract.php',
    'PhpParser\\ParserFactory' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ParserFactory.php',
    'PhpParser\\Parser\\Php7' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser/Php7.php',
    'PhpParser\\Parser\\Php8' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser/Php8.php',
    'PhpParser\\PhpVersion' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PhpVersion.php',
    'PhpParser\\PrettyPrinter' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinter.php',
    'PhpParser\\PrettyPrinterAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinterAbstract.php',
    'PhpParser\\PrettyPrinter\\Standard' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinter/Standard.php',
    'PhpParser\\Token' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Token.php',
    'PhpToken' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/PhpToken.php',
    'Psr\\Cache\\CacheException' => $vendorDir . '/psr/cache/src/CacheException.php',
    'Psr\\Cache\\CacheItemInterface' => $vendorDir . '/psr/cache/src/CacheItemInterface.php',
    'Psr\\Cache\\CacheItemPoolInterface' => $vendorDir . '/psr/cache/src/CacheItemPoolInterface.php',
    'Psr\\Cache\\InvalidArgumentException' => $vendorDir . '/psr/cache/src/InvalidArgumentException.php',
    'Psr\\Clock\\ClockInterface' => $vendorDir . '/psr/clock/src/ClockInterface.php',
    'Psr\\Container\\ContainerExceptionInterface' => $vendorDir . '/psr/container/src/ContainerExceptionInterface.php',
    'Psr\\Container\\ContainerInterface' => $vendorDir . '/psr/container/src/ContainerInterface.php',
    'Psr\\Container\\NotFoundExceptionInterface' => $vendorDir . '/psr/container/src/NotFoundExceptionInterface.php',
    'Psr\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/psr/event-dispatcher/src/EventDispatcherInterface.php',
    'Psr\\EventDispatcher\\ListenerProviderInterface' => $vendorDir . '/psr/event-dispatcher/src/ListenerProviderInterface.php',
    'Psr\\EventDispatcher\\StoppableEventInterface' => $vendorDir . '/psr/event-dispatcher/src/StoppableEventInterface.php',
    'Psr\\Http\\Message\\MessageInterface' => $vendorDir . '/psr/http-message/src/MessageInterface.php',
    'Psr\\Http\\Message\\RequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/RequestFactoryInterface.php',
    'Psr\\Http\\Message\\RequestInterface' => $vendorDir . '/psr/http-message/src/RequestInterface.php',
    'Psr\\Http\\Message\\ResponseFactoryInterface' => $vendorDir . '/psr/http-factory/src/ResponseFactoryInterface.php',
    'Psr\\Http\\Message\\ResponseInterface' => $vendorDir . '/psr/http-message/src/ResponseInterface.php',
    'Psr\\Http\\Message\\ServerRequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/ServerRequestFactoryInterface.php',
    'Psr\\Http\\Message\\ServerRequestInterface' => $vendorDir . '/psr/http-message/src/ServerRequestInterface.php',
    'Psr\\Http\\Message\\StreamFactoryInterface' => $vendorDir . '/psr/http-factory/src/StreamFactoryInterface.php',
    'Psr\\Http\\Message\\StreamInterface' => $vendorDir . '/psr/http-message/src/StreamInterface.php',
    'Psr\\Http\\Message\\UploadedFileFactoryInterface' => $vendorDir . '/psr/http-factory/src/UploadedFileFactoryInterface.php',
    'Psr\\Http\\Message\\UploadedFileInterface' => $vendorDir . '/psr/http-message/src/UploadedFileInterface.php',
    'Psr\\Http\\Message\\UriFactoryInterface' => $vendorDir . '/psr/http-factory/src/UriFactoryInterface.php',
    'Psr\\Http\\Message\\UriInterface' => $vendorDir . '/psr/http-message/src/UriInterface.php',
    'Psr\\Log\\AbstractLogger' => $vendorDir . '/psr/log/src/AbstractLogger.php',
    'Psr\\Log\\InvalidArgumentException' => $vendorDir . '/psr/log/src/InvalidArgumentException.php',
    'Psr\\Log\\LogLevel' => $vendorDir . '/psr/log/src/LogLevel.php',
    'Psr\\Log\\LoggerAwareInterface' => $vendorDir . '/psr/log/src/LoggerAwareInterface.php',
    'Psr\\Log\\LoggerAwareTrait' => $vendorDir . '/psr/log/src/LoggerAwareTrait.php',
    'Psr\\Log\\LoggerInterface' => $vendorDir . '/psr/log/src/LoggerInterface.php',
    'Psr\\Log\\LoggerTrait' => $vendorDir . '/psr/log/src/LoggerTrait.php',
    'Psr\\Log\\NullLogger' => $vendorDir . '/psr/log/src/NullLogger.php',
    'RobRichards\\XMLSecLibs\\Utils\\XPath' => $vendorDir . '/robrichards/xmlseclibs/src/Utils/XPath.php',
    'RobRichards\\XMLSecLibs\\XMLSecEnc' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecEnc.php',
    'RobRichards\\XMLSecLibs\\XMLSecurityDSig' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityDSig.php',
    'RobRichards\\XMLSecLibs\\XMLSecurityKey' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityKey.php',
    'SAML2\\ArtifactResolve' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/ArtifactResolve.php',
    'SAML2\\ArtifactResponse' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/ArtifactResponse.php',
    'SAML2\\Assertion' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion.php',
    'SAML2\\Assertion\\Decrypter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Decrypter.php',
    'SAML2\\Assertion\\Exception\\InvalidAssertionException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Exception/InvalidAssertionException.php',
    'SAML2\\Assertion\\Exception\\InvalidSubjectConfirmationException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Exception/InvalidSubjectConfirmationException.php',
    'SAML2\\Assertion\\Exception\\NotDecryptedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Exception/NotDecryptedException.php',
    'SAML2\\Assertion\\Processor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Processor.php',
    'SAML2\\Assertion\\ProcessorBuilder' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/ProcessorBuilder.php',
    'SAML2\\Assertion\\Transformer\\DecodeBase64Transformer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Transformer/DecodeBase64Transformer.php',
    'SAML2\\Assertion\\Transformer\\NameIdDecryptionTransformer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Transformer/NameIdDecryptionTransformer.php',
    'SAML2\\Assertion\\Transformer\\Transformer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Transformer/Transformer.php',
    'SAML2\\Assertion\\Transformer\\TransformerChain' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Transformer/TransformerChain.php',
    'SAML2\\Assertion\\Validation\\AssertionConstraintValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/AssertionConstraintValidator.php',
    'SAML2\\Assertion\\Validation\\AssertionValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/AssertionValidator.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\NotBefore' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/NotBefore.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\NotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/NotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SessionNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SessionNotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SpIsValidAudience' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SpIsValidAudience.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationMethod' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationMethod.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotBefore' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotBefore.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationRecipientMatches' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationRecipientMatches.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationResponseToMatches' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationResponseToMatches.php',
    'SAML2\\Assertion\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/Result.php',
    'SAML2\\Assertion\\Validation\\SubjectConfirmationConstraintValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/SubjectConfirmationConstraintValidator.php',
    'SAML2\\Assertion\\Validation\\SubjectConfirmationValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Assertion/Validation/SubjectConfirmationValidator.php',
    'SAML2\\AttributeQuery' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/AttributeQuery.php',
    'SAML2\\AuthnRequest' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/AuthnRequest.php',
    'SAML2\\Binding' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Binding.php',
    'SAML2\\Certificate\\Exception\\InvalidCertificateStructureException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/Exception/InvalidCertificateStructureException.php',
    'SAML2\\Certificate\\Exception\\InvalidKeyUsageException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/Exception/InvalidKeyUsageException.php',
    'SAML2\\Certificate\\Exception\\NoKeysFoundException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/Exception/NoKeysFoundException.php',
    'SAML2\\Certificate\\Key' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/Key.php',
    'SAML2\\Certificate\\KeyCollection' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/KeyCollection.php',
    'SAML2\\Certificate\\KeyLoader' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/KeyLoader.php',
    'SAML2\\Certificate\\PrivateKey' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/PrivateKey.php',
    'SAML2\\Certificate\\PrivateKeyLoader' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/PrivateKeyLoader.php',
    'SAML2\\Certificate\\X509' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Certificate/X509.php',
    'SAML2\\Compat\\AbstractContainer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Compat/AbstractContainer.php',
    'SAML2\\Compat\\ContainerSingleton' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Compat/ContainerSingleton.php',
    'SAML2\\Compat\\MockContainer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Compat/MockContainer.php',
    'SAML2\\Compat\\Ssp\\Container' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Compat/Ssp/Container.php',
    'SAML2\\Compat\\Ssp\\Logger' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Compat/Ssp/Logger.php',
    'SAML2\\Configuration\\ArrayAdapter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/ArrayAdapter.php',
    'SAML2\\Configuration\\CertificateProvider' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/CertificateProvider.php',
    'SAML2\\Configuration\\DecryptionProvider' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/DecryptionProvider.php',
    'SAML2\\Configuration\\Destination' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/Destination.php',
    'SAML2\\Configuration\\EntityIdProvider' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/EntityIdProvider.php',
    'SAML2\\Configuration\\IdentityProvider' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/IdentityProvider.php',
    'SAML2\\Configuration\\IdentityProviderAware' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/IdentityProviderAware.php',
    'SAML2\\Configuration\\PrivateKey' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/PrivateKey.php',
    'SAML2\\Configuration\\Queryable' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/Queryable.php',
    'SAML2\\Configuration\\ServiceProvider' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/ServiceProvider.php',
    'SAML2\\Configuration\\ServiceProviderAware' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/ServiceProviderAware.php',
    'SAML2\\Configuration\\SimpleSAMLConverter' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Configuration/SimpleSAMLConverter.php',
    'SAML2\\Constants' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Constants.php',
    'SAML2\\DOMDocumentFactory' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/DOMDocumentFactory.php',
    'SAML2\\EncryptedAssertion' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/EncryptedAssertion.php',
    'SAML2\\Exception\\InvalidArgumentException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/InvalidArgumentException.php',
    'SAML2\\Exception\\MetadataExpiredException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/MetadataExpiredException.php',
    'SAML2\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/ProtocolViolationException.php',
    'SAML2\\Exception\\Protocol\\AuthnFailedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/AuthnFailedException.php',
    'SAML2\\Exception\\Protocol\\InvalidAttrNameOrValueException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/InvalidAttrNameOrValueException.php',
    'SAML2\\Exception\\Protocol\\InvalidNameIDPolicyException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/InvalidNameIDPolicyException.php',
    'SAML2\\Exception\\Protocol\\NoAuthnContextException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/NoAuthnContextException.php',
    'SAML2\\Exception\\Protocol\\NoAvailableIDPException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/NoAvailableIDPException.php',
    'SAML2\\Exception\\Protocol\\NoPassiveException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/NoPassiveException.php',
    'SAML2\\Exception\\Protocol\\NoSupportedIDPException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/NoSupportedIDPException.php',
    'SAML2\\Exception\\Protocol\\PartialLogoutException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/PartialLogoutException.php',
    'SAML2\\Exception\\Protocol\\ProxyCountExceededException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/ProxyCountExceededException.php',
    'SAML2\\Exception\\Protocol\\RequestDeniedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/RequestDeniedException.php',
    'SAML2\\Exception\\Protocol\\RequestUnsupportedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/RequestUnsupportedException.php',
    'SAML2\\Exception\\Protocol\\RequestVersionDeprecatedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/RequestVersionDeprecatedException.php',
    'SAML2\\Exception\\Protocol\\RequestVersionTooHighException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/RequestVersionTooHighException.php',
    'SAML2\\Exception\\Protocol\\RequestVersionTooLowException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/RequestVersionTooLowException.php',
    'SAML2\\Exception\\Protocol\\ResourceNotRecognizedException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/ResourceNotRecognizedException.php',
    'SAML2\\Exception\\Protocol\\TooManyResponsesException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/TooManyResponsesException.php',
    'SAML2\\Exception\\Protocol\\UnknownAttrProfileException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/UnknownAttrProfileException.php',
    'SAML2\\Exception\\Protocol\\UnknownPrincipalException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/UnknownPrincipalException.php',
    'SAML2\\Exception\\Protocol\\UnsupportedBindingException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Protocol/UnsupportedBindingException.php',
    'SAML2\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/RuntimeException.php',
    'SAML2\\Exception\\Throwable' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/Throwable.php',
    'SAML2\\Exception\\UnparseableXmlException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Exception/UnparseableXmlException.php',
    'SAML2\\HTTPArtifact' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/HTTPArtifact.php',
    'SAML2\\HTTPPost' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/HTTPPost.php',
    'SAML2\\HTTPRedirect' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/HTTPRedirect.php',
    'SAML2\\LogoutRequest' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/LogoutRequest.php',
    'SAML2\\LogoutResponse' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/LogoutResponse.php',
    'SAML2\\Message' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Message.php',
    'SAML2\\Request' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Request.php',
    'SAML2\\Response' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response.php',
    'SAML2\\Response\\Exception\\InvalidResponseException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/InvalidResponseException.php',
    'SAML2\\Response\\Exception\\InvalidSignatureException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/InvalidSignatureException.php',
    'SAML2\\Response\\Exception\\NoAssertionsFoundException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/NoAssertionsFoundException.php',
    'SAML2\\Response\\Exception\\PreconditionNotMetException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/PreconditionNotMetException.php',
    'SAML2\\Response\\Exception\\UnencryptedAssertionFoundException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/UnencryptedAssertionFoundException.php',
    'SAML2\\Response\\Exception\\UnsignedResponseException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Exception/UnsignedResponseException.php',
    'SAML2\\Response\\Processor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Processor.php',
    'SAML2\\Response\\Validation\\ConstraintValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/ConstraintValidator.php',
    'SAML2\\Response\\Validation\\ConstraintValidator\\DestinationMatches' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/ConstraintValidator/DestinationMatches.php',
    'SAML2\\Response\\Validation\\ConstraintValidator\\IsSuccessful' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/ConstraintValidator/IsSuccessful.php',
    'SAML2\\Response\\Validation\\PreconditionValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/PreconditionValidator.php',
    'SAML2\\Response\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/Result.php',
    'SAML2\\Response\\Validation\\Validator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Response/Validation/Validator.php',
    'SAML2\\SOAP' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/SOAP.php',
    'SAML2\\SOAPClient' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/SOAPClient.php',
    'SAML2\\Signature\\AbstractChainedValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/AbstractChainedValidator.php',
    'SAML2\\Signature\\ChainedValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/ChainedValidator.php',
    'SAML2\\Signature\\MissingConfigurationException' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/MissingConfigurationException.php',
    'SAML2\\Signature\\PublicKeyValidator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/PublicKeyValidator.php',
    'SAML2\\Signature\\Validator' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/Validator.php',
    'SAML2\\Signature\\ValidatorChain' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/ValidatorChain.php',
    'SAML2\\Signature\\ValidatorInterface' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Signature/ValidatorInterface.php',
    'SAML2\\SignedElement' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/SignedElement.php',
    'SAML2\\SignedElementHelper' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/SignedElementHelper.php',
    'SAML2\\StatusResponse' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/StatusResponse.php',
    'SAML2\\SubjectQuery' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/SubjectQuery.php',
    'SAML2\\Utilities\\ArrayCollection' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utilities/ArrayCollection.php',
    'SAML2\\Utilities\\Certificate' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utilities/Certificate.php',
    'SAML2\\Utilities\\Collection' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utilities/Collection.php',
    'SAML2\\Utilities\\File' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utilities/File.php',
    'SAML2\\Utilities\\Temporal' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utilities/Temporal.php',
    'SAML2\\Utils' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/Utils.php',
    'SAML2\\XML\\alg\\Common' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/alg/Common.php',
    'SAML2\\XML\\alg\\DigestMethod' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/alg/DigestMethod.php',
    'SAML2\\XML\\alg\\SigningMethod' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/alg/SigningMethod.php',
    'SAML2\\XML\\ds\\KeyInfo' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/ds/KeyInfo.php',
    'SAML2\\XML\\ds\\KeyName' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/ds/KeyName.php',
    'SAML2\\XML\\ds\\X509Certificate' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/ds/X509Certificate.php',
    'SAML2\\XML\\ds\\X509Data' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/ds/X509Data.php',
    'SAML2\\XML\\ecp\\Response' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/ecp/Response.php',
    'SAML2\\XML\\idpdisc\\DiscoveryResponse' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/idpdisc/DiscoveryResponse.php',
    'SAML2\\XML\\md\\AdditionalMetadataLocation' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/AdditionalMetadataLocation.php',
    'SAML2\\XML\\md\\AffiliationDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/AffiliationDescriptor.php',
    'SAML2\\XML\\md\\AttributeAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/AttributeAuthorityDescriptor.php',
    'SAML2\\XML\\md\\AttributeConsumingService' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/AttributeConsumingService.php',
    'SAML2\\XML\\md\\AuthnAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/AuthnAuthorityDescriptor.php',
    'SAML2\\XML\\md\\ContactPerson' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/ContactPerson.php',
    'SAML2\\XML\\md\\EndpointType' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/EndpointType.php',
    'SAML2\\XML\\md\\EntitiesDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/EntitiesDescriptor.php',
    'SAML2\\XML\\md\\EntityDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/EntityDescriptor.php',
    'SAML2\\XML\\md\\Extensions' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/Extensions.php',
    'SAML2\\XML\\md\\IDPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/IDPSSODescriptor.php',
    'SAML2\\XML\\md\\IndexedEndpointType' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/IndexedEndpointType.php',
    'SAML2\\XML\\md\\KeyDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/KeyDescriptor.php',
    'SAML2\\XML\\md\\Organization' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/Organization.php',
    'SAML2\\XML\\md\\PDPDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/PDPDescriptor.php',
    'SAML2\\XML\\md\\RequestedAttribute' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/RequestedAttribute.php',
    'SAML2\\XML\\md\\RoleDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/RoleDescriptor.php',
    'SAML2\\XML\\md\\SPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/SPSSODescriptor.php',
    'SAML2\\XML\\md\\SSODescriptorType' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/SSODescriptorType.php',
    'SAML2\\XML\\md\\UnknownRoleDescriptor' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/md/UnknownRoleDescriptor.php',
    'SAML2\\XML\\mdattr\\EntityAttributes' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdattr/EntityAttributes.php',
    'SAML2\\XML\\mdrpi\\Common' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdrpi/Common.php',
    'SAML2\\XML\\mdrpi\\PublicationInfo' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdrpi/PublicationInfo.php',
    'SAML2\\XML\\mdrpi\\RegistrationInfo' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdrpi/RegistrationInfo.php',
    'SAML2\\XML\\mdui\\Common' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdui/Common.php',
    'SAML2\\XML\\mdui\\DiscoHints' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdui/DiscoHints.php',
    'SAML2\\XML\\mdui\\Keywords' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdui/Keywords.php',
    'SAML2\\XML\\mdui\\Logo' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdui/Logo.php',
    'SAML2\\XML\\mdui\\UIInfo' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/mdui/UIInfo.php',
    'SAML2\\XML\\saml\\Attribute' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/Attribute.php',
    'SAML2\\XML\\saml\\AttributeValue' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/AttributeValue.php',
    'SAML2\\XML\\saml\\BaseIDType' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/BaseIDType.php',
    'SAML2\\XML\\saml\\IDNameQualifiersTrait' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/IDNameQualifiersTrait.php',
    'SAML2\\XML\\saml\\Issuer' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/Issuer.php',
    'SAML2\\XML\\saml\\NameID' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/NameID.php',
    'SAML2\\XML\\saml\\NameIDType' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/NameIDType.php',
    'SAML2\\XML\\saml\\SubjectConfirmation' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/SubjectConfirmation.php',
    'SAML2\\XML\\saml\\SubjectConfirmationData' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/saml/SubjectConfirmationData.php',
    'SAML2\\XML\\samlp\\Extensions' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/samlp/Extensions.php',
    'SAML2\\XML\\shibmd\\Scope' => $vendorDir . '/simplesamlphp/saml2-legacy/src/SAML2/XML/shibmd/Scope.php',
    'SQLite3Exception' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/SQLite3Exception.php',
    'SimpleSAML\\Assert\\Assert' => $vendorDir . '/simplesamlphp/assert/src/Assert.php',
    'SimpleSAML\\Assert\\AssertionFailedException' => $vendorDir . '/simplesamlphp/assert/src/AssertionFailedException.php',
    'SimpleSAML\\Assert\\Base64Trait' => $vendorDir . '/simplesamlphp/assert/src/Base64Trait.php',
    'SimpleSAML\\Assert\\URITrait' => $vendorDir . '/simplesamlphp/assert/src/URITrait.php',
    'SimpleSAML\\Auth\\AuthenticationFactory' => $baseDir . '/src/SimpleSAML/Auth/AuthenticationFactory.php',
    'SimpleSAML\\Auth\\ProcessingChain' => $baseDir . '/src/SimpleSAML/Auth/ProcessingChain.php',
    'SimpleSAML\\Auth\\ProcessingFilter' => $baseDir . '/src/SimpleSAML/Auth/ProcessingFilter.php',
    'SimpleSAML\\Auth\\Simple' => $baseDir . '/src/SimpleSAML/Auth/Simple.php',
    'SimpleSAML\\Auth\\Source' => $baseDir . '/src/SimpleSAML/Auth/Source.php',
    'SimpleSAML\\Auth\\SourceFactory' => $baseDir . '/src/SimpleSAML/Auth/SourceFactory.php',
    'SimpleSAML\\Auth\\State' => $baseDir . '/src/SimpleSAML/Auth/State.php',
    'SimpleSAML\\Command\\RouterDebugCommand' => $baseDir . '/src/SimpleSAML/Command/RouterDebugCommand.php',
    'SimpleSAML\\Command\\SspCacheClearCommand' => $baseDir . '/src/SimpleSAML/Command/SspCacheClearCommand.php',
    'SimpleSAML\\Command\\UnusedTranslatableStringsCommand' => $baseDir . '/src/SimpleSAML/Command/UnusedTranslatableStringsCommand.php',
    'SimpleSAML\\Command\\UpdateTranslatableStringsCommand' => $baseDir . '/src/SimpleSAML/Command/UpdateTranslatableStringsCommand.php',
    'SimpleSAML\\Compat\\Logger' => $baseDir . '/src/SimpleSAML/Compat/Logger.php',
    'SimpleSAML\\Compat\\SspContainer' => $baseDir . '/src/SimpleSAML/Compat/SspContainer.php',
    'SimpleSAML\\Composer\\ModuleInstaller' => $vendorDir . '/simplesamlphp/composer-module-installer/src/ModuleInstaller.php',
    'SimpleSAML\\Composer\\ModuleInstallerPlugin' => $vendorDir . '/simplesamlphp/composer-module-installer/src/ModuleInstallerPlugin.php',
    'SimpleSAML\\Composer\\XMLProvider\\XMLProviderInstaller' => $vendorDir . '/simplesamlphp/composer-xmlprovider-installer/src/XMLProviderInstaller.php',
    'SimpleSAML\\Composer\\XMLProvider\\XMLProviderInstallerPlugin' => $vendorDir . '/simplesamlphp/composer-xmlprovider-installer/src/XMLProviderInstallerPlugin.php',
    'SimpleSAML\\Configuration' => $baseDir . '/src/SimpleSAML/Configuration.php',
    'SimpleSAML\\Console\\Application' => $baseDir . '/src/SimpleSAML/Console/Application.php',
    'SimpleSAML\\Database' => $baseDir . '/src/SimpleSAML/Database.php',
    'SimpleSAML\\Error\\AuthSource' => $baseDir . '/src/SimpleSAML/Error/AuthSource.php',
    'SimpleSAML\\Error\\BadRequest' => $baseDir . '/src/SimpleSAML/Error/BadRequest.php',
    'SimpleSAML\\Error\\BadUserInput' => $baseDir . '/src/SimpleSAML/Error/BadUserInput.php',
    'SimpleSAML\\Error\\CannotSetCookie' => $baseDir . '/src/SimpleSAML/Error/CannotSetCookie.php',
    'SimpleSAML\\Error\\ConfigurationError' => $baseDir . '/src/SimpleSAML/Error/ConfigurationError.php',
    'SimpleSAML\\Error\\CriticalConfigurationError' => $baseDir . '/src/SimpleSAML/Error/CriticalConfigurationError.php',
    'SimpleSAML\\Error\\Error' => $baseDir . '/src/SimpleSAML/Error/Error.php',
    'SimpleSAML\\Error\\ErrorCodes' => $baseDir . '/src/SimpleSAML/Error/ErrorCodes.php',
    'SimpleSAML\\Error\\ErrorHandler' => $baseDir . '/src/SimpleSAML/Error/ErrorHandler.php',
    'SimpleSAML\\Error\\Exception' => $baseDir . '/src/SimpleSAML/Error/Exception.php',
    'SimpleSAML\\Error\\ExceptionHandler' => $baseDir . '/src/SimpleSAML/Error/ExceptionHandler.php',
    'SimpleSAML\\Error\\InvalidCredential' => $baseDir . '/src/SimpleSAML/Error/InvalidCredential.php',
    'SimpleSAML\\Error\\MetadataNotFound' => $baseDir . '/src/SimpleSAML/Error/MetadataNotFound.php',
    'SimpleSAML\\Error\\MethodNotAllowed' => $baseDir . '/src/SimpleSAML/Error/MethodNotAllowed.php',
    'SimpleSAML\\Error\\NoState' => $baseDir . '/src/SimpleSAML/Error/NoState.php',
    'SimpleSAML\\Error\\NotFound' => $baseDir . '/src/SimpleSAML/Error/NotFound.php',
    'SimpleSAML\\Error\\UnserializableException' => $baseDir . '/src/SimpleSAML/Error/UnserializableException.php',
    'SimpleSAML\\Error\\User' => $baseDir . '/src/SimpleSAML/Error/User.php',
    'SimpleSAML\\Error\\UserAborted' => $baseDir . '/src/SimpleSAML/Error/UserAborted.php',
    'SimpleSAML\\Error\\UserNotFound' => $baseDir . '/src/SimpleSAML/Error/UserNotFound.php',
    'SimpleSAML\\Event\\Dispatcher\\EventDispatcher' => $baseDir . '/src/SimpleSAML/Event/Dispatcher/EventDispatcher.php',
    'SimpleSAML\\Event\\Dispatcher\\ModuleEventDispatcherFactory' => $baseDir . '/src/SimpleSAML/Event/Dispatcher/ModuleEventDispatcherFactory.php',
    'SimpleSAML\\Event\\ExceptionHandlerEvent' => $baseDir . '/src/SimpleSAML/Event/ExceptionHandlerEvent.php',
    'SimpleSAML\\Event\\Provider\\ModuleListenerProvider' => $baseDir . '/src/SimpleSAML/Event/Provider/ModuleListenerProvider.php',
    'SimpleSAML\\HTTP\\RunnableResponse' => $baseDir . '/src/SimpleSAML/HTTP/RunnableResponse.php',
    'SimpleSAML\\IdP' => $baseDir . '/src/SimpleSAML/IdP.php',
    'SimpleSAML\\IdP\\IFrameLogoutHandler' => $baseDir . '/src/SimpleSAML/IdP/IFrameLogoutHandler.php',
    'SimpleSAML\\IdP\\LogoutHandlerInterface' => $baseDir . '/src/SimpleSAML/IdP/LogoutHandlerInterface.php',
    'SimpleSAML\\IdP\\TraditionalLogoutHandler' => $baseDir . '/src/SimpleSAML/IdP/TraditionalLogoutHandler.php',
    'SimpleSAML\\Kernel' => $baseDir . '/src/SimpleSAML/Kernel.php',
    'SimpleSAML\\Locale\\Language' => $baseDir . '/src/SimpleSAML/Locale/Language.php',
    'SimpleSAML\\Locale\\Localization' => $baseDir . '/src/SimpleSAML/Locale/Localization.php',
    'SimpleSAML\\Locale\\Translate' => $baseDir . '/src/SimpleSAML/Locale/Translate.php',
    'SimpleSAML\\Locale\\TwigTranslator' => $baseDir . '/src/SimpleSAML/Locale/TwigTranslator.php',
    'SimpleSAML\\Logger' => $baseDir . '/src/SimpleSAML/Logger.php',
    'SimpleSAML\\Logger\\ErrorLogLoggingHandler' => $baseDir . '/src/SimpleSAML/Logger/ErrorLogLoggingHandler.php',
    'SimpleSAML\\Logger\\FileLoggingHandler' => $baseDir . '/src/SimpleSAML/Logger/FileLoggingHandler.php',
    'SimpleSAML\\Logger\\LoggingHandlerInterface' => $baseDir . '/src/SimpleSAML/Logger/LoggingHandlerInterface.php',
    'SimpleSAML\\Logger\\StandardErrorLoggingHandler' => $baseDir . '/src/SimpleSAML/Logger/StandardErrorLoggingHandler.php',
    'SimpleSAML\\Logger\\SyslogLoggingHandler' => $baseDir . '/src/SimpleSAML/Logger/SyslogLoggingHandler.php',
    'SimpleSAML\\Memcache' => $baseDir . '/src/SimpleSAML/Memcache.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandler' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandler.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandlerDirectory' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandlerDirectory.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandlerFlatFile' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandlerFlatFile.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandlerPdo' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandlerPdo.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandlerSerialize' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandlerSerialize.php',
    'SimpleSAML\\Metadata\\MetaDataStorageHandlerXML' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageHandlerXML.php',
    'SimpleSAML\\Metadata\\MetaDataStorageSource' => $baseDir . '/src/SimpleSAML/Metadata/MetaDataStorageSource.php',
    'SimpleSAML\\Metadata\\SAMLBuilder' => $baseDir . '/src/SimpleSAML/Metadata/SAMLBuilder.php',
    'SimpleSAML\\Metadata\\SAMLParser' => $baseDir . '/src/SimpleSAML/Metadata/SAMLParser.php',
    'SimpleSAML\\Metadata\\Signer' => $baseDir . '/src/SimpleSAML/Metadata/Signer.php',
    'SimpleSAML\\Metadata\\Sources\\MDQ' => $baseDir . '/src/SimpleSAML/Metadata/Sources/MDQ.php',
    'SimpleSAML\\Module' => $baseDir . '/src/SimpleSAML/Module.php',
    'SimpleSAML\\Module\\admin\\Controller\\Config' => $baseDir . '/modules/admin/src/Controller/Config.php',
    'SimpleSAML\\Module\\admin\\Controller\\Federation' => $baseDir . '/modules/admin/src/Controller/Federation.php',
    'SimpleSAML\\Module\\admin\\Controller\\Menu' => $baseDir . '/modules/admin/src/Controller/Menu.php',
    'SimpleSAML\\Module\\admin\\Controller\\Sandbox' => $baseDir . '/modules/admin/src/Controller/Sandbox.php',
    'SimpleSAML\\Module\\admin\\Controller\\Test' => $baseDir . '/modules/admin/src/Controller/Test.php',
    'SimpleSAML\\Module\\admin\\Event\\AdminMenuEvent' => $baseDir . '/modules/admin/src/Event/AdminMenuEvent.php',
    'SimpleSAML\\Module\\admin\\Event\\ConfigPageEvent' => $baseDir . '/modules/admin/src/Event/ConfigPageEvent.php',
    'SimpleSAML\\Module\\admin\\Event\\FederationPageEvent' => $baseDir . '/modules/admin/src/Event/FederationPageEvent.php',
    'SimpleSAML\\Module\\admin\\Event\\SanityCheckEvent' => $baseDir . '/modules/admin/src/Event/SanityCheckEvent.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeAdd' => $baseDir . '/modules/core/src/Auth/Process/AttributeAdd.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeAlter' => $baseDir . '/modules/core/src/Auth/Process/AttributeAlter.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeConditionalAdd' => $baseDir . '/modules/core/src/Auth/Process/AttributeConditionalAdd.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeCopy' => $baseDir . '/modules/core/src/Auth/Process/AttributeCopy.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeDump' => $baseDir . '/modules/core/src/Auth/Process/AttributeDump.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeLimit' => $baseDir . '/modules/core/src/Auth/Process/AttributeLimit.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeMap' => $baseDir . '/modules/core/src/Auth/Process/AttributeMap.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\AttributeValueMap' => $baseDir . '/modules/core/src/Auth/Process/AttributeValueMap.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\Cardinality' => $baseDir . '/modules/core/src/Auth/Process/Cardinality.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\CardinalitySingle' => $baseDir . '/modules/core/src/Auth/Process/CardinalitySingle.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\ExtendIdPSession' => $baseDir . '/modules/core/src/Auth/Process/ExtendIdPSession.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\GenerateGroups' => $baseDir . '/modules/core/src/Auth/Process/GenerateGroups.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\LanguageAdaptor' => $baseDir . '/modules/core/src/Auth/Process/LanguageAdaptor.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\PHP' => $baseDir . '/modules/core/src/Auth/Process/PHP.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\ScopeAttribute' => $baseDir . '/modules/core/src/Auth/Process/ScopeAttribute.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\ScopeFromAttribute' => $baseDir . '/modules/core/src/Auth/Process/ScopeFromAttribute.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\TargetedID' => $baseDir . '/modules/core/src/Auth/Process/TargetedID.php',
    'SimpleSAML\\Module\\core\\Auth\\Process\\WarnShortSSOInterval' => $baseDir . '/modules/core/src/Auth/Process/WarnShortSSOInterval.php',
    'SimpleSAML\\Module\\core\\Auth\\Source\\AbstractSourceSelector' => $baseDir . '/modules/core/src/Auth/Source/AbstractSourceSelector.php',
    'SimpleSAML\\Module\\core\\Auth\\Source\\AdminPassword' => $baseDir . '/modules/core/src/Auth/Source/AdminPassword.php',
    'SimpleSAML\\Module\\core\\Auth\\Source\\RequestedAuthnContextSelector' => $baseDir . '/modules/core/src/Auth/Source/RequestedAuthnContextSelector.php',
    'SimpleSAML\\Module\\core\\Auth\\Source\\SourceIPSelector' => $baseDir . '/modules/core/src/Auth/Source/SourceIPSelector.php',
    'SimpleSAML\\Module\\core\\Auth\\UserPassBase' => $baseDir . '/modules/core/src/Auth/UserPassBase.php',
    'SimpleSAML\\Module\\core\\Auth\\UserPassOrgBase' => $baseDir . '/modules/core/src/Auth/UserPassOrgBase.php',
    'SimpleSAML\\Module\\core\\Controller\\ErrorReport' => $baseDir . '/modules/core/src/Controller/ErrorReport.php',
    'SimpleSAML\\Module\\core\\Controller\\Exception' => $baseDir . '/modules/core/src/Controller/Exception.php',
    'SimpleSAML\\Module\\core\\Controller\\Login' => $baseDir . '/modules/core/src/Controller/Login.php',
    'SimpleSAML\\Module\\core\\Controller\\Logout' => $baseDir . '/modules/core/src/Controller/Logout.php',
    'SimpleSAML\\Module\\core\\Controller\\Redirection' => $baseDir . '/modules/core/src/Controller/Redirection.php',
    'SimpleSAML\\Module\\core\\Stats\\Output\\File' => $baseDir . '/modules/core/src/Stats/Output/File.php',
    'SimpleSAML\\Module\\core\\Stats\\Output\\Log' => $baseDir . '/modules/core/src/Stats/Output/Log.php',
    'SimpleSAML\\Module\\core\\Storage\\SQLPermanentStorage' => $baseDir . '/modules/core/src/Storage/SQLPermanentStorage.php',
    'SimpleSAML\\Module\\cron\\Controller\\Cron' => $baseDir . '/modules/cron/src/Controller/Cron.php',
    'SimpleSAML\\Module\\cron\\Cron' => $baseDir . '/modules/cron/src/Cron.php',
    'SimpleSAML\\Module\\cron\\Event\\CronEvent' => $baseDir . '/modules/cron/src/Event/CronEvent.php',
    'SimpleSAML\\Module\\cron\\Event\\Listener\\ConfigPageListener' => $baseDir . '/modules/cron/src/Event/Listener/ConfigPageListener.php',
    'SimpleSAML\\Module\\cron\\Event\\Listener\\CronEventListener' => $baseDir . '/modules/cron/src/Event/Listener/CronEventListener.php',
    'SimpleSAML\\Module\\debugsp\\Controller\\Test' => $baseDir . '/modules/debugsp/src/Controller/Test.php',
    'SimpleSAML\\Module\\exampleauth\\Auth\\Process\\RedirectTest' => $baseDir . '/modules/exampleauth/src/Auth/Process/RedirectTest.php',
    'SimpleSAML\\Module\\exampleauth\\Auth\\Source\\External' => $baseDir . '/modules/exampleauth/src/Auth/Source/External.php',
    'SimpleSAML\\Module\\exampleauth\\Auth\\Source\\StaticSource' => $baseDir . '/modules/exampleauth/src/Auth/Source/StaticSource.php',
    'SimpleSAML\\Module\\exampleauth\\Auth\\Source\\UserClick' => $baseDir . '/modules/exampleauth/src/Auth/Source/UserClick.php',
    'SimpleSAML\\Module\\exampleauth\\Auth\\Source\\UserPass' => $baseDir . '/modules/exampleauth/src/Auth/Source/UserPass.php',
    'SimpleSAML\\Module\\exampleauth\\Controller\\ExampleAuth' => $baseDir . '/modules/exampleauth/src/Controller/ExampleAuth.php',
    'SimpleSAML\\Module\\exampleauth\\Controller\\ProfileAuth' => $baseDir . '/modules/exampleauth/src/Controller/ProfileAuth.php',
    'SimpleSAML\\Module\\multiauth\\Auth\\Source\\MultiAuth' => $baseDir . '/modules/multiauth/src/Auth/Source/MultiAuth.php',
    'SimpleSAML\\Module\\multiauth\\Controller\\DiscoController' => $baseDir . '/modules/multiauth/src/Controller/DiscoController.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\AttributeNameID' => $baseDir . '/modules/saml/src/Auth/Process/AttributeNameID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\AuthnContextClassRef' => $baseDir . '/modules/saml/src/Auth/Process/AuthnContextClassRef.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\ExpectedAuthnContextClassRef' => $baseDir . '/modules/saml/src/Auth/Process/ExpectedAuthnContextClassRef.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\FilterScopes' => $baseDir . '/modules/saml/src/Auth/Process/FilterScopes.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\NameIDAttribute' => $baseDir . '/modules/saml/src/Auth/Process/NameIDAttribute.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\PairwiseID' => $baseDir . '/modules/saml/src/Auth/Process/PairwiseID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\PersistentNameID' => $baseDir . '/modules/saml/src/Auth/Process/PersistentNameID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\PersistentNameID2TargetedID' => $baseDir . '/modules/saml/src/Auth/Process/PersistentNameID2TargetedID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\SQLPersistentNameID' => $baseDir . '/modules/saml/src/Auth/Process/SQLPersistentNameID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\ScopedIssuer' => $baseDir . '/modules/saml/src/Auth/Process/ScopedIssuer.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\SubjectID' => $baseDir . '/modules/saml/src/Auth/Process/SubjectID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Process\\TransientNameID' => $baseDir . '/modules/saml/src/Auth/Process/TransientNameID.php',
    'SimpleSAML\\Module\\saml\\Auth\\Source\\SP' => $baseDir . '/modules/saml/src/Auth/Source/SP.php',
    'SimpleSAML\\Module\\saml\\BaseNameIDGenerator' => $baseDir . '/modules/saml/src/BaseNameIDGenerator.php',
    'SimpleSAML\\Module\\saml\\Controller\\Disco' => $baseDir . '/modules/saml/src/Controller/Disco.php',
    'SimpleSAML\\Module\\saml\\Controller\\Exception' => $baseDir . '/modules/saml/src/Controller/Exception.php',
    'SimpleSAML\\Module\\saml\\Controller\\Metadata' => $baseDir . '/modules/saml/src/Controller/Metadata.php',
    'SimpleSAML\\Module\\saml\\Controller\\Proxy' => $baseDir . '/modules/saml/src/Controller/Proxy.php',
    'SimpleSAML\\Module\\saml\\Controller\\ServiceProvider' => $baseDir . '/modules/saml/src/Controller/ServiceProvider.php',
    'SimpleSAML\\Module\\saml\\Controller\\SingleLogout' => $baseDir . '/modules/saml/src/Controller/SingleLogout.php',
    'SimpleSAML\\Module\\saml\\Controller\\WebBrowserSingleSignOn' => $baseDir . '/modules/saml/src/Controller/WebBrowserSingleSignOn.php',
    'SimpleSAML\\Module\\saml\\Error' => $baseDir . '/modules/saml/src/Error.php',
    'SimpleSAML\\Module\\saml\\Error\\NoAuthnContext' => $baseDir . '/modules/saml/src/Error/NoAuthnContext.php',
    'SimpleSAML\\Module\\saml\\Error\\NoAvailableIDP' => $baseDir . '/modules/saml/src/Error/NoAvailableIDP.php',
    'SimpleSAML\\Module\\saml\\Error\\NoPassive' => $baseDir . '/modules/saml/src/Error/NoPassive.php',
    'SimpleSAML\\Module\\saml\\Error\\NoSupportedIDP' => $baseDir . '/modules/saml/src/Error/NoSupportedIDP.php',
    'SimpleSAML\\Module\\saml\\Error\\ProxyCountExceeded' => $baseDir . '/modules/saml/src/Error/ProxyCountExceeded.php',
    'SimpleSAML\\Module\\saml\\Event\\Listener\\SamlSanityCheckListener' => $baseDir . '/modules/saml/src/Event/Listener/SamlSanityCheckListener.php',
    'SimpleSAML\\Module\\saml\\IdP\\SAML2' => $baseDir . '/modules/saml/src/IdP/SAML2.php',
    'SimpleSAML\\Module\\saml\\IdP\\SQLNameID' => $baseDir . '/modules/saml/src/IdP/SQLNameID.php',
    'SimpleSAML\\Module\\saml\\Message' => $baseDir . '/modules/saml/src/Message.php',
    'SimpleSAML\\Module\\saml\\SP\\LogoutStore' => $baseDir . '/modules/saml/src/SP/LogoutStore.php',
    'SimpleSAML\\SAML2\\Assert\\Assert' => $vendorDir . '/simplesamlphp/saml2/src/Assert/Assert.php',
    'SimpleSAML\\SAML2\\Assert\\CIDRTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/CIDRTrait.php',
    'SimpleSAML\\SAML2\\Assert\\DomainTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/DomainTrait.php',
    'SimpleSAML\\SAML2\\Assert\\EntityIDTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/EntityIDTrait.php',
    'SimpleSAML\\SAML2\\Assert\\GeolocationTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/GeolocationTrait.php',
    'SimpleSAML\\SAML2\\Assert\\RelayStateTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/RelayStateTrait.php',
    'SimpleSAML\\SAML2\\Assert\\SAMLAnyURITrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/SAMLAnyURITrait.php',
    'SimpleSAML\\SAML2\\Assert\\SAMLDateTimeTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/SAMLDateTimeTrait.php',
    'SimpleSAML\\SAML2\\Assert\\SAMLStringTrait' => $vendorDir . '/simplesamlphp/saml2/src/Assert/SAMLStringTrait.php',
    'SimpleSAML\\SAML2\\Assertion\\Decrypter' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Decrypter.php',
    'SimpleSAML\\SAML2\\Assertion\\Exception\\InvalidAssertionException' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Exception/InvalidAssertionException.php',
    'SimpleSAML\\SAML2\\Assertion\\Exception\\InvalidSubjectConfirmationException' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Exception/InvalidSubjectConfirmationException.php',
    'SimpleSAML\\SAML2\\Assertion\\Exception\\NotDecryptedException' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Exception/NotDecryptedException.php',
    'SimpleSAML\\SAML2\\Assertion\\Processor' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Processor.php',
    'SimpleSAML\\SAML2\\Assertion\\ProcessorBuilder' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/ProcessorBuilder.php',
    'SimpleSAML\\SAML2\\Assertion\\Transformer\\NameIdDecryptionTransformer' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Transformer/NameIdDecryptionTransformer.php',
    'SimpleSAML\\SAML2\\Assertion\\Transformer\\TransformerChain' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Transformer/TransformerChain.php',
    'SimpleSAML\\SAML2\\Assertion\\Transformer\\TransformerInterface' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Transformer/TransformerInterface.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\AssertionConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/AssertionConstraintValidator.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\AssertionValidator' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/AssertionValidator.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\NotBefore' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/NotBefore.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\NotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/NotOnOrAfter.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SessionNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SessionNotOnOrAfter.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SpIsValidAudience' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SpIsValidAudience.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationMethod' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SubjectConfirmationMethod.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotBefore' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotBefore.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotOnOrAfter.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationRecipientMatches' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SubjectConfirmationRecipientMatches.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationResponseToMatches' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/ConstraintValidator/SubjectConfirmationResponseToMatches.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/Result.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\SubjectConfirmationConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/SubjectConfirmationConstraintValidator.php',
    'SimpleSAML\\SAML2\\Assertion\\Validation\\SubjectConfirmationValidator' => $vendorDir . '/simplesamlphp/saml2/src/Assertion/Validation/SubjectConfirmationValidator.php',
    'SimpleSAML\\SAML2\\Binding' => $vendorDir . '/simplesamlphp/saml2/src/Binding.php',
    'SimpleSAML\\SAML2\\Binding\\AsynchronousBindingInterface' => $vendorDir . '/simplesamlphp/saml2/src/Binding/AsynchronousBindingInterface.php',
    'SimpleSAML\\SAML2\\Binding\\HTTPArtifact' => $vendorDir . '/simplesamlphp/saml2/src/Binding/HTTPArtifact.php',
    'SimpleSAML\\SAML2\\Binding\\HTTPPost' => $vendorDir . '/simplesamlphp/saml2/src/Binding/HTTPPost.php',
    'SimpleSAML\\SAML2\\Binding\\HTTPRedirect' => $vendorDir . '/simplesamlphp/saml2/src/Binding/HTTPRedirect.php',
    'SimpleSAML\\SAML2\\Binding\\RelayStateInterface' => $vendorDir . '/simplesamlphp/saml2/src/Binding/RelayStateInterface.php',
    'SimpleSAML\\SAML2\\Binding\\RelayStateTrait' => $vendorDir . '/simplesamlphp/saml2/src/Binding/RelayStateTrait.php',
    'SimpleSAML\\SAML2\\Binding\\SOAP' => $vendorDir . '/simplesamlphp/saml2/src/Binding/SOAP.php',
    'SimpleSAML\\SAML2\\Binding\\SynchronousBindingInterface' => $vendorDir . '/simplesamlphp/saml2/src/Binding/SynchronousBindingInterface.php',
    'SimpleSAML\\SAML2\\Certificate\\Exception\\InvalidCertificateStructureException' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/Exception/InvalidCertificateStructureException.php',
    'SimpleSAML\\SAML2\\Certificate\\Exception\\InvalidKeyUsageException' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/Exception/InvalidKeyUsageException.php',
    'SimpleSAML\\SAML2\\Certificate\\Exception\\NoKeysFoundException' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/Exception/NoKeysFoundException.php',
    'SimpleSAML\\SAML2\\Certificate\\Key' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/Key.php',
    'SimpleSAML\\SAML2\\Certificate\\KeyCollection' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/KeyCollection.php',
    'SimpleSAML\\SAML2\\Certificate\\KeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/KeyLoader.php',
    'SimpleSAML\\SAML2\\Certificate\\PrivateKeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/PrivateKeyLoader.php',
    'SimpleSAML\\SAML2\\Certificate\\X509' => $vendorDir . '/simplesamlphp/saml2/src/Certificate/X509.php',
    'SimpleSAML\\SAML2\\Compat\\AbstractContainer' => $vendorDir . '/simplesamlphp/saml2/src/Compat/AbstractContainer.php',
    'SimpleSAML\\SAML2\\Compat\\ContainerSingleton' => $vendorDir . '/simplesamlphp/saml2/src/Compat/ContainerSingleton.php',
    'SimpleSAML\\SAML2\\Compat\\MockContainer' => $vendorDir . '/simplesamlphp/saml2/src/Compat/MockContainer.php',
    'SimpleSAML\\SAML2\\Configuration\\ArrayAdapter' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/ArrayAdapter.php',
    'SimpleSAML\\SAML2\\Configuration\\CertificateProvider' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/CertificateProvider.php',
    'SimpleSAML\\SAML2\\Configuration\\DecryptionProvider' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/DecryptionProvider.php',
    'SimpleSAML\\SAML2\\Configuration\\Destination' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/Destination.php',
    'SimpleSAML\\SAML2\\Configuration\\EntityIdProvider' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/EntityIdProvider.php',
    'SimpleSAML\\SAML2\\Configuration\\IdentityProvider' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/IdentityProvider.php',
    'SimpleSAML\\SAML2\\Configuration\\IdentityProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/IdentityProviderAware.php',
    'SimpleSAML\\SAML2\\Configuration\\PrivateKey' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/PrivateKey.php',
    'SimpleSAML\\SAML2\\Configuration\\Queryable' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/Queryable.php',
    'SimpleSAML\\SAML2\\Configuration\\ServiceProvider' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/ServiceProvider.php',
    'SimpleSAML\\SAML2\\Configuration\\ServiceProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/Configuration/ServiceProviderAware.php',
    'SimpleSAML\\SAML2\\Constants' => $vendorDir . '/simplesamlphp/saml2/src/Constants.php',
    'SimpleSAML\\SAML2\\Exception\\ArrayValidationException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/ArrayValidationException.php',
    'SimpleSAML\\SAML2\\Exception\\InvalidArgumentException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/InvalidArgumentException.php',
    'SimpleSAML\\SAML2\\Exception\\MetadataExpiredException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/MetadataExpiredException.php',
    'SimpleSAML\\SAML2\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/ProtocolViolationException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\AuthnFailedException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/AuthnFailedException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\InvalidAttrNameOrValueException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/InvalidAttrNameOrValueException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\InvalidNameIDPolicyException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/InvalidNameIDPolicyException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\NoAuthnContextException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/NoAuthnContextException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\NoAvailableIDPException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/NoAvailableIDPException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\NoPassiveException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/NoPassiveException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\NoSupportedIDPException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/NoSupportedIDPException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\PartialLogoutException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/PartialLogoutException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\ProxyCountExceededException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/ProxyCountExceededException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\RequestDeniedException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/RequestDeniedException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\RequestUnsupportedException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/RequestUnsupportedException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\RequestVersionDeprecatedException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/RequestVersionDeprecatedException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\RequestVersionTooHighException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/RequestVersionTooHighException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\RequestVersionTooLowException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/RequestVersionTooLowException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\ResourceNotRecognizedException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/ResourceNotRecognizedException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\TooManyResponsesException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/TooManyResponsesException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\UnknownAttrProfileException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/UnknownAttrProfileException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\UnknownPrincipalException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/UnknownPrincipalException.php',
    'SimpleSAML\\SAML2\\Exception\\Protocol\\UnsupportedBindingException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/Protocol/UnsupportedBindingException.php',
    'SimpleSAML\\SAML2\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/saml2/src/Exception/RuntimeException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\InvalidResponseException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/InvalidResponseException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\InvalidSignatureException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/InvalidSignatureException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\NoAssertionsFoundException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/NoAssertionsFoundException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\PreconditionNotMetException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/PreconditionNotMetException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\UnencryptedAssertionFoundException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/UnencryptedAssertionFoundException.php',
    'SimpleSAML\\SAML2\\Response\\Exception\\UnsignedResponseException' => $vendorDir . '/simplesamlphp/saml2/src/Response/Exception/UnsignedResponseException.php',
    'SimpleSAML\\SAML2\\Response\\Processor' => $vendorDir . '/simplesamlphp/saml2/src/Response/Processor.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\ConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/ConstraintValidator.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\ConstraintValidator\\DestinationMatches' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/ConstraintValidator/DestinationMatches.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\ConstraintValidator\\IsSuccessful' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/ConstraintValidator/IsSuccessful.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\PreconditionValidator' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/PreconditionValidator.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/Result.php',
    'SimpleSAML\\SAML2\\Response\\Validation\\Validator' => $vendorDir . '/simplesamlphp/saml2/src/Response/Validation/Validator.php',
    'SimpleSAML\\SAML2\\SOAPClient' => $vendorDir . '/simplesamlphp/saml2/src/SOAPClient.php',
    'SimpleSAML\\SAML2\\Signature\\AbstractChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/Signature/AbstractChainedValidator.php',
    'SimpleSAML\\SAML2\\Signature\\ChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/Signature/ChainedValidator.php',
    'SimpleSAML\\SAML2\\Signature\\MissingConfigurationException' => $vendorDir . '/simplesamlphp/saml2/src/Signature/MissingConfigurationException.php',
    'SimpleSAML\\SAML2\\Signature\\PublicKeyValidator' => $vendorDir . '/simplesamlphp/saml2/src/Signature/PublicKeyValidator.php',
    'SimpleSAML\\SAML2\\Signature\\Validator' => $vendorDir . '/simplesamlphp/saml2/src/Signature/Validator.php',
    'SimpleSAML\\SAML2\\Signature\\ValidatorChain' => $vendorDir . '/simplesamlphp/saml2/src/Signature/ValidatorChain.php',
    'SimpleSAML\\SAML2\\Signature\\ValidatorInterface' => $vendorDir . '/simplesamlphp/saml2/src/Signature/ValidatorInterface.php',
    'SimpleSAML\\SAML2\\Type\\AuthnContextComparisonTypeValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/AuthnContextComparisonTypeValue.php',
    'SimpleSAML\\SAML2\\Type\\CIDRValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/CIDRValue.php',
    'SimpleSAML\\SAML2\\Type\\DecisionTypeValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/DecisionTypeValue.php',
    'SimpleSAML\\SAML2\\Type\\DomainValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/DomainValue.php',
    'SimpleSAML\\SAML2\\Type\\EmailAddressValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/EmailAddressValue.php',
    'SimpleSAML\\SAML2\\Type\\EntityIDValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/EntityIDValue.php',
    'SimpleSAML\\SAML2\\Type\\GeolocationValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/GeolocationValue.php',
    'SimpleSAML\\SAML2\\Type\\KeyTypesValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/KeyTypesValue.php',
    'SimpleSAML\\SAML2\\Type\\ListOfStringsValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/ListOfStringsValue.php',
    'SimpleSAML\\SAML2\\Type\\SAMLAnyURIListValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/SAMLAnyURIListValue.php',
    'SimpleSAML\\SAML2\\Type\\SAMLAnyURIValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/SAMLAnyURIValue.php',
    'SimpleSAML\\SAML2\\Type\\SAMLDateTimeValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/SAMLDateTimeValue.php',
    'SimpleSAML\\SAML2\\Type\\SAMLStringValue' => $vendorDir . '/simplesamlphp/saml2/src/Type/SAMLStringValue.php',
    'SimpleSAML\\SAML2\\Utilities\\ArrayCollection' => $vendorDir . '/simplesamlphp/saml2/src/Utilities/ArrayCollection.php',
    'SimpleSAML\\SAML2\\Utilities\\Collection' => $vendorDir . '/simplesamlphp/saml2/src/Utilities/Collection.php',
    'SimpleSAML\\SAML2\\Utilities\\File' => $vendorDir . '/simplesamlphp/saml2/src/Utilities/File.php',
    'SimpleSAML\\SAML2\\Utils' => $vendorDir . '/simplesamlphp/saml2/src/Utils.php',
    'SimpleSAML\\SAML2\\Utils\\XPath' => $vendorDir . '/simplesamlphp/saml2/src/Utils/XPath.php',
    'SimpleSAML\\SAML2\\XML\\CanonicalizableElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/CanonicalizableElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\EncryptableElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/EncryptableElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\EncryptedElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/EncryptedElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\ExtendableElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/ExtendableElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\ExtensionPointInterface' => $vendorDir . '/simplesamlphp/saml2/src/XML/ExtensionPointInterface.php',
    'SimpleSAML\\SAML2\\XML\\ExtensionPointTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/ExtensionPointTrait.php',
    'SimpleSAML\\SAML2\\XML\\ExtensionsTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/ExtensionsTrait.php',
    'SimpleSAML\\SAML2\\XML\\IdentifierTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/IdentifierTrait.php',
    'SimpleSAML\\SAML2\\XML\\SignableElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/SignableElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\SignedElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/SignedElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\alg\\AbstractAlgElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/alg/AbstractAlgElement.php',
    'SimpleSAML\\SAML2\\XML\\alg\\DigestMethod' => $vendorDir . '/simplesamlphp/saml2/src/XML/alg/DigestMethod.php',
    'SimpleSAML\\SAML2\\XML\\alg\\SigningMethod' => $vendorDir . '/simplesamlphp/saml2/src/XML/alg/SigningMethod.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\AbstractEcpElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/AbstractEcpElement.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\RelayState' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/RelayState.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\Request' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/Request.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\RequestAuthenticated' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/RequestAuthenticated.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\Response' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/Response.php',
    'SimpleSAML\\SAML2\\XML\\ecp\\SubjectConfirmation' => $vendorDir . '/simplesamlphp/saml2/src/XML/ecp/SubjectConfirmation.php',
    'SimpleSAML\\SAML2\\XML\\emd\\AbstractEmdElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/emd/AbstractEmdElement.php',
    'SimpleSAML\\SAML2\\XML\\emd\\RepublishRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/emd/RepublishRequest.php',
    'SimpleSAML\\SAML2\\XML\\emd\\RepublishTarget' => $vendorDir . '/simplesamlphp/saml2/src/XML/emd/RepublishTarget.php',
    'SimpleSAML\\SAML2\\XML\\idpdisc\\DiscoveryResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/idpdisc/DiscoveryResponse.php',
    'SimpleSAML\\SAML2\\XML\\init\\RequestInitiator' => $vendorDir . '/simplesamlphp/saml2/src/XML/init/RequestInitiator.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractEndpointType' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractEndpointType.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractIndexedEndpointType' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractIndexedEndpointType.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractLocalizedName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractLocalizedName.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractLocalizedURI' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractLocalizedURI.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractMdElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractMdElement.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractMetadataDocument' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractMetadataDocument.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractRoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractRoleDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractRoleDescriptorType' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractRoleDescriptorType.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractSSODescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\AbstractSignedMdElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AbstractSignedMdElement.php',
    'SimpleSAML\\SAML2\\XML\\md\\AdditionalMetadataLocation' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AdditionalMetadataLocation.php',
    'SimpleSAML\\SAML2\\XML\\md\\AffiliateMember' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AffiliateMember.php',
    'SimpleSAML\\SAML2\\XML\\md\\AffiliationDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AffiliationDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\ArtifactResolutionService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/ArtifactResolutionService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AssertionConsumerService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AssertionConsumerService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AssertionIDRequestService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AssertionIDRequestService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AttributeAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AttributeAuthorityDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\AttributeConsumingService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AttributeConsumingService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AttributeProfile' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AttributeProfile.php',
    'SimpleSAML\\SAML2\\XML\\md\\AttributeService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AttributeService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AuthnAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AuthnAuthorityDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\AuthnQueryService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AuthnQueryService.php',
    'SimpleSAML\\SAML2\\XML\\md\\AuthzService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/AuthzService.php',
    'SimpleSAML\\SAML2\\XML\\md\\Company' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/Company.php',
    'SimpleSAML\\SAML2\\XML\\md\\ContactPerson' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/ContactPerson.php',
    'SimpleSAML\\SAML2\\XML\\md\\EmailAddress' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/EmailAddress.php',
    'SimpleSAML\\SAML2\\XML\\md\\EncryptionMethod' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/EncryptionMethod.php',
    'SimpleSAML\\SAML2\\XML\\md\\EntitiesDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/EntitiesDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\EntityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/EntityDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\Extensions' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/Extensions.php',
    'SimpleSAML\\SAML2\\XML\\md\\GivenName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/GivenName.php',
    'SimpleSAML\\SAML2\\XML\\md\\IDPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/IDPSSODescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\IndexedElementTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/IndexedElementTrait.php',
    'SimpleSAML\\SAML2\\XML\\md\\KeyDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/KeyDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\KeyTypesEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/KeyTypesEnum.php',
    'SimpleSAML\\SAML2\\XML\\md\\ManageNameIDService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/ManageNameIDService.php',
    'SimpleSAML\\SAML2\\XML\\md\\NameIDFormat' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/NameIDFormat.php',
    'SimpleSAML\\SAML2\\XML\\md\\NameIDMappingService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/NameIDMappingService.php',
    'SimpleSAML\\SAML2\\XML\\md\\Organization' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/Organization.php',
    'SimpleSAML\\SAML2\\XML\\md\\OrganizationDisplayName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/OrganizationDisplayName.php',
    'SimpleSAML\\SAML2\\XML\\md\\OrganizationName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/OrganizationName.php',
    'SimpleSAML\\SAML2\\XML\\md\\OrganizationURL' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/OrganizationURL.php',
    'SimpleSAML\\SAML2\\XML\\md\\PDPDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/PDPDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\RequestedAttribute' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/RequestedAttribute.php',
    'SimpleSAML\\SAML2\\XML\\md\\SPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/SPSSODescriptor.php',
    'SimpleSAML\\SAML2\\XML\\md\\ServiceDescription' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/ServiceDescription.php',
    'SimpleSAML\\SAML2\\XML\\md\\ServiceName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/ServiceName.php',
    'SimpleSAML\\SAML2\\XML\\md\\SingleLogoutService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/SingleLogoutService.php',
    'SimpleSAML\\SAML2\\XML\\md\\SingleSignOnService' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/SingleSignOnService.php',
    'SimpleSAML\\SAML2\\XML\\md\\SurName' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/SurName.php',
    'SimpleSAML\\SAML2\\XML\\md\\TelephoneNumber' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/TelephoneNumber.php',
    'SimpleSAML\\SAML2\\XML\\md\\UnknownRoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/XML/md/UnknownRoleDescriptor.php',
    'SimpleSAML\\SAML2\\XML\\mdattr\\AbstractMdattrElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdattr/AbstractMdattrElement.php',
    'SimpleSAML\\SAML2\\XML\\mdattr\\EntityAttributes' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdattr/EntityAttributes.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\AbstractMdrpiElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/AbstractMdrpiElement.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\Publication' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/Publication.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\PublicationInfo' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/PublicationInfo.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\PublicationPath' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/PublicationPath.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\RegistrationInfo' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/RegistrationInfo.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\RegistrationPolicy' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/RegistrationPolicy.php',
    'SimpleSAML\\SAML2\\XML\\mdrpi\\UsagePolicy' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdrpi/UsagePolicy.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\AbstractMduiElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/AbstractMduiElement.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\Description' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/Description.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\DiscoHints' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/DiscoHints.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\DisplayName' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/DisplayName.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\DomainHint' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/DomainHint.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\GeolocationHint' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/GeolocationHint.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\IPHint' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/IPHint.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\InformationURL' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/InformationURL.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\Keywords' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/Keywords.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\Logo' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/Logo.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\PrivacyStatementURL' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/PrivacyStatementURL.php',
    'SimpleSAML\\SAML2\\XML\\mdui\\UIInfo' => $vendorDir . '/simplesamlphp/saml2/src/XML/mdui/UIInfo.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractBaseID' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractBaseID.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractBaseIDType' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractBaseIDType.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractCondition' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractCondition.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractConditionType' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractConditionType.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractEncryptedElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractEncryptedElement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractKeyInfoConfirmationData' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractKeyInfoConfirmationData.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractSamlElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractSamlElement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractStatement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractStatement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractStatementType' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractStatementType.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AbstractSubjectConfirmationData' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AbstractSubjectConfirmationData.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Action' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Action.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Advice' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Advice.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Assertion' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Assertion.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AssertionIDRef' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AssertionIDRef.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AssertionURIRef' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AssertionURIRef.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Attribute' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Attribute.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AttributeStatement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AttributeStatement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AttributeValue' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AttributeValue.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Audience' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Audience.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AudienceRestriction' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AudienceRestriction.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthenticatingAuthority' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthenticatingAuthority.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthnContext' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthnContext.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthnContextClassRef' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthnContextClassRef.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthnContextDecl' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthnContextDecl.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthnContextDeclRef' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthnContextDeclRef.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthnStatement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthnStatement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\AuthzDecisionStatement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/AuthzDecisionStatement.php',
    'SimpleSAML\\SAML2\\XML\\saml\\BaseIdentifierInterface' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/BaseIdentifierInterface.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Conditions' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Conditions.php',
    'SimpleSAML\\SAML2\\XML\\saml\\DecisionTypeEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/DecisionTypeEnum.php',
    'SimpleSAML\\SAML2\\XML\\saml\\EncryptedAssertion' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/EncryptedAssertion.php',
    'SimpleSAML\\SAML2\\XML\\saml\\EncryptedAttribute' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/EncryptedAttribute.php',
    'SimpleSAML\\SAML2\\XML\\saml\\EncryptedID' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/EncryptedID.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Evidence' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Evidence.php',
    'SimpleSAML\\SAML2\\XML\\saml\\GHPPEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/GHPPEnum.php',
    'SimpleSAML\\SAML2\\XML\\saml\\IDNameQualifiersTrait' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/IDNameQualifiersTrait.php',
    'SimpleSAML\\SAML2\\XML\\saml\\IdentifierInterface' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/IdentifierInterface.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Issuer' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Issuer.php',
    'SimpleSAML\\SAML2\\XML\\saml\\NameID' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/NameID.php',
    'SimpleSAML\\SAML2\\XML\\saml\\NameIDType' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/NameIDType.php',
    'SimpleSAML\\SAML2\\XML\\saml\\OneTimeUse' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/OneTimeUse.php',
    'SimpleSAML\\SAML2\\XML\\saml\\ProxyRestriction' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/ProxyRestriction.php',
    'SimpleSAML\\SAML2\\XML\\saml\\RWEDCEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/RWEDCEnum.php',
    'SimpleSAML\\SAML2\\XML\\saml\\RWEDCNegationEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/RWEDCNegationEnum.php',
    'SimpleSAML\\SAML2\\XML\\saml\\Subject' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/Subject.php',
    'SimpleSAML\\SAML2\\XML\\saml\\SubjectConfirmation' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/SubjectConfirmation.php',
    'SimpleSAML\\SAML2\\XML\\saml\\SubjectConfirmationData' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/SubjectConfirmationData.php',
    'SimpleSAML\\SAML2\\XML\\saml\\SubjectLocality' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/SubjectLocality.php',
    'SimpleSAML\\SAML2\\XML\\saml\\UnknownCondition' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/UnknownCondition.php',
    'SimpleSAML\\SAML2\\XML\\saml\\UnknownID' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/UnknownID.php',
    'SimpleSAML\\SAML2\\XML\\saml\\UnknownStatement' => $vendorDir . '/simplesamlphp/saml2/src/XML/saml/UnknownStatement.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractManageNameIDRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractManageNameIDRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractMessage' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractMessage.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractNameIDMappingRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractNameIDMappingRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractSamlpElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractSamlpElement.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractStatusResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractStatusResponse.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AbstractSubjectQuery' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AbstractSubjectQuery.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Artifact' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Artifact.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\ArtifactResolve' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/ArtifactResolve.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\ArtifactResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/ArtifactResponse.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AssertionIDRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AssertionIDRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AttributeQuery' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AttributeQuery.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AuthnContextComparisonTypeEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AuthnContextComparisonTypeEnum.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AuthnQuery' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AuthnQuery.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AuthnRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AuthnRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\AuthzDecisionQuery' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/AuthzDecisionQuery.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Extensions' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Extensions.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\GetComplete' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/GetComplete.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\IDPEntry' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/IDPEntry.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\IDPList' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/IDPList.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\LogoutRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/LogoutRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\LogoutResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/LogoutResponse.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\ManageNameIDRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/ManageNameIDRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\ManageNameIDResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/ManageNameIDResponse.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\MessageFactory' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/MessageFactory.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\NameIDMappingRequest' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/NameIDMappingRequest.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\NameIDMappingResponse' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/NameIDMappingResponse.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\NameIDPolicy' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/NameIDPolicy.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\NewEncryptedID' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/NewEncryptedID.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\NewID' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/NewID.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\RequestedAuthnContext' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/RequestedAuthnContext.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\RequesterID' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/RequesterID.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Response' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Response.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Scoping' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Scoping.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\SessionIndex' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/SessionIndex.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Status' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Status.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\StatusCode' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/StatusCode.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\StatusDetail' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/StatusDetail.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\StatusMessage' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/StatusMessage.php',
    'SimpleSAML\\SAML2\\XML\\samlp\\Terminate' => $vendorDir . '/simplesamlphp/saml2/src/XML/samlp/Terminate.php',
    'SimpleSAML\\SAML2\\XML\\shibmd\\AbstractShibmdElement' => $vendorDir . '/simplesamlphp/saml2/src/XML/shibmd/AbstractShibmdElement.php',
    'SimpleSAML\\SAML2\\XML\\shibmd\\KeyAuthority' => $vendorDir . '/simplesamlphp/saml2/src/XML/shibmd/KeyAuthority.php',
    'SimpleSAML\\SAML2\\XML\\shibmd\\Scope' => $vendorDir . '/simplesamlphp/saml2/src/XML/shibmd/Scope.php',
    'SimpleSAML\\SAML2\\XML\\subjectid\\SignalEnum' => $vendorDir . '/simplesamlphp/saml2/src/XML/subjectid/SignalEnum.php',
    'SimpleSAML\\SOAP11\\Assert\\Assert' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Assert/Assert.php',
    'SimpleSAML\\SOAP11\\Assert\\MustUnderstandTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Assert/MustUnderstandTrait.php',
    'SimpleSAML\\SOAP11\\Constants' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Constants.php',
    'SimpleSAML\\SOAP11\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Exception/ProtocolViolationException.php',
    'SimpleSAML\\SOAP11\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Exception/RuntimeException.php',
    'SimpleSAML\\SOAP11\\Type\\ActorValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Type/ActorValue.php',
    'SimpleSAML\\SOAP11\\Type\\EncodingStyleValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Type/EncodingStyleValue.php',
    'SimpleSAML\\SOAP11\\Type\\MustUnderstandValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Type/MustUnderstandValue.php',
    'SimpleSAML\\SOAP11\\Utils\\XPath' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/Utils/XPath.php',
    'SimpleSAML\\SOAP11\\XML\\AbstractSoapElement' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/AbstractSoapElement.php',
    'SimpleSAML\\SOAP11\\XML\\ActorTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/ActorTrait.php',
    'SimpleSAML\\SOAP11\\XML\\Body' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/Body.php',
    'SimpleSAML\\SOAP11\\XML\\Detail' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/Detail.php',
    'SimpleSAML\\SOAP11\\XML\\Envelope' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/Envelope.php',
    'SimpleSAML\\SOAP11\\XML\\Fault' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/Fault.php',
    'SimpleSAML\\SOAP11\\XML\\FaultActor' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/FaultActor.php',
    'SimpleSAML\\SOAP11\\XML\\FaultCode' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/FaultCode.php',
    'SimpleSAML\\SOAP11\\XML\\FaultString' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/FaultString.php',
    'SimpleSAML\\SOAP11\\XML\\Header' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/Header.php',
    'SimpleSAML\\SOAP11\\XML\\MustUnderstandTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP11/XML/MustUnderstandTrait.php',
    'SimpleSAML\\SOAP12\\Assert\\Assert' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Assert/Assert.php',
    'SimpleSAML\\SOAP12\\Constants' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Constants.php',
    'SimpleSAML\\SOAP12\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Exception/ProtocolViolationException.php',
    'SimpleSAML\\SOAP12\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Exception/RuntimeException.php',
    'SimpleSAML\\SOAP12\\Type\\MustUnderstandValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Type/MustUnderstandValue.php',
    'SimpleSAML\\SOAP12\\Type\\RelayValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Type/RelayValue.php',
    'SimpleSAML\\SOAP12\\Type\\RoleValue' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Type/RoleValue.php',
    'SimpleSAML\\SOAP12\\Utils\\XPath' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/Utils/XPath.php',
    'SimpleSAML\\SOAP12\\XML\\AbstractSoapElement' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/AbstractSoapElement.php',
    'SimpleSAML\\SOAP12\\XML\\Body' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Body.php',
    'SimpleSAML\\SOAP12\\XML\\Code' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Code.php',
    'SimpleSAML\\SOAP12\\XML\\Detail' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Detail.php',
    'SimpleSAML\\SOAP12\\XML\\Envelope' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Envelope.php',
    'SimpleSAML\\SOAP12\\XML\\Fault' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Fault.php',
    'SimpleSAML\\SOAP12\\XML\\Header' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Header.php',
    'SimpleSAML\\SOAP12\\XML\\MustUnderstandTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/MustUnderstandTrait.php',
    'SimpleSAML\\SOAP12\\XML\\Node' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Node.php',
    'SimpleSAML\\SOAP12\\XML\\NotUnderstood' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/NotUnderstood.php',
    'SimpleSAML\\SOAP12\\XML\\Reason' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Reason.php',
    'SimpleSAML\\SOAP12\\XML\\RelayTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/RelayTrait.php',
    'SimpleSAML\\SOAP12\\XML\\Role' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Role.php',
    'SimpleSAML\\SOAP12\\XML\\RoleTrait' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/RoleTrait.php',
    'SimpleSAML\\SOAP12\\XML\\Subcode' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Subcode.php',
    'SimpleSAML\\SOAP12\\XML\\SupportedEnvelope' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/SupportedEnvelope.php',
    'SimpleSAML\\SOAP12\\XML\\Text' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Text.php',
    'SimpleSAML\\SOAP12\\XML\\Upgrade' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Upgrade.php',
    'SimpleSAML\\SOAP12\\XML\\Value' => $vendorDir . '/simplesamlphp/xml-soap/src/SOAP12/XML/Value.php',
    'SimpleSAML\\Session' => $baseDir . '/src/SimpleSAML/Session.php',
    'SimpleSAML\\SessionHandler' => $baseDir . '/src/SimpleSAML/SessionHandler.php',
    'SimpleSAML\\SessionHandlerCookie' => $baseDir . '/src/SimpleSAML/SessionHandlerCookie.php',
    'SimpleSAML\\SessionHandlerPHP' => $baseDir . '/src/SimpleSAML/SessionHandlerPHP.php',
    'SimpleSAML\\SessionHandlerStore' => $baseDir . '/src/SimpleSAML/SessionHandlerStore.php',
    'SimpleSAML\\Stats' => $baseDir . '/src/SimpleSAML/Stats.php',
    'SimpleSAML\\Stats\\Output' => $baseDir . '/src/SimpleSAML/Stats/Output.php',
    'SimpleSAML\\Store\\MemcacheStore' => $baseDir . '/src/SimpleSAML/Store/MemcacheStore.php',
    'SimpleSAML\\Store\\RedisStore' => $baseDir . '/src/SimpleSAML/Store/RedisStore.php',
    'SimpleSAML\\Store\\SQLStore' => $baseDir . '/src/SimpleSAML/Store/SQLStore.php',
    'SimpleSAML\\Store\\StoreFactory' => $baseDir . '/src/SimpleSAML/Store/StoreFactory.php',
    'SimpleSAML\\Store\\StoreInterface' => $baseDir . '/src/SimpleSAML/Store/StoreInterface.php',
    'SimpleSAML\\Utils\\Arrays' => $baseDir . '/src/SimpleSAML/Utils/Arrays.php',
    'SimpleSAML\\Utils\\Attributes' => $baseDir . '/src/SimpleSAML/Utils/Attributes.php',
    'SimpleSAML\\Utils\\Auth' => $baseDir . '/src/SimpleSAML/Utils/Auth.php',
    'SimpleSAML\\Utils\\ClearableState' => $baseDir . '/src/SimpleSAML/Utils/ClearableState.php',
    'SimpleSAML\\Utils\\Config' => $baseDir . '/src/SimpleSAML/Utils/Config.php',
    'SimpleSAML\\Utils\\Config\\Metadata' => $baseDir . '/src/SimpleSAML/Utils/Config/Metadata.php',
    'SimpleSAML\\Utils\\Crypto' => $baseDir . '/src/SimpleSAML/Utils/Crypto.php',
    'SimpleSAML\\Utils\\EMail' => $baseDir . '/src/SimpleSAML/Utils/EMail.php',
    'SimpleSAML\\Utils\\HTTP' => $baseDir . '/src/SimpleSAML/Utils/HTTP.php',
    'SimpleSAML\\Utils\\Net' => $baseDir . '/src/SimpleSAML/Utils/Net.php',
    'SimpleSAML\\Utils\\Random' => $baseDir . '/src/SimpleSAML/Utils/Random.php',
    'SimpleSAML\\Utils\\System' => $baseDir . '/src/SimpleSAML/Utils/System.php',
    'SimpleSAML\\Utils\\Time' => $baseDir . '/src/SimpleSAML/Utils/Time.php',
    'SimpleSAML\\Utils\\Translate' => $baseDir . '/src/SimpleSAML/Utils/Translate.php',
    'SimpleSAML\\Utils\\XML' => $baseDir . '/src/SimpleSAML/Utils/XML.php',
    'SimpleSAML\\XHTML\\IdPDisco' => $baseDir . '/src/SimpleSAML/XHTML/IdPDisco.php',
    'SimpleSAML\\XHTML\\Template' => $baseDir . '/src/SimpleSAML/XHTML/Template.php',
    'SimpleSAML\\XHTML\\TemplateControllerInterface' => $baseDir . '/src/SimpleSAML/XHTML/TemplateControllerInterface.php',
    'SimpleSAML\\XHTML\\TemplateLoader' => $baseDir . '/src/SimpleSAML/XHTML/TemplateLoader.php',
    'SimpleSAML\\XMLSchema\\Constants' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Constants.php',
    'SimpleSAML\\XMLSchema\\Exception\\InvalidDOMAttributeException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/InvalidDOMAttributeException.php',
    'SimpleSAML\\XMLSchema\\Exception\\InvalidDOMElementException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/InvalidDOMElementException.php',
    'SimpleSAML\\XMLSchema\\Exception\\InvalidValueTypeException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/InvalidValueTypeException.php',
    'SimpleSAML\\XMLSchema\\Exception\\MissingAttributeException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/MissingAttributeException.php',
    'SimpleSAML\\XMLSchema\\Exception\\MissingElementException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/MissingElementException.php',
    'SimpleSAML\\XMLSchema\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/ProtocolViolationException.php',
    'SimpleSAML\\XMLSchema\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/RuntimeException.php',
    'SimpleSAML\\XMLSchema\\Exception\\SchemaViolationException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/SchemaViolationException.php',
    'SimpleSAML\\XMLSchema\\Exception\\TooManyElementsException' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Exception/TooManyElementsException.php',
    'SimpleSAML\\XMLSchema\\Type\\AnyURIValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/AnyURIValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Base64BinaryValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Base64BinaryValue.php',
    'SimpleSAML\\XMLSchema\\Type\\BooleanValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/BooleanValue.php',
    'SimpleSAML\\XMLSchema\\Type\\ByteValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/ByteValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DateTimeValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DateTimeValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DateValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DateValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DayValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DayValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DecimalValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DecimalValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DoubleValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DoubleValue.php',
    'SimpleSAML\\XMLSchema\\Type\\DurationValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/DurationValue.php',
    'SimpleSAML\\XMLSchema\\Type\\EntitiesValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/EntitiesValue.php',
    'SimpleSAML\\XMLSchema\\Type\\EntityValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/EntityValue.php',
    'SimpleSAML\\XMLSchema\\Type\\FloatValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/FloatValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Helper\\AnyURIListValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Helper/AnyURIListValue.php',
    'SimpleSAML\\XMLSchema\\Type\\HexBinaryValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/HexBinaryValue.php',
    'SimpleSAML\\XMLSchema\\Type\\IDRefValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/IDRefValue.php',
    'SimpleSAML\\XMLSchema\\Type\\IDRefsValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/IDRefsValue.php',
    'SimpleSAML\\XMLSchema\\Type\\IDValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/IDValue.php',
    'SimpleSAML\\XMLSchema\\Type\\IntValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/IntValue.php',
    'SimpleSAML\\XMLSchema\\Type\\IntegerValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/IntegerValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Interface\\AbstractAnySimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Interface/AbstractAnySimpleType.php',
    'SimpleSAML\\XMLSchema\\Type\\Interface\\AttributeTypeInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Interface/AttributeTypeInterface.php',
    'SimpleSAML\\XMLSchema\\Type\\Interface\\ListTypeInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Interface/ListTypeInterface.php',
    'SimpleSAML\\XMLSchema\\Type\\Interface\\ValueTypeInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Interface/ValueTypeInterface.php',
    'SimpleSAML\\XMLSchema\\Type\\LanguageValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/LanguageValue.php',
    'SimpleSAML\\XMLSchema\\Type\\LongValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/LongValue.php',
    'SimpleSAML\\XMLSchema\\Type\\MonthValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/MonthValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NCNameValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NCNameValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NMTokenValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NMTokenValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NMTokensValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NMTokensValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NameValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NameValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NegativeIntegerValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NegativeIntegerValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NonNegativeIntegerValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NonNegativeIntegerValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NonPositiveIntegerValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NonPositiveIntegerValue.php',
    'SimpleSAML\\XMLSchema\\Type\\NormalizedStringValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/NormalizedStringValue.php',
    'SimpleSAML\\XMLSchema\\Type\\PositiveIntegerValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/PositiveIntegerValue.php',
    'SimpleSAML\\XMLSchema\\Type\\QNameValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/QNameValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\AbstractAllNNIValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/AbstractAllNNIValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\BlockSetValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/BlockSetValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\DerivationControlValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/DerivationControlValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\DerivationSetValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/DerivationSetValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\FormChoiceValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/FormChoiceValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\FullDerivationSetValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/FullDerivationSetValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\MaxOccursValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/MaxOccursValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\MinOccursValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/MinOccursValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\NamespaceListValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/NamespaceListValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\ProcessContentsValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/ProcessContentsValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\PublicValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/PublicValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\ReducedDerivationControlValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/ReducedDerivationControlValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\SimpleDerivationSetValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/SimpleDerivationSetValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\TypeDerivationControlValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/TypeDerivationControlValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\UseValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/UseValue.php',
    'SimpleSAML\\XMLSchema\\Type\\Schema\\WhiteSpaceValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/Schema/WhiteSpaceValue.php',
    'SimpleSAML\\XMLSchema\\Type\\ShortValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/ShortValue.php',
    'SimpleSAML\\XMLSchema\\Type\\StringValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/StringValue.php',
    'SimpleSAML\\XMLSchema\\Type\\TimeValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/TimeValue.php',
    'SimpleSAML\\XMLSchema\\Type\\TokenValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/TokenValue.php',
    'SimpleSAML\\XMLSchema\\Type\\UnsignedByteValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/UnsignedByteValue.php',
    'SimpleSAML\\XMLSchema\\Type\\UnsignedIntValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/UnsignedIntValue.php',
    'SimpleSAML\\XMLSchema\\Type\\UnsignedLongValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/UnsignedLongValue.php',
    'SimpleSAML\\XMLSchema\\Type\\UnsignedShortValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/UnsignedShortValue.php',
    'SimpleSAML\\XMLSchema\\Type\\YearMonthValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/YearMonthValue.php',
    'SimpleSAML\\XMLSchema\\Type\\YearValue' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/Type/YearValue.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractAll' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractAll.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractAnnotated' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractAnnotated.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractAnyType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractAnyType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractAttribute' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractAttribute.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractAttributeGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractAttributeGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractComplexRestrictionType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractComplexRestrictionType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractComplexType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractComplexType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractElement.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractExplicitGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractExplicitGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractExtensionType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractExtensionType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractFacet' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractFacet.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractKeybase' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractKeybase.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractLocalComplexType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractLocalComplexType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractLocalElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractLocalElement.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractLocalSimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractLocalSimpleType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractNamedAttributeGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractNamedAttributeGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractNamedGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractNamedGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractNarrowMaxMin' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractNarrowMaxMin.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractNoFixedFacet' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractNoFixedFacet.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractNumFacet' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractNumFacet.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractOpenAttrs' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractOpenAttrs.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractRealGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractRealGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractReferencedAttributeGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractReferencedAttributeGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractReferencedGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractReferencedGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractRestrictionType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractRestrictionType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractSimpleExplicitGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractSimpleExplicitGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractSimpleExtensionType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractSimpleExtensionType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractSimpleRestrictionType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractSimpleRestrictionType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractSimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractSimpleType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractTopLevelAttribute' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractTopLevelAttribute.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractTopLevelComplexType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractTopLevelComplexType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractTopLevelElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractTopLevelElement.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractTopLevelSimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractTopLevelSimpleType.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractWildcard' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractWildcard.php',
    'SimpleSAML\\XMLSchema\\XML\\AbstractXsElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AbstractXsElement.php',
    'SimpleSAML\\XMLSchema\\XML\\All' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/All.php',
    'SimpleSAML\\XMLSchema\\XML\\Annotation' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Annotation.php',
    'SimpleSAML\\XMLSchema\\XML\\Any' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Any.php',
    'SimpleSAML\\XMLSchema\\XML\\AnyAttribute' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/AnyAttribute.php',
    'SimpleSAML\\XMLSchema\\XML\\Appinfo' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Appinfo.php',
    'SimpleSAML\\XMLSchema\\XML\\Choice' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Choice.php',
    'SimpleSAML\\XMLSchema\\XML\\ComplexContent' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/ComplexContent.php',
    'SimpleSAML\\XMLSchema\\XML\\ComplexRestriction' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/ComplexRestriction.php',
    'SimpleSAML\\XMLSchema\\XML\\Constants\\NS' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Constants/NS.php',
    'SimpleSAML\\XMLSchema\\XML\\Documentation' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Documentation.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration\\DerivationControlEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration/DerivationControlEnum.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration\\FormChoiceEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration/FormChoiceEnum.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration\\ProcessContentsEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration/ProcessContentsEnum.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration\\UseEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration/UseEnum.php',
    'SimpleSAML\\XMLSchema\\XML\\Enumeration\\WhiteSpaceEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Enumeration/WhiteSpaceEnum.php',
    'SimpleSAML\\XMLSchema\\XML\\Extension' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Extension.php',
    'SimpleSAML\\XMLSchema\\XML\\Field' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Field.php',
    'SimpleSAML\\XMLSchema\\XML\\FractionDigits' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/FractionDigits.php',
    'SimpleSAML\\XMLSchema\\XML\\Import' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Import.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\FacetInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/FacetInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\IdentityConstraintInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/IdentityConstraintInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\NestedParticleInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/NestedParticleInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\ParticleInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/ParticleInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\RedefinableInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/RedefinableInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\SchemaTopInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/SchemaTopInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\SimpleDerivationInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/SimpleDerivationInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Interface\\TypeDefParticleInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Interface/TypeDefParticleInterface.php',
    'SimpleSAML\\XMLSchema\\XML\\Key' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Key.php',
    'SimpleSAML\\XMLSchema\\XML\\Keyref' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Keyref.php',
    'SimpleSAML\\XMLSchema\\XML\\Length' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Length.php',
    'SimpleSAML\\XMLSchema\\XML\\LocalAttribute' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/LocalAttribute.php',
    'SimpleSAML\\XMLSchema\\XML\\LocalComplexType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/LocalComplexType.php',
    'SimpleSAML\\XMLSchema\\XML\\LocalElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/LocalElement.php',
    'SimpleSAML\\XMLSchema\\XML\\LocalSimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/LocalSimpleType.php',
    'SimpleSAML\\XMLSchema\\XML\\MaxExclusive' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MaxExclusive.php',
    'SimpleSAML\\XMLSchema\\XML\\MaxInclusive' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MaxInclusive.php',
    'SimpleSAML\\XMLSchema\\XML\\MaxLength' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MaxLength.php',
    'SimpleSAML\\XMLSchema\\XML\\MinExclusive' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MinExclusive.php',
    'SimpleSAML\\XMLSchema\\XML\\MinInclusive' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MinInclusive.php',
    'SimpleSAML\\XMLSchema\\XML\\MinLength' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/MinLength.php',
    'SimpleSAML\\XMLSchema\\XML\\NamedAttributeGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/NamedAttributeGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\NamedGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/NamedGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\NarrowMaxMinElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/NarrowMaxMinElement.php',
    'SimpleSAML\\XMLSchema\\XML\\Notation' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Notation.php',
    'SimpleSAML\\XMLSchema\\XML\\Pattern' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Pattern.php',
    'SimpleSAML\\XMLSchema\\XML\\Redefine' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Redefine.php',
    'SimpleSAML\\XMLSchema\\XML\\ReferencedAttributeGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/ReferencedAttributeGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\ReferencedGroup' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/ReferencedGroup.php',
    'SimpleSAML\\XMLSchema\\XML\\Restriction' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Restriction.php',
    'SimpleSAML\\XMLSchema\\XML\\Schema' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Schema.php',
    'SimpleSAML\\XMLSchema\\XML\\Selector' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Selector.php',
    'SimpleSAML\\XMLSchema\\XML\\Sequence' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Sequence.php',
    'SimpleSAML\\XMLSchema\\XML\\SimpleChoice' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/SimpleChoice.php',
    'SimpleSAML\\XMLSchema\\XML\\SimpleContent' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/SimpleContent.php',
    'SimpleSAML\\XMLSchema\\XML\\SimpleExtension' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/SimpleExtension.php',
    'SimpleSAML\\XMLSchema\\XML\\SimpleRestriction' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/SimpleRestriction.php',
    'SimpleSAML\\XMLSchema\\XML\\SimpleSequence' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/SimpleSequence.php',
    'SimpleSAML\\XMLSchema\\XML\\TopLevelAttribute' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/TopLevelAttribute.php',
    'SimpleSAML\\XMLSchema\\XML\\TopLevelComplexType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/TopLevelComplexType.php',
    'SimpleSAML\\XMLSchema\\XML\\TopLevelElement' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/TopLevelElement.php',
    'SimpleSAML\\XMLSchema\\XML\\TopLevelSimpleType' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/TopLevelSimpleType.php',
    'SimpleSAML\\XMLSchema\\XML\\TotalDigits' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/TotalDigits.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\AttrDeclsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/AttrDeclsTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\ComplexTypeModelTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/ComplexTypeModelTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\DefRefTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/DefRefTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\FacetsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/FacetsTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\FormChoiceTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/FormChoiceTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\MaxOccursTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/MaxOccursTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\MinOccursTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/MinOccursTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\OccursTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/OccursTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\SimpleDerivationTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/SimpleDerivationTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\SimpleRestrictionModelTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/SimpleRestrictionModelTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Trait\\TypeDefParticleTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Trait/TypeDefParticleTrait.php',
    'SimpleSAML\\XMLSchema\\XML\\Union' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Union.php',
    'SimpleSAML\\XMLSchema\\XML\\Unique' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/Unique.php',
    'SimpleSAML\\XMLSchema\\XML\\WhiteSpace' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/WhiteSpace.php',
    'SimpleSAML\\XMLSchema\\XML\\XsInclude' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/XsInclude.php',
    'SimpleSAML\\XMLSchema\\XML\\XsList' => $vendorDir . '/simplesamlphp/xml-common/src/XMLSchema/XML/XsList.php',
    'SimpleSAML\\XMLSecurity\\Alg\\AlgorithmInterface' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/AlgorithmInterface.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Encryption\\AES' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Encryption/AES.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Encryption\\AbstractEncryptor' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Encryption/AbstractEncryptor.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Encryption\\EncryptionAlgorithmFactory' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Encryption/EncryptionAlgorithmFactory.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Encryption\\EncryptionAlgorithmInterface' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Encryption/EncryptionAlgorithmInterface.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Encryption\\TripleDES' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Encryption/TripleDES.php',
    'SimpleSAML\\XMLSecurity\\Alg\\KeyTransport\\AbstractKeyTransporter' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/KeyTransport/AbstractKeyTransporter.php',
    'SimpleSAML\\XMLSecurity\\Alg\\KeyTransport\\KeyTransportAlgorithmFactory' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/KeyTransport/KeyTransportAlgorithmFactory.php',
    'SimpleSAML\\XMLSecurity\\Alg\\KeyTransport\\KeyTransportAlgorithmInterface' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/KeyTransport/KeyTransportAlgorithmInterface.php',
    'SimpleSAML\\XMLSecurity\\Alg\\KeyTransport\\RSA' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/KeyTransport/RSA.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Signature\\AbstractSigner' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Signature/AbstractSigner.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Signature\\HMAC' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Signature/HMAC.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Signature\\RSA' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Signature/RSA.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Signature\\SignatureAlgorithmFactory' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Signature/SignatureAlgorithmFactory.php',
    'SimpleSAML\\XMLSecurity\\Alg\\Signature\\SignatureAlgorithmInterface' => $vendorDir . '/simplesamlphp/xml-security/src/Alg/Signature/SignatureAlgorithmInterface.php',
    'SimpleSAML\\XMLSecurity\\Assert\\Assert' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/Assert.php',
    'SimpleSAML\\XMLSecurity\\Assert\\CryptoBinaryTrait' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/CryptoBinaryTrait.php',
    'SimpleSAML\\XMLSecurity\\Assert\\DigestValueTrait' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/DigestValueTrait.php',
    'SimpleSAML\\XMLSecurity\\Assert\\ECPointTrait' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/ECPointTrait.php',
    'SimpleSAML\\XMLSecurity\\Assert\\HMACOutputLengthTrait' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/HMACOutputLengthTrait.php',
    'SimpleSAML\\XMLSecurity\\Assert\\KeySizeTrait' => $vendorDir . '/simplesamlphp/xml-security/src/Assert/KeySizeTrait.php',
    'SimpleSAML\\XMLSecurity\\Backend\\EncryptionBackend' => $vendorDir . '/simplesamlphp/xml-security/src/Backend/EncryptionBackend.php',
    'SimpleSAML\\XMLSecurity\\Backend\\HMAC' => $vendorDir . '/simplesamlphp/xml-security/src/Backend/HMAC.php',
    'SimpleSAML\\XMLSecurity\\Backend\\OpenSSL' => $vendorDir . '/simplesamlphp/xml-security/src/Backend/OpenSSL.php',
    'SimpleSAML\\XMLSecurity\\Backend\\SignatureBackend' => $vendorDir . '/simplesamlphp/xml-security/src/Backend/SignatureBackend.php',
    'SimpleSAML\\XMLSecurity\\Constants' => $vendorDir . '/simplesamlphp/xml-security/src/Constants.php',
    'SimpleSAML\\XMLSecurity\\CryptoEncoding\\PEM' => $vendorDir . '/simplesamlphp/xml-security/src/CryptoEncoding/PEM.php',
    'SimpleSAML\\XMLSecurity\\CryptoEncoding\\PEMBundle' => $vendorDir . '/simplesamlphp/xml-security/src/CryptoEncoding/PEMBundle.php',
    'SimpleSAML\\XMLSecurity\\Exception\\BlacklistedAlgorithmException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/BlacklistedAlgorithmException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\CanonicalizationFailedException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/CanonicalizationFailedException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\IOException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/IOException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\InvalidArgumentException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/InvalidArgumentException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\NoEncryptedDataException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/NoEncryptedDataException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\NoSignatureFoundException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/NoSignatureFoundException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\OpenSSLException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/OpenSSLException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\ProtocolViolationException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/ProtocolViolationException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\ReferenceValidationFailedException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/ReferenceValidationFailedException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/RuntimeException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\SignatureVerificationFailedException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/SignatureVerificationFailedException.php',
    'SimpleSAML\\XMLSecurity\\Exception\\UnsupportedAlgorithmException' => $vendorDir . '/simplesamlphp/xml-security/src/Exception/UnsupportedAlgorithmException.php',
    'SimpleSAML\\XMLSecurity\\Key\\AsymmetricKey' => $vendorDir . '/simplesamlphp/xml-security/src/Key/AsymmetricKey.php',
    'SimpleSAML\\XMLSecurity\\Key\\KeyInterface' => $vendorDir . '/simplesamlphp/xml-security/src/Key/KeyInterface.php',
    'SimpleSAML\\XMLSecurity\\Key\\PrivateKey' => $vendorDir . '/simplesamlphp/xml-security/src/Key/PrivateKey.php',
    'SimpleSAML\\XMLSecurity\\Key\\PublicKey' => $vendorDir . '/simplesamlphp/xml-security/src/Key/PublicKey.php',
    'SimpleSAML\\XMLSecurity\\Key\\SymmetricKey' => $vendorDir . '/simplesamlphp/xml-security/src/Key/SymmetricKey.php',
    'SimpleSAML\\XMLSecurity\\Key\\X509Certificate' => $vendorDir . '/simplesamlphp/xml-security/src/Key/X509Certificate.php',
    'SimpleSAML\\XMLSecurity\\TestUtils\\PEMCertificatesMock' => $vendorDir . '/simplesamlphp/xml-security/src/TestUtils/PEMCertificatesMock.php',
    'SimpleSAML\\XMLSecurity\\TestUtils\\SignedElementTestTrait' => $vendorDir . '/simplesamlphp/xml-security/src/TestUtils/SignedElementTestTrait.php',
    'SimpleSAML\\XMLSecurity\\Type\\CryptoBinaryValue' => $vendorDir . '/simplesamlphp/xml-security/src/Type/CryptoBinaryValue.php',
    'SimpleSAML\\XMLSecurity\\Type\\DigestValue' => $vendorDir . '/simplesamlphp/xml-security/src/Type/DigestValue.php',
    'SimpleSAML\\XMLSecurity\\Type\\ECPointValue' => $vendorDir . '/simplesamlphp/xml-security/src/Type/ECPointValue.php',
    'SimpleSAML\\XMLSecurity\\Type\\HMACOutputLengthValue' => $vendorDir . '/simplesamlphp/xml-security/src/Type/HMACOutputLengthValue.php',
    'SimpleSAML\\XMLSecurity\\Type\\KeySizeValue' => $vendorDir . '/simplesamlphp/xml-security/src/Type/KeySizeValue.php',
    'SimpleSAML\\XMLSecurity\\Utils\\Certificate' => $vendorDir . '/simplesamlphp/xml-security/src/Utils/Certificate.php',
    'SimpleSAML\\XMLSecurity\\Utils\\Random' => $vendorDir . '/simplesamlphp/xml-security/src/Utils/Random.php',
    'SimpleSAML\\XMLSecurity\\Utils\\XPath' => $vendorDir . '/simplesamlphp/xml-security/src/Utils/XPath.php',
    'SimpleSAML\\XMLSecurity\\XML\\CanonicalizableElementInterface' => $vendorDir . '/simplesamlphp/xml-security/src/XML/CanonicalizableElementInterface.php',
    'SimpleSAML\\XMLSecurity\\XML\\CanonicalizableElementTrait' => $vendorDir . '/simplesamlphp/xml-security/src/XML/CanonicalizableElementTrait.php',
    'SimpleSAML\\XMLSecurity\\XML\\EncryptableElementInterface' => $vendorDir . '/simplesamlphp/xml-security/src/XML/EncryptableElementInterface.php',
    'SimpleSAML\\XMLSecurity\\XML\\EncryptableElementTrait' => $vendorDir . '/simplesamlphp/xml-security/src/XML/EncryptableElementTrait.php',
    'SimpleSAML\\XMLSecurity\\XML\\EncryptedElementInterface' => $vendorDir . '/simplesamlphp/xml-security/src/XML/EncryptedElementInterface.php',
    'SimpleSAML\\XMLSecurity\\XML\\EncryptedElementTrait' => $vendorDir . '/simplesamlphp/xml-security/src/XML/EncryptedElementTrait.php',
    'SimpleSAML\\XMLSecurity\\XML\\SignableElementInterface' => $vendorDir . '/simplesamlphp/xml-security/src/XML/SignableElementInterface.php',
    'SimpleSAML\\XMLSecurity\\XML\\SignableElementTrait' => $vendorDir . '/simplesamlphp/xml-security/src/XML/SignableElementTrait.php',
    'SimpleSAML\\XMLSecurity\\XML\\SignedElementInterface' => $vendorDir . '/simplesamlphp/xml-security/src/XML/SignedElementInterface.php',
    'SimpleSAML\\XMLSecurity\\XML\\SignedElementTrait' => $vendorDir . '/simplesamlphp/xml-security/src/XML/SignedElementTrait.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\AbstractDSAKeyValueType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/AbstractDSAKeyValueType.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\AbstractDsElement' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/AbstractDsElement.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\AbstractKeyInfoType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/AbstractKeyInfoType.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\AbstractPGPDataType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/AbstractPGPDataType.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\AbstractSPKIDataType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/AbstractSPKIDataType.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\CanonicalizationMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/CanonicalizationMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\DSAKeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/DSAKeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\DigestMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/DigestMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\DigestValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/DigestValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\DsObject' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/DsObject.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Exponent' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Exponent.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\G' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/G.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\HMACOutputLength' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/HMACOutputLength.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\J' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/J.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\KeyInfo' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/KeyInfo.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\KeyName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/KeyName.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\KeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/KeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Manifest' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Manifest.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\MgmtData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/MgmtData.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Modulus' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Modulus.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\P' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/P.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\PGPData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/PGPData.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\PGPKeyID' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/PGPKeyID.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\PGPKeyPacket' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/PGPKeyPacket.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\PgenCounter' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/PgenCounter.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Q' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Q.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\RSAKeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/RSAKeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Reference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Reference.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\RetrievalMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/RetrievalMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SPKIData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SPKIData.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SPKISexp' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SPKISexp.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Seed' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Seed.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Signature' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Signature.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SignatureMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SignatureMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SignatureProperties' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SignatureProperties.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SignatureProperty' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SignatureProperty.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SignatureValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SignatureValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\SignedInfo' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/SignedInfo.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Transform' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Transform.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Transforms' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Transforms.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509CRL' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509CRL.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509Certificate' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509Certificate.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509Data' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509Data.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509IssuerName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509IssuerName.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509IssuerSerial' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509IssuerSerial.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509SKI' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509SKI.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509SerialNumber' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509SerialNumber.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\X509SubjectName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/X509SubjectName.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\XPath' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/XPath.php',
    'SimpleSAML\\XMLSecurity\\XML\\ds\\Y' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ds/Y.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\A' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/A.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractCharTwoFieldParamsType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractCharTwoFieldParamsType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractCurveType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractCurveType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractDsig11Element' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractDsig11Element.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractECKeyValueType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractECKeyValueType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractECParametersType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractECParametersType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractECValidationDataType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractECValidationDataType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractFieldIDType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractFieldIDType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractNamedCurveType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractNamedCurveType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractPnBFieldParamsType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractPnBFieldParamsType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractPrimeFieldParamsType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractPrimeFieldParamsType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\AbstractTnBFieldParamsType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/AbstractTnBFieldParamsType.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\B' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/B.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\Base' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/Base.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\CoFactor' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/CoFactor.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\Curve' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/Curve.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\DEREncodedKeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/DEREncodedKeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\ECKeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/ECKeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\ECParameters' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/ECParameters.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\FieldID' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/FieldID.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\GnB' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/GnB.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\K' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/K.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\K1' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/K1.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\K2' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/K2.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\K3' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/K3.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\KeyInfoReference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/KeyInfoReference.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\M' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/M.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\NamedCurve' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/NamedCurve.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\Order' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/Order.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\P' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/P.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\PnB' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/PnB.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\Prime' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/Prime.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\PublicKey' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/PublicKey.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\Seed' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/Seed.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\TnB' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/TnB.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\ValidationData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/ValidationData.php',
    'SimpleSAML\\XMLSecurity\\XML\\dsig11\\X509Digest' => $vendorDir . '/simplesamlphp/xml-security/src/XML/dsig11/X509Digest.php',
    'SimpleSAML\\XMLSecurity\\XML\\ec\\AbstractEcElement' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ec/AbstractEcElement.php',
    'SimpleSAML\\XMLSecurity\\XML\\ec\\InclusiveNamespaces' => $vendorDir . '/simplesamlphp/xml-security/src/XML/ec/InclusiveNamespaces.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractAlgorithmIdentifierType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractAlgorithmIdentifierType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractConcatKDFParamsType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractConcatKDFParamsType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractDerivedKeyType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractDerivedKeyType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractKeyDerivationMethodType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractKeyDerivationMethodType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractMGFType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractMGFType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractPBKDF2ParameterType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractPBKDF2ParameterType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractPRFAlgorithmIdentifierType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractPRFAlgorithmIdentifierType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\AbstractXenc11Element' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/AbstractXenc11Element.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\ConcatKDFParams' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/ConcatKDFParams.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\DerivedKey' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/DerivedKey.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\DerivedKeyName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/DerivedKeyName.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\IterationCount' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/IterationCount.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\KeyDerivationMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/KeyDerivationMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\KeyLength' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/KeyLength.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\MGF' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/MGF.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\MasterKeyName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/MasterKeyName.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\OtherSource' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/OtherSource.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\PBKDF2params' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/PBKDF2params.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\PRF' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/PRF.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\Parameters' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/Parameters.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\Salt' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/Salt.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc11\\Specified' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc11/Specified.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractAgreementMethodType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractAgreementMethodType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractDHKeyValueType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractDHKeyValueType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractEncryptedType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractEncryptedType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractEncryptionMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractEncryptionMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractEncryptionPropertiesType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractEncryptionPropertiesType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractEncryptionPropertyType' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractEncryptionPropertyType.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractReference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractReference.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AbstractXencElement' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AbstractXencElement.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\AgreementMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/AgreementMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\CarriedKeyName' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/CarriedKeyName.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\CipherData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/CipherData.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\CipherReference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/CipherReference.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\CipherValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/CipherValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\DHKeyValue' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/DHKeyValue.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\DataReference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/DataReference.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\EncryptedData' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/EncryptedData.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\EncryptedKey' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/EncryptedKey.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\EncryptionMethod' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/EncryptionMethod.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\EncryptionProperties' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/EncryptionProperties.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\EncryptionProperty' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/EncryptionProperty.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\Generator' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/Generator.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\KANonce' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/KANonce.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\KeyReference' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/KeyReference.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\KeySize' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/KeySize.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\OAEPparams' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/OAEPparams.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\OriginatorKeyInfo' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/OriginatorKeyInfo.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\P' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/P.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\PgenCounter' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/PgenCounter.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\Q' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/Q.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\RecipientKeyInfo' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/RecipientKeyInfo.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\ReferenceList' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/ReferenceList.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\Seed' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/Seed.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\Transforms' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/Transforms.php',
    'SimpleSAML\\XMLSecurity\\XML\\xenc\\XencPublic' => $vendorDir . '/simplesamlphp/xml-security/src/XML/xenc/XencPublic.php',
    'SimpleSAML\\XML\\AbstractElement' => $vendorDir . '/simplesamlphp/xml-common/src/XML/AbstractElement.php',
    'SimpleSAML\\XML\\ArrayizableElementInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XML/ArrayizableElementInterface.php',
    'SimpleSAML\\XML\\Assert\\AnyURITrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/AnyURITrait.php',
    'SimpleSAML\\XML\\Assert\\Assert' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/Assert.php',
    'SimpleSAML\\XML\\Assert\\Base64BinaryTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/Base64BinaryTrait.php',
    'SimpleSAML\\XML\\Assert\\BooleanTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/BooleanTrait.php',
    'SimpleSAML\\XML\\Assert\\ByteTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/ByteTrait.php',
    'SimpleSAML\\XML\\Assert\\DateTimeTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DateTimeTrait.php',
    'SimpleSAML\\XML\\Assert\\DateTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DateTrait.php',
    'SimpleSAML\\XML\\Assert\\DayTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DayTrait.php',
    'SimpleSAML\\XML\\Assert\\DecimalTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DecimalTrait.php',
    'SimpleSAML\\XML\\Assert\\DoubleTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DoubleTrait.php',
    'SimpleSAML\\XML\\Assert\\DurationTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/DurationTrait.php',
    'SimpleSAML\\XML\\Assert\\EntitiesTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/EntitiesTrait.php',
    'SimpleSAML\\XML\\Assert\\EntityTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/EntityTrait.php',
    'SimpleSAML\\XML\\Assert\\FloatTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/FloatTrait.php',
    'SimpleSAML\\XML\\Assert\\HexBinaryTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/HexBinaryTrait.php',
    'SimpleSAML\\XML\\Assert\\IDRefTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/IDRefTrait.php',
    'SimpleSAML\\XML\\Assert\\IDRefsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/IDRefsTrait.php',
    'SimpleSAML\\XML\\Assert\\IDTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/IDTrait.php',
    'SimpleSAML\\XML\\Assert\\IntTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/IntTrait.php',
    'SimpleSAML\\XML\\Assert\\IntegerTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/IntegerTrait.php',
    'SimpleSAML\\XML\\Assert\\LanguageTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/LanguageTrait.php',
    'SimpleSAML\\XML\\Assert\\LongTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/LongTrait.php',
    'SimpleSAML\\XML\\Assert\\MonthTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/MonthTrait.php',
    'SimpleSAML\\XML\\Assert\\NCNameTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NCNameTrait.php',
    'SimpleSAML\\XML\\Assert\\NMTokenTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NMTokenTrait.php',
    'SimpleSAML\\XML\\Assert\\NMTokensTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NMTokensTrait.php',
    'SimpleSAML\\XML\\Assert\\NameTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NameTrait.php',
    'SimpleSAML\\XML\\Assert\\NegativeIntegerTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NegativeIntegerTrait.php',
    'SimpleSAML\\XML\\Assert\\NonNegativeIntegerTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NonNegativeIntegerTrait.php',
    'SimpleSAML\\XML\\Assert\\NonPositiveIntegerTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NonPositiveIntegerTrait.php',
    'SimpleSAML\\XML\\Assert\\NormalizedStringTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/NormalizedStringTrait.php',
    'SimpleSAML\\XML\\Assert\\PositiveIntegerTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/PositiveIntegerTrait.php',
    'SimpleSAML\\XML\\Assert\\QNameTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/QNameTrait.php',
    'SimpleSAML\\XML\\Assert\\ShortTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/ShortTrait.php',
    'SimpleSAML\\XML\\Assert\\StringTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/StringTrait.php',
    'SimpleSAML\\XML\\Assert\\TimeTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/TimeTrait.php',
    'SimpleSAML\\XML\\Assert\\TokenTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/TokenTrait.php',
    'SimpleSAML\\XML\\Assert\\UnsignedByteTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/UnsignedByteTrait.php',
    'SimpleSAML\\XML\\Assert\\UnsignedIntTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/UnsignedIntTrait.php',
    'SimpleSAML\\XML\\Assert\\UnsignedLongTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/UnsignedLongTrait.php',
    'SimpleSAML\\XML\\Assert\\UnsignedShortTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/UnsignedShortTrait.php',
    'SimpleSAML\\XML\\Assert\\XPathFilterTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/XPathFilterTrait.php',
    'SimpleSAML\\XML\\Assert\\YearMonthTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/YearMonthTrait.php',
    'SimpleSAML\\XML\\Assert\\YearTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Assert/YearTrait.php',
    'SimpleSAML\\XML\\Attribute' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Attribute.php',
    'SimpleSAML\\XML\\Chunk' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Chunk.php',
    'SimpleSAML\\XML\\Constants' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Constants.php',
    'SimpleSAML\\XML\\Container\\AbstractTestContainer' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Container/AbstractTestContainer.php',
    'SimpleSAML\\XML\\Container\\TestContainerInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Container/TestContainerInterface.php',
    'SimpleSAML\\XML\\Container\\TestContainerSingleton' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Container/TestContainerSingleton.php',
    'SimpleSAML\\XML\\Container\\XMLElementsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Container/XMLElementsTrait.php',
    'SimpleSAML\\XML\\Container\\XMLSchemaElementsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Container/XMLSchemaElementsTrait.php',
    'SimpleSAML\\XML\\DOMDocumentFactory' => $vendorDir . '/simplesamlphp/xml-common/src/XML/DOMDocumentFactory.php',
    'SimpleSAML\\XML\\ElementInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XML/ElementInterface.php',
    'SimpleSAML\\XML\\Errors' => $baseDir . '/src/SimpleSAML/XML/Errors.php',
    'SimpleSAML\\XML\\Exception\\IOException' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Exception/IOException.php',
    'SimpleSAML\\XML\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Exception/RuntimeException.php',
    'SimpleSAML\\XML\\Exception\\UnparseableXMLException' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Exception/UnparseableXMLException.php',
    'SimpleSAML\\XML\\ExtendableAttributesTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/ExtendableAttributesTrait.php',
    'SimpleSAML\\XML\\ExtendableElementTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/ExtendableElementTrait.php',
    'SimpleSAML\\XML\\Parser' => $baseDir . '/src/SimpleSAML/XML/Parser.php',
    'SimpleSAML\\XML\\Registry\\ElementRegistry' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Registry/ElementRegistry.php',
    'SimpleSAML\\XML\\SchemaValidatableElementInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XML/SchemaValidatableElementInterface.php',
    'SimpleSAML\\XML\\SchemaValidatableElementTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/SchemaValidatableElementTrait.php',
    'SimpleSAML\\XML\\SerializableElementInterface' => $vendorDir . '/simplesamlphp/xml-common/src/XML/SerializableElementInterface.php',
    'SimpleSAML\\XML\\SerializableElementTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/SerializableElementTrait.php',
    'SimpleSAML\\XML\\Signer' => $baseDir . '/src/SimpleSAML/XML/Signer.php',
    'SimpleSAML\\XML\\TestUtils\\ArrayizableElementTestTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/TestUtils/ArrayizableElementTestTrait.php',
    'SimpleSAML\\XML\\TestUtils\\SchemaValidationTestTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/TestUtils/SchemaValidationTestTrait.php',
    'SimpleSAML\\XML\\TestUtils\\SerializableElementTestTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/TestUtils/SerializableElementTestTrait.php',
    'SimpleSAML\\XML\\TestUtils\\TestContainerTestTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/TestUtils/TestContainerTestTrait.php',
    'SimpleSAML\\XML\\Type\\BaseValue' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Type/BaseValue.php',
    'SimpleSAML\\XML\\Type\\IDValue' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Type/IDValue.php',
    'SimpleSAML\\XML\\Type\\LangValue' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Type/LangValue.php',
    'SimpleSAML\\XML\\Type\\SpaceValue' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Type/SpaceValue.php',
    'SimpleSAML\\XML\\TypedTextContentTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/TypedTextContentTrait.php',
    'SimpleSAML\\XML\\Utils' => $vendorDir . '/simplesamlphp/xml-common/src/XML/Utils.php',
    'SimpleSAML\\XML\\Validator' => $baseDir . '/src/SimpleSAML/XML/Validator.php',
    'SimpleSAML\\XML\\XML\\Enumeration\\SpaceEnum' => $vendorDir . '/simplesamlphp/xml-common/src/XML/XML/Enumeration/SpaceEnum.php',
    'SimpleSAML\\XML\\XML\\Trait\\SpecialAttrsTrait' => $vendorDir . '/simplesamlphp/xml-common/src/XML/XML/Trait/SpecialAttrsTrait.php',
    'SimpleSAML\\XPath\\Constants' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/Constants.php',
    'SimpleSAML\\XPath\\Exception\\AxisNotAllowedException' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/Exception/AxisNotAllowedException.php',
    'SimpleSAML\\XPath\\Exception\\FunctionNotAllowedException' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/Exception/FunctionNotAllowedException.php',
    'SimpleSAML\\XPath\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/Exception/RuntimeException.php',
    'SimpleSAML\\XPath\\XPath' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/XPath.php',
    'SimpleSAML\\XPath\\XPathFilter' => $vendorDir . '/simplesamlphp/xml-common/src/XPath/XPathFilter.php',
    'Stringable' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/Stringable.php',
    'Symfony\\Bridge\\Twig\\AppVariable' => $vendorDir . '/symfony/twig-bridge/AppVariable.php',
    'Symfony\\Bridge\\Twig\\Attribute\\Template' => $vendorDir . '/symfony/twig-bridge/Attribute/Template.php',
    'Symfony\\Bridge\\Twig\\Command\\DebugCommand' => $vendorDir . '/symfony/twig-bridge/Command/DebugCommand.php',
    'Symfony\\Bridge\\Twig\\Command\\LintCommand' => $vendorDir . '/symfony/twig-bridge/Command/LintCommand.php',
    'Symfony\\Bridge\\Twig\\DataCollector\\TwigDataCollector' => $vendorDir . '/symfony/twig-bridge/DataCollector/TwigDataCollector.php',
    'Symfony\\Bridge\\Twig\\ErrorRenderer\\TwigErrorRenderer' => $vendorDir . '/symfony/twig-bridge/ErrorRenderer/TwigErrorRenderer.php',
    'Symfony\\Bridge\\Twig\\EventListener\\TemplateAttributeListener' => $vendorDir . '/symfony/twig-bridge/EventListener/TemplateAttributeListener.php',
    'Symfony\\Bridge\\Twig\\Extension\\AssetExtension' => $vendorDir . '/symfony/twig-bridge/Extension/AssetExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\CsrfExtension' => $vendorDir . '/symfony/twig-bridge/Extension/CsrfExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\CsrfRuntime' => $vendorDir . '/symfony/twig-bridge/Extension/CsrfRuntime.php',
    'Symfony\\Bridge\\Twig\\Extension\\DumpExtension' => $vendorDir . '/symfony/twig-bridge/Extension/DumpExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\EmojiExtension' => $vendorDir . '/symfony/twig-bridge/Extension/EmojiExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\ExpressionExtension' => $vendorDir . '/symfony/twig-bridge/Extension/ExpressionExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\FormExtension' => $vendorDir . '/symfony/twig-bridge/Extension/FormExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\HtmlSanitizerExtension' => $vendorDir . '/symfony/twig-bridge/Extension/HtmlSanitizerExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\HttpFoundationExtension' => $vendorDir . '/symfony/twig-bridge/Extension/HttpFoundationExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\HttpKernelExtension' => $vendorDir . '/symfony/twig-bridge/Extension/HttpKernelExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\HttpKernelRuntime' => $vendorDir . '/symfony/twig-bridge/Extension/HttpKernelRuntime.php',
    'Symfony\\Bridge\\Twig\\Extension\\ImportMapExtension' => $vendorDir . '/symfony/twig-bridge/Extension/ImportMapExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\ImportMapRuntime' => $vendorDir . '/symfony/twig-bridge/Extension/ImportMapRuntime.php',
    'Symfony\\Bridge\\Twig\\Extension\\LogoutUrlExtension' => $vendorDir . '/symfony/twig-bridge/Extension/LogoutUrlExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\ProfilerExtension' => $vendorDir . '/symfony/twig-bridge/Extension/ProfilerExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\RoutingExtension' => $vendorDir . '/symfony/twig-bridge/Extension/RoutingExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\SecurityExtension' => $vendorDir . '/symfony/twig-bridge/Extension/SecurityExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\SerializerExtension' => $vendorDir . '/symfony/twig-bridge/Extension/SerializerExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\SerializerRuntime' => $vendorDir . '/symfony/twig-bridge/Extension/SerializerRuntime.php',
    'Symfony\\Bridge\\Twig\\Extension\\StopwatchExtension' => $vendorDir . '/symfony/twig-bridge/Extension/StopwatchExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\TranslationExtension' => $vendorDir . '/symfony/twig-bridge/Extension/TranslationExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\WebLinkExtension' => $vendorDir . '/symfony/twig-bridge/Extension/WebLinkExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\WorkflowExtension' => $vendorDir . '/symfony/twig-bridge/Extension/WorkflowExtension.php',
    'Symfony\\Bridge\\Twig\\Extension\\YamlExtension' => $vendorDir . '/symfony/twig-bridge/Extension/YamlExtension.php',
    'Symfony\\Bridge\\Twig\\Form\\TwigRendererEngine' => $vendorDir . '/symfony/twig-bridge/Form/TwigRendererEngine.php',
    'Symfony\\Bridge\\Twig\\Mime\\BodyRenderer' => $vendorDir . '/symfony/twig-bridge/Mime/BodyRenderer.php',
    'Symfony\\Bridge\\Twig\\Mime\\NotificationEmail' => $vendorDir . '/symfony/twig-bridge/Mime/NotificationEmail.php',
    'Symfony\\Bridge\\Twig\\Mime\\TemplatedEmail' => $vendorDir . '/symfony/twig-bridge/Mime/TemplatedEmail.php',
    'Symfony\\Bridge\\Twig\\Mime\\WrappedTemplatedEmail' => $vendorDir . '/symfony/twig-bridge/Mime/WrappedTemplatedEmail.php',
    'Symfony\\Bridge\\Twig\\NodeVisitor\\Scope' => $vendorDir . '/symfony/twig-bridge/NodeVisitor/Scope.php',
    'Symfony\\Bridge\\Twig\\NodeVisitor\\TranslationDefaultDomainNodeVisitor' => $vendorDir . '/symfony/twig-bridge/NodeVisitor/TranslationDefaultDomainNodeVisitor.php',
    'Symfony\\Bridge\\Twig\\NodeVisitor\\TranslationNodeVisitor' => $vendorDir . '/symfony/twig-bridge/NodeVisitor/TranslationNodeVisitor.php',
    'Symfony\\Bridge\\Twig\\Node\\DumpNode' => $vendorDir . '/symfony/twig-bridge/Node/DumpNode.php',
    'Symfony\\Bridge\\Twig\\Node\\FormThemeNode' => $vendorDir . '/symfony/twig-bridge/Node/FormThemeNode.php',
    'Symfony\\Bridge\\Twig\\Node\\RenderBlockNode' => $vendorDir . '/symfony/twig-bridge/Node/RenderBlockNode.php',
    'Symfony\\Bridge\\Twig\\Node\\SearchAndRenderBlockNode' => $vendorDir . '/symfony/twig-bridge/Node/SearchAndRenderBlockNode.php',
    'Symfony\\Bridge\\Twig\\Node\\StopwatchNode' => $vendorDir . '/symfony/twig-bridge/Node/StopwatchNode.php',
    'Symfony\\Bridge\\Twig\\Node\\TransDefaultDomainNode' => $vendorDir . '/symfony/twig-bridge/Node/TransDefaultDomainNode.php',
    'Symfony\\Bridge\\Twig\\Node\\TransNode' => $vendorDir . '/symfony/twig-bridge/Node/TransNode.php',
    'Symfony\\Bridge\\Twig\\Test\\FormLayoutTestCase' => $vendorDir . '/symfony/twig-bridge/Test/FormLayoutTestCase.php',
    'Symfony\\Bridge\\Twig\\Test\\Traits\\RuntimeLoaderProvider' => $vendorDir . '/symfony/twig-bridge/Test/Traits/RuntimeLoaderProvider.php',
    'Symfony\\Bridge\\Twig\\TokenParser\\DumpTokenParser' => $vendorDir . '/symfony/twig-bridge/TokenParser/DumpTokenParser.php',
    'Symfony\\Bridge\\Twig\\TokenParser\\FormThemeTokenParser' => $vendorDir . '/symfony/twig-bridge/TokenParser/FormThemeTokenParser.php',
    'Symfony\\Bridge\\Twig\\TokenParser\\StopwatchTokenParser' => $vendorDir . '/symfony/twig-bridge/TokenParser/StopwatchTokenParser.php',
    'Symfony\\Bridge\\Twig\\TokenParser\\TransDefaultDomainTokenParser' => $vendorDir . '/symfony/twig-bridge/TokenParser/TransDefaultDomainTokenParser.php',
    'Symfony\\Bridge\\Twig\\TokenParser\\TransTokenParser' => $vendorDir . '/symfony/twig-bridge/TokenParser/TransTokenParser.php',
    'Symfony\\Bridge\\Twig\\Translation\\TwigExtractor' => $vendorDir . '/symfony/twig-bridge/Translation/TwigExtractor.php',
    'Symfony\\Bridge\\Twig\\UndefinedCallableHandler' => $vendorDir . '/symfony/twig-bridge/UndefinedCallableHandler.php',
    'Symfony\\Bridge\\Twig\\Validator\\Constraints\\Twig' => $vendorDir . '/symfony/twig-bridge/Validator/Constraints/Twig.php',
    'Symfony\\Bridge\\Twig\\Validator\\Constraints\\TwigValidator' => $vendorDir . '/symfony/twig-bridge/Validator/Constraints/TwigValidator.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\AbstractPhpFileCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/AbstractPhpFileCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\CachePoolClearerCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/CachePoolClearerCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\ConfigBuilderCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/ConfigBuilderCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\RouterCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/RouterCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\SerializerCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/SerializerCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\TranslationsCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/TranslationsCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\ValidatorCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/ValidatorCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AboutCommand' => $vendorDir . '/symfony/framework-bundle/Command/AboutCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AbstractConfigCommand' => $vendorDir . '/symfony/framework-bundle/Command/AbstractConfigCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AssetsInstallCommand' => $vendorDir . '/symfony/framework-bundle/Command/AssetsInstallCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\BuildDebugContainerTrait' => $vendorDir . '/symfony/framework-bundle/Command/BuildDebugContainerTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CacheClearCommand' => $vendorDir . '/symfony/framework-bundle/Command/CacheClearCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolClearCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolClearCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolDeleteCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolDeleteCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolInvalidateTagsCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolInvalidateTagsCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolListCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolListCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolPruneCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolPruneCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CacheWarmupCommand' => $vendorDir . '/symfony/framework-bundle/Command/CacheWarmupCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ConfigDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/ConfigDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ConfigDumpReferenceCommand' => $vendorDir . '/symfony/framework-bundle/Command/ConfigDumpReferenceCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ContainerDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/ContainerDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ContainerLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/ContainerLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\DebugAutowiringCommand' => $vendorDir . '/symfony/framework-bundle/Command/DebugAutowiringCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\EventDispatcherDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/EventDispatcherDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\RouterDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/RouterDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\RouterMatchCommand' => $vendorDir . '/symfony/framework-bundle/Command/RouterMatchCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsDecryptToLocalCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsDecryptToLocalCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsEncryptFromLocalCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsEncryptFromLocalCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsGenerateKeysCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsGenerateKeysCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsListCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsListCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsRemoveCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsRemoveCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsRevealCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsRevealCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsSetCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsSetCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\TranslationDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/TranslationDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\TranslationExtractCommand' => $vendorDir . '/symfony/framework-bundle/Command/TranslationExtractCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\TranslationUpdateCommand' => $vendorDir . '/symfony/framework-bundle/Command/TranslationUpdateCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\WorkflowDumpCommand' => $vendorDir . '/symfony/framework-bundle/Command/WorkflowDumpCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\XliffLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/XliffLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\YamlLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/YamlLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Application' => $vendorDir . '/symfony/framework-bundle/Console/Application.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\Descriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/Descriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\JsonDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/JsonDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\MarkdownDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/MarkdownDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\TextDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/TextDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\XmlDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/XmlDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Helper\\DescriptorHelper' => $vendorDir . '/symfony/framework-bundle/Console/Helper/DescriptorHelper.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\AbstractController' => $vendorDir . '/symfony/framework-bundle/Controller/AbstractController.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\ControllerHelper' => $vendorDir . '/symfony/framework-bundle/Controller/ControllerHelper.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\ControllerResolver' => $vendorDir . '/symfony/framework-bundle/Controller/ControllerResolver.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\RedirectController' => $vendorDir . '/symfony/framework-bundle/Controller/RedirectController.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\TemplateController' => $vendorDir . '/symfony/framework-bundle/Controller/TemplateController.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\AbstractDataCollector' => $vendorDir . '/symfony/framework-bundle/DataCollector/AbstractDataCollector.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\RouterDataCollector' => $vendorDir . '/symfony/framework-bundle/DataCollector/RouterDataCollector.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\TemplateAwareDataCollectorInterface' => $vendorDir . '/symfony/framework-bundle/DataCollector/TemplateAwareDataCollectorInterface.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AddDebugLogProcessorPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AddDebugLogProcessorPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AssetsContextPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AssetsContextPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ContainerBuilderDebugDumpPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ContainerBuilderDebugDumpPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ErrorLoggerCompilerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ErrorLoggerCompilerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\PhpConfigReferenceDumpPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/PhpConfigReferenceDumpPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ProfilerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ProfilerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\RemoveUnusedSessionMarshallingHandlerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/RemoveUnusedSessionMarshallingHandlerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TestServiceContainerRealRefPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TestServiceContainerRealRefPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TestServiceContainerWeakRefPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TestServiceContainerWeakRefPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TranslationLintCommandPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TranslationLintCommandPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TranslationUpdateCommandPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TranslationUpdateCommandPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\UnusedTagsPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/UnusedTagsPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Configuration' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Configuration.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\FrameworkExtension' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/FrameworkExtension.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\VirtualRequestStackPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/VirtualRequestStackPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\ErrorHandler\\ErrorRenderer\\RuntimeModeErrorRendererSelector' => $vendorDir . '/symfony/framework-bundle/ErrorHandler/ErrorRenderer/RuntimeModeErrorRendererSelector.php',
    'Symfony\\Bundle\\FrameworkBundle\\EventListener\\ConsoleProfilerListener' => $vendorDir . '/symfony/framework-bundle/EventListener/ConsoleProfilerListener.php',
    'Symfony\\Bundle\\FrameworkBundle\\EventListener\\SuggestMissingPackageSubscriber' => $vendorDir . '/symfony/framework-bundle/EventListener/SuggestMissingPackageSubscriber.php',
    'Symfony\\Bundle\\FrameworkBundle\\FrameworkBundle' => $vendorDir . '/symfony/framework-bundle/FrameworkBundle.php',
    'Symfony\\Bundle\\FrameworkBundle\\HttpCache\\HttpCache' => $vendorDir . '/symfony/framework-bundle/HttpCache/HttpCache.php',
    'Symfony\\Bundle\\FrameworkBundle\\KernelBrowser' => $vendorDir . '/symfony/framework-bundle/KernelBrowser.php',
    'Symfony\\Bundle\\FrameworkBundle\\Kernel\\MicroKernelTrait' => $vendorDir . '/symfony/framework-bundle/Kernel/MicroKernelTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\AttributeRouteControllerLoader' => $vendorDir . '/symfony/framework-bundle/Routing/AttributeRouteControllerLoader.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\Attribute\\AsRoutingConditionService' => $vendorDir . '/symfony/framework-bundle/Routing/Attribute/AsRoutingConditionService.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\DelegatingLoader' => $vendorDir . '/symfony/framework-bundle/Routing/DelegatingLoader.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\RedirectableCompiledUrlMatcher' => $vendorDir . '/symfony/framework-bundle/Routing/RedirectableCompiledUrlMatcher.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\RouteLoaderInterface' => $vendorDir . '/symfony/framework-bundle/Routing/RouteLoaderInterface.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\Router' => $vendorDir . '/symfony/framework-bundle/Routing/Router.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\AbstractVault' => $vendorDir . '/symfony/framework-bundle/Secrets/AbstractVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\DotenvVault' => $vendorDir . '/symfony/framework-bundle/Secrets/DotenvVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\SodiumVault' => $vendorDir . '/symfony/framework-bundle/Secrets/SodiumVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\BrowserKitAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/BrowserKitAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\DomCrawlerAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/DomCrawlerAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\HttpClientAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/HttpClientAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\KernelTestCase' => $vendorDir . '/symfony/framework-bundle/Test/KernelTestCase.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\MailerAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/MailerAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\NotificationAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/NotificationAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\TestBrowserToken' => $vendorDir . '/symfony/framework-bundle/Test/TestBrowserToken.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\TestContainer' => $vendorDir . '/symfony/framework-bundle/Test/TestContainer.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\WebTestAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/WebTestAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\WebTestCase' => $vendorDir . '/symfony/framework-bundle/Test/WebTestCase.php',
    'Symfony\\Bundle\\FrameworkBundle\\Translation\\Translator' => $vendorDir . '/symfony/framework-bundle/Translation/Translator.php',
    'Symfony\\Component\\Cache\\Adapter\\AbstractAdapter' => $vendorDir . '/symfony/cache/Adapter/AbstractAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\AbstractTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/AbstractTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\AdapterInterface' => $vendorDir . '/symfony/cache/Adapter/AdapterInterface.php',
    'Symfony\\Component\\Cache\\Adapter\\ApcuAdapter' => $vendorDir . '/symfony/cache/Adapter/ApcuAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ArrayAdapter' => $vendorDir . '/symfony/cache/Adapter/ArrayAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ChainAdapter' => $vendorDir . '/symfony/cache/Adapter/ChainAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\CouchbaseBucketAdapter' => $vendorDir . '/symfony/cache/Adapter/CouchbaseBucketAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\CouchbaseCollectionAdapter' => $vendorDir . '/symfony/cache/Adapter/CouchbaseCollectionAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\DoctrineDbalAdapter' => $vendorDir . '/symfony/cache/Adapter/DoctrineDbalAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\FilesystemAdapter' => $vendorDir . '/symfony/cache/Adapter/FilesystemAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\FilesystemTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/FilesystemTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\MemcachedAdapter' => $vendorDir . '/symfony/cache/Adapter/MemcachedAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\NullAdapter' => $vendorDir . '/symfony/cache/Adapter/NullAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ParameterNormalizer' => $vendorDir . '/symfony/cache/Adapter/ParameterNormalizer.php',
    'Symfony\\Component\\Cache\\Adapter\\PdoAdapter' => $vendorDir . '/symfony/cache/Adapter/PdoAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\PhpArrayAdapter' => $vendorDir . '/symfony/cache/Adapter/PhpArrayAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\PhpFilesAdapter' => $vendorDir . '/symfony/cache/Adapter/PhpFilesAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ProxyAdapter' => $vendorDir . '/symfony/cache/Adapter/ProxyAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\Psr16Adapter' => $vendorDir . '/symfony/cache/Adapter/Psr16Adapter.php',
    'Symfony\\Component\\Cache\\Adapter\\RedisAdapter' => $vendorDir . '/symfony/cache/Adapter/RedisAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\RedisTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/RedisTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/TagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TagAwareAdapterInterface' => $vendorDir . '/symfony/cache/Adapter/TagAwareAdapterInterface.php',
    'Symfony\\Component\\Cache\\Adapter\\TraceableAdapter' => $vendorDir . '/symfony/cache/Adapter/TraceableAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TraceableTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/TraceableTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\CacheItem' => $vendorDir . '/symfony/cache/CacheItem.php',
    'Symfony\\Component\\Cache\\DataCollector\\CacheDataCollector' => $vendorDir . '/symfony/cache/DataCollector/CacheDataCollector.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CacheCollectorPass' => $vendorDir . '/symfony/cache/DependencyInjection/CacheCollectorPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolClearerPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolClearerPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolPrunerPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolPrunerPass.php',
    'Symfony\\Component\\Cache\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/cache/Exception/BadMethodCallException.php',
    'Symfony\\Component\\Cache\\Exception\\CacheException' => $vendorDir . '/symfony/cache/Exception/CacheException.php',
    'Symfony\\Component\\Cache\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/cache/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Cache\\Exception\\LogicException' => $vendorDir . '/symfony/cache/Exception/LogicException.php',
    'Symfony\\Component\\Cache\\LockRegistry' => $vendorDir . '/symfony/cache/LockRegistry.php',
    'Symfony\\Component\\Cache\\Marshaller\\DefaultMarshaller' => $vendorDir . '/symfony/cache/Marshaller/DefaultMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\DeflateMarshaller' => $vendorDir . '/symfony/cache/Marshaller/DeflateMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\MarshallerInterface' => $vendorDir . '/symfony/cache/Marshaller/MarshallerInterface.php',
    'Symfony\\Component\\Cache\\Marshaller\\SodiumMarshaller' => $vendorDir . '/symfony/cache/Marshaller/SodiumMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\TagAwareMarshaller' => $vendorDir . '/symfony/cache/Marshaller/TagAwareMarshaller.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationDispatcher' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationDispatcher.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationHandler' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationHandler.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationMessage' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationMessage.php',
    'Symfony\\Component\\Cache\\PruneableInterface' => $vendorDir . '/symfony/cache/PruneableInterface.php',
    'Symfony\\Component\\Cache\\Psr16Cache' => $vendorDir . '/symfony/cache/Psr16Cache.php',
    'Symfony\\Component\\Cache\\ResettableInterface' => $vendorDir . '/symfony/cache/ResettableInterface.php',
    'Symfony\\Component\\Cache\\Traits\\AbstractAdapterTrait' => $vendorDir . '/symfony/cache/Traits/AbstractAdapterTrait.php',
    'Symfony\\Component\\Cache\\Traits\\CachedValueInterface' => $vendorDir . '/symfony/cache/Traits/CachedValueInterface.php',
    'Symfony\\Component\\Cache\\Traits\\ContractsTrait' => $vendorDir . '/symfony/cache/Traits/ContractsTrait.php',
    'Symfony\\Component\\Cache\\Traits\\FilesystemCommonTrait' => $vendorDir . '/symfony/cache/Traits/FilesystemCommonTrait.php',
    'Symfony\\Component\\Cache\\Traits\\FilesystemTrait' => $vendorDir . '/symfony/cache/Traits/FilesystemTrait.php',
    'Symfony\\Component\\Cache\\Traits\\ProxyTrait' => $vendorDir . '/symfony/cache/Traits/ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis5Proxy' => $vendorDir . '/symfony/cache/Traits/Redis5Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\Redis61ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis61ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis62ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis62ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis63ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis63ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis6Proxy' => $vendorDir . '/symfony/cache/Traits/Redis6Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster5Proxy' => $vendorDir . '/symfony/cache/Traits/RedisCluster5Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster61ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster61ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster62ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster62ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster63ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster63ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster6Proxy' => $vendorDir . '/symfony/cache/Traits/RedisCluster6Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisClusterNodeProxy' => $vendorDir . '/symfony/cache/Traits/RedisClusterNodeProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisClusterProxy' => $vendorDir . '/symfony/cache/Traits/RedisClusterProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisProxy' => $vendorDir . '/symfony/cache/Traits/RedisProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisTrait' => $vendorDir . '/symfony/cache/Traits/RedisTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RelayClusterProxy' => $vendorDir . '/symfony/cache/Traits/RelayClusterProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RelayProxy' => $vendorDir . '/symfony/cache/Traits/RelayProxy.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay20Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay20Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay21Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay21Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay22Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay22Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay30Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay30Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay40Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay40Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\RelayCluster20Trait' => $vendorDir . '/symfony/cache/Traits/Relay/RelayCluster20Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\RelayCluster21Trait' => $vendorDir . '/symfony/cache/Traits/Relay/RelayCluster21Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\RelayCluster30Trait' => $vendorDir . '/symfony/cache/Traits/Relay/RelayCluster30Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\RelayCluster40Trait' => $vendorDir . '/symfony/cache/Traits/Relay/RelayCluster40Trait.php',
    'Symfony\\Component\\Config\\Builder\\ClassBuilder' => $vendorDir . '/symfony/config/Builder/ClassBuilder.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderGenerator' => $vendorDir . '/symfony/config/Builder/ConfigBuilderGenerator.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderGeneratorInterface' => $vendorDir . '/symfony/config/Builder/ConfigBuilderGeneratorInterface.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderInterface' => $vendorDir . '/symfony/config/Builder/ConfigBuilderInterface.php',
    'Symfony\\Component\\Config\\Builder\\Method' => $vendorDir . '/symfony/config/Builder/Method.php',
    'Symfony\\Component\\Config\\Builder\\Property' => $vendorDir . '/symfony/config/Builder/Property.php',
    'Symfony\\Component\\Config\\ConfigCache' => $vendorDir . '/symfony/config/ConfigCache.php',
    'Symfony\\Component\\Config\\ConfigCacheFactory' => $vendorDir . '/symfony/config/ConfigCacheFactory.php',
    'Symfony\\Component\\Config\\ConfigCacheFactoryInterface' => $vendorDir . '/symfony/config/ConfigCacheFactoryInterface.php',
    'Symfony\\Component\\Config\\ConfigCacheInterface' => $vendorDir . '/symfony/config/ConfigCacheInterface.php',
    'Symfony\\Component\\Config\\Definition\\ArrayNode' => $vendorDir . '/symfony/config/Definition/ArrayNode.php',
    'Symfony\\Component\\Config\\Definition\\ArrayShapeGenerator' => $vendorDir . '/symfony/config/Definition/ArrayShapeGenerator.php',
    'Symfony\\Component\\Config\\Definition\\BaseNode' => $vendorDir . '/symfony/config/Definition/BaseNode.php',
    'Symfony\\Component\\Config\\Definition\\BooleanNode' => $vendorDir . '/symfony/config/Definition/BooleanNode.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ArrayNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/ArrayNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\BooleanNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/BooleanNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\BuilderAwareInterface' => $vendorDir . '/symfony/config/Definition/Builder/BuilderAwareInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\EnumNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/EnumNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ExprBuilder' => $vendorDir . '/symfony/config/Definition/Builder/ExprBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\FloatNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/FloatNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\IntegerNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/IntegerNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\MergeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/MergeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/NodeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/NodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeParentInterface' => $vendorDir . '/symfony/config/Definition/Builder/NodeParentInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NormalizationBuilder' => $vendorDir . '/symfony/config/Definition/Builder/NormalizationBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NumericNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/NumericNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ParentNodeDefinitionInterface' => $vendorDir . '/symfony/config/Definition/Builder/ParentNodeDefinitionInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ScalarNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/ScalarNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\StringNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/StringNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\TreeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/TreeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ValidationBuilder' => $vendorDir . '/symfony/config/Definition/Builder/ValidationBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\VariableNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/VariableNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\ConfigurableInterface' => $vendorDir . '/symfony/config/Definition/ConfigurableInterface.php',
    'Symfony\\Component\\Config\\Definition\\Configuration' => $vendorDir . '/symfony/config/Definition/Configuration.php',
    'Symfony\\Component\\Config\\Definition\\ConfigurationInterface' => $vendorDir . '/symfony/config/Definition/ConfigurationInterface.php',
    'Symfony\\Component\\Config\\Definition\\Configurator\\DefinitionConfigurator' => $vendorDir . '/symfony/config/Definition/Configurator/DefinitionConfigurator.php',
    'Symfony\\Component\\Config\\Definition\\Dumper\\XmlReferenceDumper' => $vendorDir . '/symfony/config/Definition/Dumper/XmlReferenceDumper.php',
    'Symfony\\Component\\Config\\Definition\\Dumper\\YamlReferenceDumper' => $vendorDir . '/symfony/config/Definition/Dumper/YamlReferenceDumper.php',
    'Symfony\\Component\\Config\\Definition\\EnumNode' => $vendorDir . '/symfony/config/Definition/EnumNode.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\DuplicateKeyException' => $vendorDir . '/symfony/config/Definition/Exception/DuplicateKeyException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\Exception' => $vendorDir . '/symfony/config/Definition/Exception/Exception.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\ForbiddenOverwriteException' => $vendorDir . '/symfony/config/Definition/Exception/ForbiddenOverwriteException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidConfigurationException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidConfigurationException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidDefinitionException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidDefinitionException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidTypeException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidTypeException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\UnsetKeyException' => $vendorDir . '/symfony/config/Definition/Exception/UnsetKeyException.php',
    'Symfony\\Component\\Config\\Definition\\FloatNode' => $vendorDir . '/symfony/config/Definition/FloatNode.php',
    'Symfony\\Component\\Config\\Definition\\IntegerNode' => $vendorDir . '/symfony/config/Definition/IntegerNode.php',
    'Symfony\\Component\\Config\\Definition\\Loader\\DefinitionFileLoader' => $vendorDir . '/symfony/config/Definition/Loader/DefinitionFileLoader.php',
    'Symfony\\Component\\Config\\Definition\\NodeInterface' => $vendorDir . '/symfony/config/Definition/NodeInterface.php',
    'Symfony\\Component\\Config\\Definition\\NumericNode' => $vendorDir . '/symfony/config/Definition/NumericNode.php',
    'Symfony\\Component\\Config\\Definition\\Processor' => $vendorDir . '/symfony/config/Definition/Processor.php',
    'Symfony\\Component\\Config\\Definition\\PrototypeNodeInterface' => $vendorDir . '/symfony/config/Definition/PrototypeNodeInterface.php',
    'Symfony\\Component\\Config\\Definition\\PrototypedArrayNode' => $vendorDir . '/symfony/config/Definition/PrototypedArrayNode.php',
    'Symfony\\Component\\Config\\Definition\\ScalarNode' => $vendorDir . '/symfony/config/Definition/ScalarNode.php',
    'Symfony\\Component\\Config\\Definition\\StringNode' => $vendorDir . '/symfony/config/Definition/StringNode.php',
    'Symfony\\Component\\Config\\Definition\\VariableNode' => $vendorDir . '/symfony/config/Definition/VariableNode.php',
    'Symfony\\Component\\Config\\Exception\\FileLoaderImportCircularReferenceException' => $vendorDir . '/symfony/config/Exception/FileLoaderImportCircularReferenceException.php',
    'Symfony\\Component\\Config\\Exception\\FileLocatorFileNotFoundException' => $vendorDir . '/symfony/config/Exception/FileLocatorFileNotFoundException.php',
    'Symfony\\Component\\Config\\Exception\\LoaderLoadException' => $vendorDir . '/symfony/config/Exception/LoaderLoadException.php',
    'Symfony\\Component\\Config\\Exception\\LogicException' => $vendorDir . '/symfony/config/Exception/LogicException.php',
    'Symfony\\Component\\Config\\FileLocator' => $vendorDir . '/symfony/config/FileLocator.php',
    'Symfony\\Component\\Config\\FileLocatorInterface' => $vendorDir . '/symfony/config/FileLocatorInterface.php',
    'Symfony\\Component\\Config\\Loader\\DelegatingLoader' => $vendorDir . '/symfony/config/Loader/DelegatingLoader.php',
    'Symfony\\Component\\Config\\Loader\\DirectoryAwareLoaderInterface' => $vendorDir . '/symfony/config/Loader/DirectoryAwareLoaderInterface.php',
    'Symfony\\Component\\Config\\Loader\\FileLoader' => $vendorDir . '/symfony/config/Loader/FileLoader.php',
    'Symfony\\Component\\Config\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/config/Loader/GlobFileLoader.php',
    'Symfony\\Component\\Config\\Loader\\Loader' => $vendorDir . '/symfony/config/Loader/Loader.php',
    'Symfony\\Component\\Config\\Loader\\LoaderInterface' => $vendorDir . '/symfony/config/Loader/LoaderInterface.php',
    'Symfony\\Component\\Config\\Loader\\LoaderResolver' => $vendorDir . '/symfony/config/Loader/LoaderResolver.php',
    'Symfony\\Component\\Config\\Loader\\LoaderResolverInterface' => $vendorDir . '/symfony/config/Loader/LoaderResolverInterface.php',
    'Symfony\\Component\\Config\\Loader\\ParamConfigurator' => $vendorDir . '/symfony/config/Loader/ParamConfigurator.php',
    'Symfony\\Component\\Config\\ResourceCheckerConfigCache' => $vendorDir . '/symfony/config/ResourceCheckerConfigCache.php',
    'Symfony\\Component\\Config\\ResourceCheckerConfigCacheFactory' => $vendorDir . '/symfony/config/ResourceCheckerConfigCacheFactory.php',
    'Symfony\\Component\\Config\\ResourceCheckerInterface' => $vendorDir . '/symfony/config/ResourceCheckerInterface.php',
    'Symfony\\Component\\Config\\Resource\\ClassExistenceResource' => $vendorDir . '/symfony/config/Resource/ClassExistenceResource.php',
    'Symfony\\Component\\Config\\Resource\\ComposerResource' => $vendorDir . '/symfony/config/Resource/ComposerResource.php',
    'Symfony\\Component\\Config\\Resource\\DirectoryResource' => $vendorDir . '/symfony/config/Resource/DirectoryResource.php',
    'Symfony\\Component\\Config\\Resource\\FileExistenceResource' => $vendorDir . '/symfony/config/Resource/FileExistenceResource.php',
    'Symfony\\Component\\Config\\Resource\\FileResource' => $vendorDir . '/symfony/config/Resource/FileResource.php',
    'Symfony\\Component\\Config\\Resource\\GlobResource' => $vendorDir . '/symfony/config/Resource/GlobResource.php',
    'Symfony\\Component\\Config\\Resource\\ReflectionClassResource' => $vendorDir . '/symfony/config/Resource/ReflectionClassResource.php',
    'Symfony\\Component\\Config\\Resource\\ResourceInterface' => $vendorDir . '/symfony/config/Resource/ResourceInterface.php',
    'Symfony\\Component\\Config\\Resource\\SelfCheckingResourceChecker' => $vendorDir . '/symfony/config/Resource/SelfCheckingResourceChecker.php',
    'Symfony\\Component\\Config\\Resource\\SelfCheckingResourceInterface' => $vendorDir . '/symfony/config/Resource/SelfCheckingResourceInterface.php',
    'Symfony\\Component\\Config\\Resource\\SkippingResourceChecker' => $vendorDir . '/symfony/config/Resource/SkippingResourceChecker.php',
    'Symfony\\Component\\Config\\Util\\Exception\\InvalidXmlException' => $vendorDir . '/symfony/config/Util/Exception/InvalidXmlException.php',
    'Symfony\\Component\\Config\\Util\\Exception\\XmlParsingException' => $vendorDir . '/symfony/config/Util/Exception/XmlParsingException.php',
    'Symfony\\Component\\Config\\Util\\XmlUtils' => $vendorDir . '/symfony/config/Util/XmlUtils.php',
    'Symfony\\Component\\Console\\Application' => $vendorDir . '/symfony/console/Application.php',
    'Symfony\\Component\\Console\\Attribute\\Argument' => $vendorDir . '/symfony/console/Attribute/Argument.php',
    'Symfony\\Component\\Console\\Attribute\\AsCommand' => $vendorDir . '/symfony/console/Attribute/AsCommand.php',
    'Symfony\\Component\\Console\\Attribute\\Ask' => $vendorDir . '/symfony/console/Attribute/Ask.php',
    'Symfony\\Component\\Console\\Attribute\\Interact' => $vendorDir . '/symfony/console/Attribute/Interact.php',
    'Symfony\\Component\\Console\\Attribute\\InteractiveAttributeInterface' => $vendorDir . '/symfony/console/Attribute/InteractiveAttributeInterface.php',
    'Symfony\\Component\\Console\\Attribute\\MapInput' => $vendorDir . '/symfony/console/Attribute/MapInput.php',
    'Symfony\\Component\\Console\\Attribute\\Option' => $vendorDir . '/symfony/console/Attribute/Option.php',
    'Symfony\\Component\\Console\\Attribute\\Reflection\\ReflectionMember' => $vendorDir . '/symfony/console/Attribute/Reflection/ReflectionMember.php',
    'Symfony\\Component\\Console\\CI\\GithubActionReporter' => $vendorDir . '/symfony/console/CI/GithubActionReporter.php',
    'Symfony\\Component\\Console\\Color' => $vendorDir . '/symfony/console/Color.php',
    'Symfony\\Component\\Console\\CommandLoader\\CommandLoaderInterface' => $vendorDir . '/symfony/console/CommandLoader/CommandLoaderInterface.php',
    'Symfony\\Component\\Console\\CommandLoader\\ContainerCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/ContainerCommandLoader.php',
    'Symfony\\Component\\Console\\CommandLoader\\FactoryCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/FactoryCommandLoader.php',
    'Symfony\\Component\\Console\\Command\\Command' => $vendorDir . '/symfony/console/Command/Command.php',
    'Symfony\\Component\\Console\\Command\\CompleteCommand' => $vendorDir . '/symfony/console/Command/CompleteCommand.php',
    'Symfony\\Component\\Console\\Command\\DumpCompletionCommand' => $vendorDir . '/symfony/console/Command/DumpCompletionCommand.php',
    'Symfony\\Component\\Console\\Command\\HelpCommand' => $vendorDir . '/symfony/console/Command/HelpCommand.php',
    'Symfony\\Component\\Console\\Command\\InvokableCommand' => $vendorDir . '/symfony/console/Command/InvokableCommand.php',
    'Symfony\\Component\\Console\\Command\\LazyCommand' => $vendorDir . '/symfony/console/Command/LazyCommand.php',
    'Symfony\\Component\\Console\\Command\\ListCommand' => $vendorDir . '/symfony/console/Command/ListCommand.php',
    'Symfony\\Component\\Console\\Command\\LockableTrait' => $vendorDir . '/symfony/console/Command/LockableTrait.php',
    'Symfony\\Component\\Console\\Command\\SignalableCommandInterface' => $vendorDir . '/symfony/console/Command/SignalableCommandInterface.php',
    'Symfony\\Component\\Console\\Command\\TraceableCommand' => $vendorDir . '/symfony/console/Command/TraceableCommand.php',
    'Symfony\\Component\\Console\\Completion\\CompletionInput' => $vendorDir . '/symfony/console/Completion/CompletionInput.php',
    'Symfony\\Component\\Console\\Completion\\CompletionSuggestions' => $vendorDir . '/symfony/console/Completion/CompletionSuggestions.php',
    'Symfony\\Component\\Console\\Completion\\Output\\BashCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/BashCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\CompletionOutputInterface' => $vendorDir . '/symfony/console/Completion/Output/CompletionOutputInterface.php',
    'Symfony\\Component\\Console\\Completion\\Output\\FishCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/FishCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\ZshCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/ZshCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Suggestion' => $vendorDir . '/symfony/console/Completion/Suggestion.php',
    'Symfony\\Component\\Console\\ConsoleEvents' => $vendorDir . '/symfony/console/ConsoleEvents.php',
    'Symfony\\Component\\Console\\Cursor' => $vendorDir . '/symfony/console/Cursor.php',
    'Symfony\\Component\\Console\\DataCollector\\CommandDataCollector' => $vendorDir . '/symfony/console/DataCollector/CommandDataCollector.php',
    'Symfony\\Component\\Console\\Debug\\CliRequest' => $vendorDir . '/symfony/console/Debug/CliRequest.php',
    'Symfony\\Component\\Console\\DependencyInjection\\AddConsoleCommandPass' => $vendorDir . '/symfony/console/DependencyInjection/AddConsoleCommandPass.php',
    'Symfony\\Component\\Console\\Descriptor\\ApplicationDescription' => $vendorDir . '/symfony/console/Descriptor/ApplicationDescription.php',
    'Symfony\\Component\\Console\\Descriptor\\Descriptor' => $vendorDir . '/symfony/console/Descriptor/Descriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\DescriptorInterface' => $vendorDir . '/symfony/console/Descriptor/DescriptorInterface.php',
    'Symfony\\Component\\Console\\Descriptor\\JsonDescriptor' => $vendorDir . '/symfony/console/Descriptor/JsonDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\MarkdownDescriptor' => $vendorDir . '/symfony/console/Descriptor/MarkdownDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\ReStructuredTextDescriptor' => $vendorDir . '/symfony/console/Descriptor/ReStructuredTextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\TextDescriptor' => $vendorDir . '/symfony/console/Descriptor/TextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\XmlDescriptor' => $vendorDir . '/symfony/console/Descriptor/XmlDescriptor.php',
    'Symfony\\Component\\Console\\EventListener\\ErrorListener' => $vendorDir . '/symfony/console/EventListener/ErrorListener.php',
    'Symfony\\Component\\Console\\Event\\ConsoleAlarmEvent' => $vendorDir . '/symfony/console/Event/ConsoleAlarmEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleCommandEvent' => $vendorDir . '/symfony/console/Event/ConsoleCommandEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleErrorEvent' => $vendorDir . '/symfony/console/Event/ConsoleErrorEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleEvent' => $vendorDir . '/symfony/console/Event/ConsoleEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleSignalEvent' => $vendorDir . '/symfony/console/Event/ConsoleSignalEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleTerminateEvent' => $vendorDir . '/symfony/console/Event/ConsoleTerminateEvent.php',
    'Symfony\\Component\\Console\\Exception\\CommandNotFoundException' => $vendorDir . '/symfony/console/Exception/CommandNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/console/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Console\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/console/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Console\\Exception\\InvalidOptionException' => $vendorDir . '/symfony/console/Exception/InvalidOptionException.php',
    'Symfony\\Component\\Console\\Exception\\LogicException' => $vendorDir . '/symfony/console/Exception/LogicException.php',
    'Symfony\\Component\\Console\\Exception\\MissingInputException' => $vendorDir . '/symfony/console/Exception/MissingInputException.php',
    'Symfony\\Component\\Console\\Exception\\NamespaceNotFoundException' => $vendorDir . '/symfony/console/Exception/NamespaceNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\RunCommandFailedException' => $vendorDir . '/symfony/console/Exception/RunCommandFailedException.php',
    'Symfony\\Component\\Console\\Exception\\RuntimeException' => $vendorDir . '/symfony/console/Exception/RuntimeException.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatter' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatter' => $vendorDir . '/symfony/console/Formatter/OutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleStack' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleStack.php',
    'Symfony\\Component\\Console\\Formatter\\WrappableOutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/WrappableOutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Helper\\DebugFormatterHelper' => $vendorDir . '/symfony/console/Helper/DebugFormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\DescriptorHelper' => $vendorDir . '/symfony/console/Helper/DescriptorHelper.php',
    'Symfony\\Component\\Console\\Helper\\Dumper' => $vendorDir . '/symfony/console/Helper/Dumper.php',
    'Symfony\\Component\\Console\\Helper\\FormatterHelper' => $vendorDir . '/symfony/console/Helper/FormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\Helper' => $vendorDir . '/symfony/console/Helper/Helper.php',
    'Symfony\\Component\\Console\\Helper\\HelperInterface' => $vendorDir . '/symfony/console/Helper/HelperInterface.php',
    'Symfony\\Component\\Console\\Helper\\HelperSet' => $vendorDir . '/symfony/console/Helper/HelperSet.php',
    'Symfony\\Component\\Console\\Helper\\InputAwareHelper' => $vendorDir . '/symfony/console/Helper/InputAwareHelper.php',
    'Symfony\\Component\\Console\\Helper\\OutputWrapper' => $vendorDir . '/symfony/console/Helper/OutputWrapper.php',
    'Symfony\\Component\\Console\\Helper\\ProcessHelper' => $vendorDir . '/symfony/console/Helper/ProcessHelper.php',
    'Symfony\\Component\\Console\\Helper\\ProgressBar' => $vendorDir . '/symfony/console/Helper/ProgressBar.php',
    'Symfony\\Component\\Console\\Helper\\ProgressIndicator' => $vendorDir . '/symfony/console/Helper/ProgressIndicator.php',
    'Symfony\\Component\\Console\\Helper\\QuestionHelper' => $vendorDir . '/symfony/console/Helper/QuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\SymfonyQuestionHelper' => $vendorDir . '/symfony/console/Helper/SymfonyQuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\Table' => $vendorDir . '/symfony/console/Helper/Table.php',
    'Symfony\\Component\\Console\\Helper\\TableCell' => $vendorDir . '/symfony/console/Helper/TableCell.php',
    'Symfony\\Component\\Console\\Helper\\TableCellStyle' => $vendorDir . '/symfony/console/Helper/TableCellStyle.php',
    'Symfony\\Component\\Console\\Helper\\TableRows' => $vendorDir . '/symfony/console/Helper/TableRows.php',
    'Symfony\\Component\\Console\\Helper\\TableSeparator' => $vendorDir . '/symfony/console/Helper/TableSeparator.php',
    'Symfony\\Component\\Console\\Helper\\TableStyle' => $vendorDir . '/symfony/console/Helper/TableStyle.php',
    'Symfony\\Component\\Console\\Helper\\TerminalInputHelper' => $vendorDir . '/symfony/console/Helper/TerminalInputHelper.php',
    'Symfony\\Component\\Console\\Helper\\TreeHelper' => $vendorDir . '/symfony/console/Helper/TreeHelper.php',
    'Symfony\\Component\\Console\\Helper\\TreeNode' => $vendorDir . '/symfony/console/Helper/TreeNode.php',
    'Symfony\\Component\\Console\\Helper\\TreeStyle' => $vendorDir . '/symfony/console/Helper/TreeStyle.php',
    'Symfony\\Component\\Console\\Input\\ArgvInput' => $vendorDir . '/symfony/console/Input/ArgvInput.php',
    'Symfony\\Component\\Console\\Input\\ArrayInput' => $vendorDir . '/symfony/console/Input/ArrayInput.php',
    'Symfony\\Component\\Console\\Input\\Input' => $vendorDir . '/symfony/console/Input/Input.php',
    'Symfony\\Component\\Console\\Input\\InputArgument' => $vendorDir . '/symfony/console/Input/InputArgument.php',
    'Symfony\\Component\\Console\\Input\\InputAwareInterface' => $vendorDir . '/symfony/console/Input/InputAwareInterface.php',
    'Symfony\\Component\\Console\\Input\\InputDefinition' => $vendorDir . '/symfony/console/Input/InputDefinition.php',
    'Symfony\\Component\\Console\\Input\\InputInterface' => $vendorDir . '/symfony/console/Input/InputInterface.php',
    'Symfony\\Component\\Console\\Input\\InputOption' => $vendorDir . '/symfony/console/Input/InputOption.php',
    'Symfony\\Component\\Console\\Input\\StreamableInputInterface' => $vendorDir . '/symfony/console/Input/StreamableInputInterface.php',
    'Symfony\\Component\\Console\\Input\\StringInput' => $vendorDir . '/symfony/console/Input/StringInput.php',
    'Symfony\\Component\\Console\\Interaction\\Interaction' => $vendorDir . '/symfony/console/Interaction/Interaction.php',
    'Symfony\\Component\\Console\\Logger\\ConsoleLogger' => $vendorDir . '/symfony/console/Logger/ConsoleLogger.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandContext' => $vendorDir . '/symfony/console/Messenger/RunCommandContext.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessage' => $vendorDir . '/symfony/console/Messenger/RunCommandMessage.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessageHandler' => $vendorDir . '/symfony/console/Messenger/RunCommandMessageHandler.php',
    'Symfony\\Component\\Console\\Output\\AnsiColorMode' => $vendorDir . '/symfony/console/Output/AnsiColorMode.php',
    'Symfony\\Component\\Console\\Output\\BufferedOutput' => $vendorDir . '/symfony/console/Output/BufferedOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutput' => $vendorDir . '/symfony/console/Output/ConsoleOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutputInterface' => $vendorDir . '/symfony/console/Output/ConsoleOutputInterface.php',
    'Symfony\\Component\\Console\\Output\\ConsoleSectionOutput' => $vendorDir . '/symfony/console/Output/ConsoleSectionOutput.php',
    'Symfony\\Component\\Console\\Output\\NullOutput' => $vendorDir . '/symfony/console/Output/NullOutput.php',
    'Symfony\\Component\\Console\\Output\\Output' => $vendorDir . '/symfony/console/Output/Output.php',
    'Symfony\\Component\\Console\\Output\\OutputInterface' => $vendorDir . '/symfony/console/Output/OutputInterface.php',
    'Symfony\\Component\\Console\\Output\\StreamOutput' => $vendorDir . '/symfony/console/Output/StreamOutput.php',
    'Symfony\\Component\\Console\\Output\\TrimmedBufferOutput' => $vendorDir . '/symfony/console/Output/TrimmedBufferOutput.php',
    'Symfony\\Component\\Console\\Question\\ChoiceQuestion' => $vendorDir . '/symfony/console/Question/ChoiceQuestion.php',
    'Symfony\\Component\\Console\\Question\\ConfirmationQuestion' => $vendorDir . '/symfony/console/Question/ConfirmationQuestion.php',
    'Symfony\\Component\\Console\\Question\\Question' => $vendorDir . '/symfony/console/Question/Question.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalMap' => $vendorDir . '/symfony/console/SignalRegistry/SignalMap.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalRegistry' => $vendorDir . '/symfony/console/SignalRegistry/SignalRegistry.php',
    'Symfony\\Component\\Console\\SingleCommandApplication' => $vendorDir . '/symfony/console/SingleCommandApplication.php',
    'Symfony\\Component\\Console\\Style\\OutputStyle' => $vendorDir . '/symfony/console/Style/OutputStyle.php',
    'Symfony\\Component\\Console\\Style\\StyleInterface' => $vendorDir . '/symfony/console/Style/StyleInterface.php',
    'Symfony\\Component\\Console\\Style\\SymfonyStyle' => $vendorDir . '/symfony/console/Style/SymfonyStyle.php',
    'Symfony\\Component\\Console\\Terminal' => $vendorDir . '/symfony/console/Terminal.php',
    'Symfony\\Component\\Console\\Tester\\ApplicationTester' => $vendorDir . '/symfony/console/Tester/ApplicationTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandCompletionTester' => $vendorDir . '/symfony/console/Tester/CommandCompletionTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandTester' => $vendorDir . '/symfony/console/Tester/CommandTester.php',
    'Symfony\\Component\\Console\\Tester\\Constraint\\CommandIsSuccessful' => $vendorDir . '/symfony/console/Tester/Constraint/CommandIsSuccessful.php',
    'Symfony\\Component\\Console\\Tester\\TesterTrait' => $vendorDir . '/symfony/console/Tester/TesterTrait.php',
    'Symfony\\Component\\DependencyInjection\\Alias' => $vendorDir . '/symfony/dependency-injection/Alias.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\AbstractArgument' => $vendorDir . '/symfony/dependency-injection/Argument/AbstractArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ArgumentInterface' => $vendorDir . '/symfony/dependency-injection/Argument/ArgumentInterface.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ArgumentTrait' => $vendorDir . '/symfony/dependency-injection/Argument/ArgumentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\BoundArgument' => $vendorDir . '/symfony/dependency-injection/Argument/BoundArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\IteratorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/IteratorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\LazyClosure' => $vendorDir . '/symfony/dependency-injection/Argument/LazyClosure.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\RewindableGenerator' => $vendorDir . '/symfony/dependency-injection/Argument/RewindableGenerator.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceClosureArgument' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceClosureArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceLocator' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceLocator.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceLocatorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceLocatorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\TaggedIteratorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/TaggedIteratorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsAlias' => $vendorDir . '/symfony/dependency-injection/Attribute/AsAlias.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsDecorator' => $vendorDir . '/symfony/dependency-injection/Attribute/AsDecorator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsTaggedItem' => $vendorDir . '/symfony/dependency-injection/Attribute/AsTaggedItem.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Autoconfigure' => $vendorDir . '/symfony/dependency-injection/Attribute/Autoconfigure.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutoconfigureResourceTag' => $vendorDir . '/symfony/dependency-injection/Attribute/AutoconfigureResourceTag.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutoconfigureTag' => $vendorDir . '/symfony/dependency-injection/Attribute/AutoconfigureTag.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Autowire' => $vendorDir . '/symfony/dependency-injection/Attribute/Autowire.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireCallable' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireCallable.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireDecorated' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireDecorated.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireInline' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireInline.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireIterator' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireIterator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireLocator' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireLocator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireMethodOf' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireMethodOf.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireServiceClosure' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireServiceClosure.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Exclude' => $vendorDir . '/symfony/dependency-injection/Attribute/Exclude.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Lazy' => $vendorDir . '/symfony/dependency-injection/Attribute/Lazy.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\TaggedIterator' => $vendorDir . '/symfony/dependency-injection/Attribute/TaggedIterator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\TaggedLocator' => $vendorDir . '/symfony/dependency-injection/Attribute/TaggedLocator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Target' => $vendorDir . '/symfony/dependency-injection/Attribute/Target.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\When' => $vendorDir . '/symfony/dependency-injection/Attribute/When.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\WhenNot' => $vendorDir . '/symfony/dependency-injection/Attribute/WhenNot.php',
    'Symfony\\Component\\DependencyInjection\\ChildDefinition' => $vendorDir . '/symfony/dependency-injection/ChildDefinition.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AbstractRecursivePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AbstractRecursivePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AliasDeprecatedPublicServicesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AliasDeprecatedPublicServicesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AnalyzeServiceReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AnalyzeServiceReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AttributeAutoconfigurationPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AttributeAutoconfigurationPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutoAliasServicePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutoAliasServicePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireAsDecoratorPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireAsDecoratorPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowirePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowirePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireRequiredMethodsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireRequiredMethodsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireRequiredPropertiesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireRequiredPropertiesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckAliasValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckAliasValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckArgumentsValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckArgumentsValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckCircularReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckCircularReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckDefinitionValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckDefinitionValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckExceptionOnInvalidReferenceBehaviorPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckExceptionOnInvalidReferenceBehaviorPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckFactoryBuilderCircularReferencePass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckFactoryBuilderCircularReferencePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckReferenceValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckReferenceValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckTypeDeclarationsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckTypeDeclarationsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\Compiler' => $vendorDir . '/symfony/dependency-injection/Compiler/Compiler.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CompilerPassInterface' => $vendorDir . '/symfony/dependency-injection/Compiler/CompilerPassInterface.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\DecoratorServicePass' => $vendorDir . '/symfony/dependency-injection/Compiler/DecoratorServicePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\DefinitionErrorExceptionPass' => $vendorDir . '/symfony/dependency-injection/Compiler/DefinitionErrorExceptionPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ExtensionCompilerPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ExtensionCompilerPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\InlineServiceDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/InlineServiceDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\MergeExtensionConfigurationPass' => $vendorDir . '/symfony/dependency-injection/Compiler/MergeExtensionConfigurationPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\PassConfig' => $vendorDir . '/symfony/dependency-injection/Compiler/PassConfig.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\PriorityTaggedServiceTrait' => $vendorDir . '/symfony/dependency-injection/Compiler/PriorityTaggedServiceTrait.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterAutoconfigureAttributesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterAutoconfigureAttributesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterEnvVarProcessorsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterEnvVarProcessorsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterReverseContainerPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterReverseContainerPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterServiceSubscribersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterServiceSubscribersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveAbstractDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveAbstractDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveBuildParametersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveBuildParametersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemovePrivateAliasesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemovePrivateAliasesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveUnusedDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveUnusedDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ReplaceAliasByActualDefinitionPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ReplaceAliasByActualDefinitionPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveAutowireInlineAttributesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveAutowireInlineAttributesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveBindingsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveBindingsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveChildDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveChildDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveClassPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveClassPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveDecoratorStackPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveDecoratorStackPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveEnvPlaceholdersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveEnvPlaceholdersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveFactoryClassPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveFactoryClassPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveHotPathPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveHotPathPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveInstanceofConditionalsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveInstanceofConditionalsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveInvalidReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveInvalidReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveNamedArgumentsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveNamedArgumentsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveNoPreloadPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveNoPreloadPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveParameterPlaceHoldersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveParameterPlaceHoldersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveReferencesToAliasesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveReferencesToAliasesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveServiceSubscribersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveServiceSubscribersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveTaggedIteratorArgumentPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveTaggedIteratorArgumentPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceLocatorTagPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceLocatorTagPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraph' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraph.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraphEdge' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraphEdge.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraphNode' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraphNode.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ValidateEnvPlaceholdersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ValidateEnvPlaceholdersPass.php',
    'Symfony\\Component\\DependencyInjection\\Config\\ContainerParametersResource' => $vendorDir . '/symfony/dependency-injection/Config/ContainerParametersResource.php',
    'Symfony\\Component\\DependencyInjection\\Config\\ContainerParametersResourceChecker' => $vendorDir . '/symfony/dependency-injection/Config/ContainerParametersResourceChecker.php',
    'Symfony\\Component\\DependencyInjection\\Container' => $vendorDir . '/symfony/dependency-injection/Container.php',
    'Symfony\\Component\\DependencyInjection\\ContainerBuilder' => $vendorDir . '/symfony/dependency-injection/ContainerBuilder.php',
    'Symfony\\Component\\DependencyInjection\\ContainerInterface' => $vendorDir . '/symfony/dependency-injection/ContainerInterface.php',
    'Symfony\\Component\\DependencyInjection\\Definition' => $vendorDir . '/symfony/dependency-injection/Definition.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\Dumper' => $vendorDir . '/symfony/dependency-injection/Dumper/Dumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\DumperInterface' => $vendorDir . '/symfony/dependency-injection/Dumper/DumperInterface.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\GraphvizDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/GraphvizDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\PhpDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/PhpDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\Preloader' => $vendorDir . '/symfony/dependency-injection/Dumper/Preloader.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\XmlDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/XmlDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\YamlDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/YamlDumper.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarLoaderInterface' => $vendorDir . '/symfony/dependency-injection/EnvVarLoaderInterface.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarProcessor' => $vendorDir . '/symfony/dependency-injection/EnvVarProcessor.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarProcessorInterface' => $vendorDir . '/symfony/dependency-injection/EnvVarProcessorInterface.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\AutoconfigureFailedException' => $vendorDir . '/symfony/dependency-injection/Exception/AutoconfigureFailedException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\AutowiringFailedException' => $vendorDir . '/symfony/dependency-injection/Exception/AutowiringFailedException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/dependency-injection/Exception/BadMethodCallException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\EmptyParameterValueException' => $vendorDir . '/symfony/dependency-injection/Exception/EmptyParameterValueException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\EnvNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/EnvNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\EnvParameterException' => $vendorDir . '/symfony/dependency-injection/Exception/EnvParameterException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/dependency-injection/Exception/ExceptionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/dependency-injection/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\InvalidParameterTypeException' => $vendorDir . '/symfony/dependency-injection/Exception/InvalidParameterTypeException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\LogicException' => $vendorDir . '/symfony/dependency-injection/Exception/LogicException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\OutOfBoundsException' => $vendorDir . '/symfony/dependency-injection/Exception/OutOfBoundsException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ParameterCircularReferenceException' => $vendorDir . '/symfony/dependency-injection/Exception/ParameterCircularReferenceException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ParameterNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/ParameterNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\RuntimeException' => $vendorDir . '/symfony/dependency-injection/Exception/RuntimeException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ServiceCircularReferenceException' => $vendorDir . '/symfony/dependency-injection/Exception/ServiceCircularReferenceException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ServiceNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/ServiceNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\ExpressionLanguage' => $vendorDir . '/symfony/dependency-injection/ExpressionLanguage.php',
    'Symfony\\Component\\DependencyInjection\\ExpressionLanguageProvider' => $vendorDir . '/symfony/dependency-injection/ExpressionLanguageProvider.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\AbstractExtension' => $vendorDir . '/symfony/dependency-injection/Extension/AbstractExtension.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ConfigurableExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ConfigurableExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ConfigurationExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ConfigurationExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\Extension' => $vendorDir . '/symfony/dependency-injection/Extension/Extension.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ExtensionTrait' => $vendorDir . '/symfony/dependency-injection/Extension/ExtensionTrait.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\PrependExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/PrependExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\InstantiatorInterface' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/InstantiatorInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\LazyServiceInstantiator' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/LazyServiceInstantiator.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\RealServiceInstantiator' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/RealServiceInstantiator.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\DumperInterface' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/DumperInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\LazyServiceDumper' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/LazyServiceDumper.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\NullDumper' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/NullDumper.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\ClosureLoader' => $vendorDir . '/symfony/dependency-injection/Loader/ClosureLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AbstractConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AbstractConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AbstractServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AbstractServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AliasConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AliasConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AppReference' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AppReference.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ClosureReferenceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ClosureReferenceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ContainerConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ContainerConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\DefaultsConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/DefaultsConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\EnvConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/EnvConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ExpressionConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ExpressionConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\FromCallableConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/FromCallableConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\InlineServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/InlineServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\InstanceofConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/InstanceofConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ParametersConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ParametersConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\PrototypeConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/PrototypeConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ReferenceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ReferenceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ServicesConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ServicesConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AbstractTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AbstractTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ArgumentTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ArgumentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AutoconfigureTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AutoconfigureTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AutowireTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AutowireTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\BindTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/BindTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\CallTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/CallTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ClassTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ClassTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ConfiguratorTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ConfiguratorTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ConstructorTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ConstructorTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\DecorateTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/DecorateTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\DeprecateTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/DeprecateTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FactoryTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FactoryTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FileTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FileTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FromCallableTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FromCallableTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\LazyTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/LazyTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ParentTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ParentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\PropertyTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/PropertyTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\PublicTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/PublicTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ShareTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ShareTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\SyntheticTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/SyntheticTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\TagTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/TagTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\DirectoryLoader' => $vendorDir . '/symfony/dependency-injection/Loader/DirectoryLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\FileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/FileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/GlobFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\IniFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/IniFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/PhpFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\UndefinedExtensionHandler' => $vendorDir . '/symfony/dependency-injection/Loader/UndefinedExtensionHandler.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/XmlFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/YamlFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Parameter' => $vendorDir . '/symfony/dependency-injection/Parameter.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ContainerBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ContainerBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ContainerBagInterface' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ContainerBagInterface.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\EnvPlaceholderParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/EnvPlaceholderParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\FrozenParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/FrozenParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ParameterBagInterface' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ParameterBagInterface.php',
    'Symfony\\Component\\DependencyInjection\\Reference' => $vendorDir . '/symfony/dependency-injection/Reference.php',
    'Symfony\\Component\\DependencyInjection\\ReverseContainer' => $vendorDir . '/symfony/dependency-injection/ReverseContainer.php',
    'Symfony\\Component\\DependencyInjection\\ServiceLocator' => $vendorDir . '/symfony/dependency-injection/ServiceLocator.php',
    'Symfony\\Component\\DependencyInjection\\StaticEnvVarLoader' => $vendorDir . '/symfony/dependency-injection/StaticEnvVarLoader.php',
    'Symfony\\Component\\DependencyInjection\\TaggedContainerInterface' => $vendorDir . '/symfony/dependency-injection/TaggedContainerInterface.php',
    'Symfony\\Component\\DependencyInjection\\TypedReference' => $vendorDir . '/symfony/dependency-injection/TypedReference.php',
    'Symfony\\Component\\DependencyInjection\\Variable' => $vendorDir . '/symfony/dependency-injection/Variable.php',
    'Symfony\\Component\\ErrorHandler\\BufferingLogger' => $vendorDir . '/symfony/error-handler/BufferingLogger.php',
    'Symfony\\Component\\ErrorHandler\\Command\\ErrorDumpCommand' => $vendorDir . '/symfony/error-handler/Command/ErrorDumpCommand.php',
    'Symfony\\Component\\ErrorHandler\\Debug' => $vendorDir . '/symfony/error-handler/Debug.php',
    'Symfony\\Component\\ErrorHandler\\DebugClassLoader' => $vendorDir . '/symfony/error-handler/DebugClassLoader.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ClassNotFoundErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ClassNotFoundErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ErrorEnhancerInterface' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ErrorEnhancerInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedFunctionErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedFunctionErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedMethodErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedMethodErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorHandler' => $vendorDir . '/symfony/error-handler/ErrorHandler.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\CliErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/CliErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\ErrorRendererInterface' => $vendorDir . '/symfony/error-handler/ErrorRenderer/ErrorRendererInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\FileLinkFormatter' => $vendorDir . '/symfony/error-handler/ErrorRenderer/FileLinkFormatter.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\HtmlErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/HtmlErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\SerializerErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/SerializerErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\Error\\ClassNotFoundError' => $vendorDir . '/symfony/error-handler/Error/ClassNotFoundError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\FatalError' => $vendorDir . '/symfony/error-handler/Error/FatalError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\OutOfMemoryError' => $vendorDir . '/symfony/error-handler/Error/OutOfMemoryError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedFunctionError' => $vendorDir . '/symfony/error-handler/Error/UndefinedFunctionError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedMethodError' => $vendorDir . '/symfony/error-handler/Error/UndefinedMethodError.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\FlattenException' => $vendorDir . '/symfony/error-handler/Exception/FlattenException.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\SilencedErrorContext' => $vendorDir . '/symfony/error-handler/Exception/SilencedErrorContext.php',
    'Symfony\\Component\\ErrorHandler\\Internal\\TentativeTypes' => $vendorDir . '/symfony/error-handler/Internal/TentativeTypes.php',
    'Symfony\\Component\\ErrorHandler\\ThrowableUtils' => $vendorDir . '/symfony/error-handler/ThrowableUtils.php',
    'Symfony\\Component\\EventDispatcher\\Attribute\\AsEventListener' => $vendorDir . '/symfony/event-dispatcher/Attribute/AsEventListener.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\WrappedListener' => $vendorDir . '/symfony/event-dispatcher/Debug/WrappedListener.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\AddEventAliasesPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/AddEventAliasesPass.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\RegisterListenersPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/RegisterListenersPass.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcher' => $vendorDir . '/symfony/event-dispatcher/EventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher/EventDispatcherInterface.php',
    'Symfony\\Component\\EventDispatcher\\EventSubscriberInterface' => $vendorDir . '/symfony/event-dispatcher/EventSubscriberInterface.php',
    'Symfony\\Component\\EventDispatcher\\GenericEvent' => $vendorDir . '/symfony/event-dispatcher/GenericEvent.php',
    'Symfony\\Component\\EventDispatcher\\ImmutableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/ImmutableEventDispatcher.php',
    'Symfony\\Component\\ExpressionLanguage\\Compiler' => $vendorDir . '/symfony/expression-language/Compiler.php',
    'Symfony\\Component\\ExpressionLanguage\\Expression' => $vendorDir . '/symfony/expression-language/Expression.php',
    'Symfony\\Component\\ExpressionLanguage\\ExpressionFunction' => $vendorDir . '/symfony/expression-language/ExpressionFunction.php',
    'Symfony\\Component\\ExpressionLanguage\\ExpressionFunctionProviderInterface' => $vendorDir . '/symfony/expression-language/ExpressionFunctionProviderInterface.php',
    'Symfony\\Component\\ExpressionLanguage\\ExpressionLanguage' => $vendorDir . '/symfony/expression-language/ExpressionLanguage.php',
    'Symfony\\Component\\ExpressionLanguage\\Lexer' => $vendorDir . '/symfony/expression-language/Lexer.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\ArgumentsNode' => $vendorDir . '/symfony/expression-language/Node/ArgumentsNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\ArrayNode' => $vendorDir . '/symfony/expression-language/Node/ArrayNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\BinaryNode' => $vendorDir . '/symfony/expression-language/Node/BinaryNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\ConditionalNode' => $vendorDir . '/symfony/expression-language/Node/ConditionalNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\ConstantNode' => $vendorDir . '/symfony/expression-language/Node/ConstantNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\FunctionNode' => $vendorDir . '/symfony/expression-language/Node/FunctionNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\GetAttrNode' => $vendorDir . '/symfony/expression-language/Node/GetAttrNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\NameNode' => $vendorDir . '/symfony/expression-language/Node/NameNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\Node' => $vendorDir . '/symfony/expression-language/Node/Node.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\NullCoalesceNode' => $vendorDir . '/symfony/expression-language/Node/NullCoalesceNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\NullCoalescedNameNode' => $vendorDir . '/symfony/expression-language/Node/NullCoalescedNameNode.php',
    'Symfony\\Component\\ExpressionLanguage\\Node\\UnaryNode' => $vendorDir . '/symfony/expression-language/Node/UnaryNode.php',
    'Symfony\\Component\\ExpressionLanguage\\ParsedExpression' => $vendorDir . '/symfony/expression-language/ParsedExpression.php',
    'Symfony\\Component\\ExpressionLanguage\\Parser' => $vendorDir . '/symfony/expression-language/Parser.php',
    'Symfony\\Component\\ExpressionLanguage\\SerializedParsedExpression' => $vendorDir . '/symfony/expression-language/SerializedParsedExpression.php',
    'Symfony\\Component\\ExpressionLanguage\\SyntaxError' => $vendorDir . '/symfony/expression-language/SyntaxError.php',
    'Symfony\\Component\\ExpressionLanguage\\Token' => $vendorDir . '/symfony/expression-language/Token.php',
    'Symfony\\Component\\ExpressionLanguage\\TokenStream' => $vendorDir . '/symfony/expression-language/TokenStream.php',
    'Symfony\\Component\\Filesystem\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/filesystem/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Filesystem\\Exception\\FileNotFoundException' => $vendorDir . '/symfony/filesystem/Exception/FileNotFoundException.php',
    'Symfony\\Component\\Filesystem\\Exception\\IOException' => $vendorDir . '/symfony/filesystem/Exception/IOException.php',
    'Symfony\\Component\\Filesystem\\Exception\\IOExceptionInterface' => $vendorDir . '/symfony/filesystem/Exception/IOExceptionInterface.php',
    'Symfony\\Component\\Filesystem\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/filesystem/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Filesystem\\Exception\\RuntimeException' => $vendorDir . '/symfony/filesystem/Exception/RuntimeException.php',
    'Symfony\\Component\\Filesystem\\Filesystem' => $vendorDir . '/symfony/filesystem/Filesystem.php',
    'Symfony\\Component\\Filesystem\\Path' => $vendorDir . '/symfony/filesystem/Path.php',
    'Symfony\\Component\\Finder\\Comparator\\Comparator' => $vendorDir . '/symfony/finder/Comparator/Comparator.php',
    'Symfony\\Component\\Finder\\Comparator\\DateComparator' => $vendorDir . '/symfony/finder/Comparator/DateComparator.php',
    'Symfony\\Component\\Finder\\Comparator\\NumberComparator' => $vendorDir . '/symfony/finder/Comparator/NumberComparator.php',
    'Symfony\\Component\\Finder\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/finder/Exception/AccessDeniedException.php',
    'Symfony\\Component\\Finder\\Exception\\DirectoryNotFoundException' => $vendorDir . '/symfony/finder/Exception/DirectoryNotFoundException.php',
    'Symfony\\Component\\Finder\\Finder' => $vendorDir . '/symfony/finder/Finder.php',
    'Symfony\\Component\\Finder\\Gitignore' => $vendorDir . '/symfony/finder/Gitignore.php',
    'Symfony\\Component\\Finder\\Glob' => $vendorDir . '/symfony/finder/Glob.php',
    'Symfony\\Component\\Finder\\Iterator\\CustomFilterIterator' => $vendorDir . '/symfony/finder/Iterator/CustomFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DateRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DateRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DepthRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DepthRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\ExcludeDirectoryFilterIterator' => $vendorDir . '/symfony/finder/Iterator/ExcludeDirectoryFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FileTypeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FileTypeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilecontentFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilecontentFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilenameFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilenameFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\LazyIterator' => $vendorDir . '/symfony/finder/Iterator/LazyIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\MultiplePcreFilterIterator' => $vendorDir . '/symfony/finder/Iterator/MultiplePcreFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\PathFilterIterator' => $vendorDir . '/symfony/finder/Iterator/PathFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\RecursiveDirectoryIterator' => $vendorDir . '/symfony/finder/Iterator/RecursiveDirectoryIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SizeRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/SizeRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SortableIterator' => $vendorDir . '/symfony/finder/Iterator/SortableIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\VcsIgnoredFilterIterator' => $vendorDir . '/symfony/finder/Iterator/VcsIgnoredFilterIterator.php',
    'Symfony\\Component\\Finder\\SplFileInfo' => $vendorDir . '/symfony/finder/SplFileInfo.php',
    'Symfony\\Component\\HttpClient\\AmpHttpClient' => $vendorDir . '/symfony/http-client/AmpHttpClient.php',
    'Symfony\\Component\\HttpClient\\AsyncDecoratorTrait' => $vendorDir . '/symfony/http-client/AsyncDecoratorTrait.php',
    'Symfony\\Component\\HttpClient\\CachingHttpClient' => $vendorDir . '/symfony/http-client/CachingHttpClient.php',
    'Symfony\\Component\\HttpClient\\Caching\\Freshness' => $vendorDir . '/symfony/http-client/Caching/Freshness.php',
    'Symfony\\Component\\HttpClient\\Chunk\\DataChunk' => $vendorDir . '/symfony/http-client/Chunk/DataChunk.php',
    'Symfony\\Component\\HttpClient\\Chunk\\ErrorChunk' => $vendorDir . '/symfony/http-client/Chunk/ErrorChunk.php',
    'Symfony\\Component\\HttpClient\\Chunk\\FirstChunk' => $vendorDir . '/symfony/http-client/Chunk/FirstChunk.php',
    'Symfony\\Component\\HttpClient\\Chunk\\InformationalChunk' => $vendorDir . '/symfony/http-client/Chunk/InformationalChunk.php',
    'Symfony\\Component\\HttpClient\\Chunk\\LastChunk' => $vendorDir . '/symfony/http-client/Chunk/LastChunk.php',
    'Symfony\\Component\\HttpClient\\Chunk\\ServerSentEvent' => $vendorDir . '/symfony/http-client/Chunk/ServerSentEvent.php',
    'Symfony\\Component\\HttpClient\\CurlHttpClient' => $vendorDir . '/symfony/http-client/CurlHttpClient.php',
    'Symfony\\Component\\HttpClient\\DataCollector\\HttpClientDataCollector' => $vendorDir . '/symfony/http-client/DataCollector/HttpClientDataCollector.php',
    'Symfony\\Component\\HttpClient\\DecoratorTrait' => $vendorDir . '/symfony/http-client/DecoratorTrait.php',
    'Symfony\\Component\\HttpClient\\DependencyInjection\\HttpClientPass' => $vendorDir . '/symfony/http-client/DependencyInjection/HttpClientPass.php',
    'Symfony\\Component\\HttpClient\\EventSourceHttpClient' => $vendorDir . '/symfony/http-client/EventSourceHttpClient.php',
    'Symfony\\Component\\HttpClient\\Exception\\ChunkCacheItemNotFoundException' => $vendorDir . '/symfony/http-client/Exception/ChunkCacheItemNotFoundException.php',
    'Symfony\\Component\\HttpClient\\Exception\\ClientException' => $vendorDir . '/symfony/http-client/Exception/ClientException.php',
    'Symfony\\Component\\HttpClient\\Exception\\EventSourceException' => $vendorDir . '/symfony/http-client/Exception/EventSourceException.php',
    'Symfony\\Component\\HttpClient\\Exception\\HttpExceptionTrait' => $vendorDir . '/symfony/http-client/Exception/HttpExceptionTrait.php',
    'Symfony\\Component\\HttpClient\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/http-client/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\HttpClient\\Exception\\JsonException' => $vendorDir . '/symfony/http-client/Exception/JsonException.php',
    'Symfony\\Component\\HttpClient\\Exception\\RedirectionException' => $vendorDir . '/symfony/http-client/Exception/RedirectionException.php',
    'Symfony\\Component\\HttpClient\\Exception\\ServerException' => $vendorDir . '/symfony/http-client/Exception/ServerException.php',
    'Symfony\\Component\\HttpClient\\Exception\\TimeoutException' => $vendorDir . '/symfony/http-client/Exception/TimeoutException.php',
    'Symfony\\Component\\HttpClient\\Exception\\TransportException' => $vendorDir . '/symfony/http-client/Exception/TransportException.php',
    'Symfony\\Component\\HttpClient\\HttpClient' => $vendorDir . '/symfony/http-client/HttpClient.php',
    'Symfony\\Component\\HttpClient\\HttpClientTrait' => $vendorDir . '/symfony/http-client/HttpClientTrait.php',
    'Symfony\\Component\\HttpClient\\HttpOptions' => $vendorDir . '/symfony/http-client/HttpOptions.php',
    'Symfony\\Component\\HttpClient\\HttplugClient' => $vendorDir . '/symfony/http-client/HttplugClient.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpBodyV4' => $vendorDir . '/symfony/http-client/Internal/AmpBodyV4.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpBodyV5' => $vendorDir . '/symfony/http-client/Internal/AmpBodyV5.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpClientStateV4' => $vendorDir . '/symfony/http-client/Internal/AmpClientStateV4.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpClientStateV5' => $vendorDir . '/symfony/http-client/Internal/AmpClientStateV5.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpListenerV4' => $vendorDir . '/symfony/http-client/Internal/AmpListenerV4.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpListenerV5' => $vendorDir . '/symfony/http-client/Internal/AmpListenerV5.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpResolverV4' => $vendorDir . '/symfony/http-client/Internal/AmpResolverV4.php',
    'Symfony\\Component\\HttpClient\\Internal\\AmpResolverV5' => $vendorDir . '/symfony/http-client/Internal/AmpResolverV5.php',
    'Symfony\\Component\\HttpClient\\Internal\\Canary' => $vendorDir . '/symfony/http-client/Internal/Canary.php',
    'Symfony\\Component\\HttpClient\\Internal\\ClientState' => $vendorDir . '/symfony/http-client/Internal/ClientState.php',
    'Symfony\\Component\\HttpClient\\Internal\\CurlClientState' => $vendorDir . '/symfony/http-client/Internal/CurlClientState.php',
    'Symfony\\Component\\HttpClient\\Internal\\Dechunker' => $vendorDir . '/symfony/http-client/Internal/Dechunker.php',
    'Symfony\\Component\\HttpClient\\Internal\\DnsCache' => $vendorDir . '/symfony/http-client/Internal/DnsCache.php',
    'Symfony\\Component\\HttpClient\\Internal\\HttplugWaitLoop' => $vendorDir . '/symfony/http-client/Internal/HttplugWaitLoop.php',
    'Symfony\\Component\\HttpClient\\Internal\\NativeClientState' => $vendorDir . '/symfony/http-client/Internal/NativeClientState.php',
    'Symfony\\Component\\HttpClient\\Internal\\PushedResponse' => $vendorDir . '/symfony/http-client/Internal/PushedResponse.php',
    'Symfony\\Component\\HttpClient\\Messenger\\PingWebhookMessage' => $vendorDir . '/symfony/http-client/Messenger/PingWebhookMessage.php',
    'Symfony\\Component\\HttpClient\\Messenger\\PingWebhookMessageHandler' => $vendorDir . '/symfony/http-client/Messenger/PingWebhookMessageHandler.php',
    'Symfony\\Component\\HttpClient\\MockHttpClient' => $vendorDir . '/symfony/http-client/MockHttpClient.php',
    'Symfony\\Component\\HttpClient\\NativeHttpClient' => $vendorDir . '/symfony/http-client/NativeHttpClient.php',
    'Symfony\\Component\\HttpClient\\NoPrivateNetworkHttpClient' => $vendorDir . '/symfony/http-client/NoPrivateNetworkHttpClient.php',
    'Symfony\\Component\\HttpClient\\Psr18Client' => $vendorDir . '/symfony/http-client/Psr18Client.php',
    'Symfony\\Component\\HttpClient\\Response\\AmpResponseV4' => $vendorDir . '/symfony/http-client/Response/AmpResponseV4.php',
    'Symfony\\Component\\HttpClient\\Response\\AmpResponseV5' => $vendorDir . '/symfony/http-client/Response/AmpResponseV5.php',
    'Symfony\\Component\\HttpClient\\Response\\AsyncContext' => $vendorDir . '/symfony/http-client/Response/AsyncContext.php',
    'Symfony\\Component\\HttpClient\\Response\\AsyncResponse' => $vendorDir . '/symfony/http-client/Response/AsyncResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\CommonResponseTrait' => $vendorDir . '/symfony/http-client/Response/CommonResponseTrait.php',
    'Symfony\\Component\\HttpClient\\Response\\CurlResponse' => $vendorDir . '/symfony/http-client/Response/CurlResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\HttplugPromise' => $vendorDir . '/symfony/http-client/Response/HttplugPromise.php',
    'Symfony\\Component\\HttpClient\\Response\\JsonMockResponse' => $vendorDir . '/symfony/http-client/Response/JsonMockResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\MockResponse' => $vendorDir . '/symfony/http-client/Response/MockResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\NativeResponse' => $vendorDir . '/symfony/http-client/Response/NativeResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\ResponseStream' => $vendorDir . '/symfony/http-client/Response/ResponseStream.php',
    'Symfony\\Component\\HttpClient\\Response\\StreamWrapper' => $vendorDir . '/symfony/http-client/Response/StreamWrapper.php',
    'Symfony\\Component\\HttpClient\\Response\\StreamableInterface' => $vendorDir . '/symfony/http-client/Response/StreamableInterface.php',
    'Symfony\\Component\\HttpClient\\Response\\TraceableResponse' => $vendorDir . '/symfony/http-client/Response/TraceableResponse.php',
    'Symfony\\Component\\HttpClient\\Response\\TransportResponseTrait' => $vendorDir . '/symfony/http-client/Response/TransportResponseTrait.php',
    'Symfony\\Component\\HttpClient\\Retry\\GenericRetryStrategy' => $vendorDir . '/symfony/http-client/Retry/GenericRetryStrategy.php',
    'Symfony\\Component\\HttpClient\\Retry\\RetryStrategyInterface' => $vendorDir . '/symfony/http-client/Retry/RetryStrategyInterface.php',
    'Symfony\\Component\\HttpClient\\RetryableHttpClient' => $vendorDir . '/symfony/http-client/RetryableHttpClient.php',
    'Symfony\\Component\\HttpClient\\ScopingHttpClient' => $vendorDir . '/symfony/http-client/ScopingHttpClient.php',
    'Symfony\\Component\\HttpClient\\Test\\HarFileResponseFactory' => $vendorDir . '/symfony/http-client/Test/HarFileResponseFactory.php',
    'Symfony\\Component\\HttpClient\\ThrottlingHttpClient' => $vendorDir . '/symfony/http-client/ThrottlingHttpClient.php',
    'Symfony\\Component\\HttpClient\\TraceableHttpClient' => $vendorDir . '/symfony/http-client/TraceableHttpClient.php',
    'Symfony\\Component\\HttpClient\\UriTemplateHttpClient' => $vendorDir . '/symfony/http-client/UriTemplateHttpClient.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeader' => $vendorDir . '/symfony/http-foundation/AcceptHeader.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeaderItem' => $vendorDir . '/symfony/http-foundation/AcceptHeaderItem.php',
    'Symfony\\Component\\HttpFoundation\\BinaryFileResponse' => $vendorDir . '/symfony/http-foundation/BinaryFileResponse.php',
    'Symfony\\Component\\HttpFoundation\\ChainRequestMatcher' => $vendorDir . '/symfony/http-foundation/ChainRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\Cookie' => $vendorDir . '/symfony/http-foundation/Cookie.php',
    'Symfony\\Component\\HttpFoundation\\EventStreamResponse' => $vendorDir . '/symfony/http-foundation/EventStreamResponse.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\BadRequestException' => $vendorDir . '/symfony/http-foundation/Exception/BadRequestException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\ConflictingHeadersException' => $vendorDir . '/symfony/http-foundation/Exception/ConflictingHeadersException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/http-foundation/Exception/ExceptionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\ExpiredSignedUriException' => $vendorDir . '/symfony/http-foundation/Exception/ExpiredSignedUriException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\JsonException' => $vendorDir . '/symfony/http-foundation/Exception/JsonException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\LogicException' => $vendorDir . '/symfony/http-foundation/Exception/LogicException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\RequestExceptionInterface' => $vendorDir . '/symfony/http-foundation/Exception/RequestExceptionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SessionNotFoundException' => $vendorDir . '/symfony/http-foundation/Exception/SessionNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SignedUriException' => $vendorDir . '/symfony/http-foundation/Exception/SignedUriException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SuspiciousOperationException' => $vendorDir . '/symfony/http-foundation/Exception/SuspiciousOperationException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\UnexpectedValueException' => $vendorDir . '/symfony/http-foundation/Exception/UnexpectedValueException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\UnsignedUriException' => $vendorDir . '/symfony/http-foundation/Exception/UnsignedUriException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\UnverifiedSignedUriException' => $vendorDir . '/symfony/http-foundation/Exception/UnverifiedSignedUriException.php',
    'Symfony\\Component\\HttpFoundation\\FileBag' => $vendorDir . '/symfony/http-foundation/FileBag.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/http-foundation/File/Exception/AccessDeniedException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\CannotWriteFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/CannotWriteFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\ExtensionFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/ExtensionFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileNotFoundException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FormSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FormSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\IniSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/IniSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoTmpDirFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoTmpDirFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\PartialFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/PartialFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/http-foundation/File/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UploadException' => $vendorDir . '/symfony/http-foundation/File/Exception/UploadException.php',
    'Symfony\\Component\\HttpFoundation\\File\\File' => $vendorDir . '/symfony/http-foundation/File/File.php',
    'Symfony\\Component\\HttpFoundation\\File\\Stream' => $vendorDir . '/symfony/http-foundation/File/Stream.php',
    'Symfony\\Component\\HttpFoundation\\File\\UploadedFile' => $vendorDir . '/symfony/http-foundation/File/UploadedFile.php',
    'Symfony\\Component\\HttpFoundation\\HeaderBag' => $vendorDir . '/symfony/http-foundation/HeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\HeaderUtils' => $vendorDir . '/symfony/http-foundation/HeaderUtils.php',
    'Symfony\\Component\\HttpFoundation\\InputBag' => $vendorDir . '/symfony/http-foundation/InputBag.php',
    'Symfony\\Component\\HttpFoundation\\IpUtils' => $vendorDir . '/symfony/http-foundation/IpUtils.php',
    'Symfony\\Component\\HttpFoundation\\JsonResponse' => $vendorDir . '/symfony/http-foundation/JsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\ParameterBag' => $vendorDir . '/symfony/http-foundation/ParameterBag.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\AbstractRequestRateLimiter' => $vendorDir . '/symfony/http-foundation/RateLimiter/AbstractRequestRateLimiter.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\PeekableRequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/PeekableRequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\RequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/RequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RedirectResponse' => $vendorDir . '/symfony/http-foundation/RedirectResponse.php',
    'Symfony\\Component\\HttpFoundation\\Request' => $vendorDir . '/symfony/http-foundation/Request.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcherInterface' => $vendorDir . '/symfony/http-foundation/RequestMatcherInterface.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\AttributesRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/AttributesRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\ExpressionRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/ExpressionRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\HeaderRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/HeaderRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\HostRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/HostRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IpsRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IpsRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IsJsonRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IsJsonRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\MethodRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/MethodRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PathRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PathRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PortRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PortRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\QueryParameterRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/QueryParameterRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\SchemeRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/SchemeRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestStack' => $vendorDir . '/symfony/http-foundation/RequestStack.php',
    'Symfony\\Component\\HttpFoundation\\Response' => $vendorDir . '/symfony/http-foundation/Response.php',
    'Symfony\\Component\\HttpFoundation\\ResponseHeaderBag' => $vendorDir . '/symfony/http-foundation/ResponseHeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\ServerBag' => $vendorDir . '/symfony/http-foundation/ServerBag.php',
    'Symfony\\Component\\HttpFoundation\\ServerEvent' => $vendorDir . '/symfony/http-foundation/ServerEvent.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBag' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\FlashBagAwareSessionInterface' => $vendorDir . '/symfony/http-foundation/Session/FlashBagAwareSessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\AutoExpireFlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/AutoExpireFlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Session' => $vendorDir . '/symfony/http-foundation/Session/Session.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagProxy' => $vendorDir . '/symfony/http-foundation/Session/SessionBagProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactory' => $vendorDir . '/symfony/http-foundation/Session/SessionFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionUtils' => $vendorDir . '/symfony/http-foundation/Session/SessionUtils.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\AbstractSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/AbstractSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\IdentityMarshaller' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/IdentityMarshaller.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MarshallingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MarshallingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MemcachedSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MemcachedSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MigratingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MigratingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MongoDbSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MongoDbSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NativeFileSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NativeFileSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NullSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NullSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\PdoSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/PdoSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\RedisSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/RedisSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\SessionHandlerFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/SessionHandlerFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\StrictSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/StrictSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MetadataBag' => $vendorDir . '/symfony/http-foundation/Session/Storage/MetadataBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockArraySessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockArraySessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\AbstractProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/AbstractProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\SessionHandlerProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/SessionHandlerProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageInterface.php',
    'Symfony\\Component\\HttpFoundation\\StreamedJsonResponse' => $vendorDir . '/symfony/http-foundation/StreamedJsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\StreamedResponse' => $vendorDir . '/symfony/http-foundation/StreamedResponse.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\RequestAttributeValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/RequestAttributeValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseCookieValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseCookieValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseFormatSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseFormatSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasCookie' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasCookie.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasHeader' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasHeader.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderLocationSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderLocationSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsRedirected' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsRedirected.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsSuccessful' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsSuccessful.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsUnprocessable' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsUnprocessable.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseStatusCodeSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseStatusCodeSame.php',
    'Symfony\\Component\\HttpFoundation\\UriSigner' => $vendorDir . '/symfony/http-foundation/UriSigner.php',
    'Symfony\\Component\\HttpFoundation\\UrlHelper' => $vendorDir . '/symfony/http-foundation/UrlHelper.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsController' => $vendorDir . '/symfony/http-kernel/Attribute/AsController.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsTargetedValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/AsTargetedValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\Cache' => $vendorDir . '/symfony/http-kernel/Attribute/Cache.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\IsSignatureValid' => $vendorDir . '/symfony/http-kernel/Attribute/IsSignatureValid.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapDateTime' => $vendorDir . '/symfony/http-kernel/Attribute/MapDateTime.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryParameter' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryParameter.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryString' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryString.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapRequestPayload' => $vendorDir . '/symfony/http-kernel/Attribute/MapRequestPayload.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapUploadedFile' => $vendorDir . '/symfony/http-kernel/Attribute/MapUploadedFile.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\ValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/ValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithHttpStatus' => $vendorDir . '/symfony/http-kernel/Attribute/WithHttpStatus.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithLogLevel' => $vendorDir . '/symfony/http-kernel/Attribute/WithLogLevel.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\AbstractBundle' => $vendorDir . '/symfony/http-kernel/Bundle/AbstractBundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\Bundle' => $vendorDir . '/symfony/http-kernel/Bundle/Bundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleExtension' => $vendorDir . '/symfony/http-kernel/Bundle/BundleExtension.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleInterface' => $vendorDir . '/symfony/http-kernel/Bundle/BundleInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\CacheClearerInterface' => $vendorDir . '/symfony/http-kernel/CacheClearer/CacheClearerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\ChainCacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/ChainCacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\Psr6CacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/Psr6CacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmer' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerAggregate' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerAggregate.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\WarmableInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/WarmableInterface.php',
    'Symfony\\Component\\HttpKernel\\Config\\FileLocator' => $vendorDir . '/symfony/http-kernel/Config/FileLocator.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadata' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadata.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactory' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactory.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactoryInterface' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactoryInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\BackedEnumValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/BackedEnumValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DateTimeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DateTimeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DefaultValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DefaultValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\NotTaggedControllerValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/NotTaggedControllerValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\QueryParameterValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/QueryParameterValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestAttributeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestAttributeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestPayloadValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestPayloadValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\ServiceValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/ServiceValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\SessionValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/SessionValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\TraceableValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/TraceableValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\UidValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/UidValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\VariadicValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/VariadicValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ContainerControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ContainerControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerReference' => $vendorDir . '/symfony/http-kernel/Controller/ControllerReference.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ErrorController' => $vendorDir . '/symfony/http-kernel/Controller/ErrorController.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ValueResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ValueResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\AjaxDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/AjaxDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ConfigDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ConfigDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DumpDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DumpDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\EventDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/EventDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ExceptionDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ExceptionDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LateDataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/LateDataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LoggerDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/LoggerDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\MemoryDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/MemoryDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RequestDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RequestDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RouterDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RouterDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\TimeDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/TimeDataCollector.php',
    'Symfony\\Component\\HttpKernel\\Debug\\ErrorHandlerConfigurator' => $vendorDir . '/symfony/http-kernel/Debug/ErrorHandlerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/http-kernel/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\HttpKernel\\Debug\\VirtualRequestStack' => $vendorDir . '/symfony/http-kernel/Debug/VirtualRequestStack.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\AddAnnotatedClassesToCachePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/AddAnnotatedClassesToCachePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ConfigurableExtension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ConfigurableExtension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ControllerArgumentValueResolverPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ControllerArgumentValueResolverPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\Extension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/Extension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\FragmentRendererPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/FragmentRendererPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LazyLoadingFragmentHandler' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LazyLoadingFragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LoggerPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LoggerPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\MergeExtensionConfigurationPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/MergeExtensionConfigurationPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterLocaleAwareServicesPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterLocaleAwareServicesPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RemoveEmptyControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RemoveEmptyControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ResettableServicePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ResettableServicePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ServicesResetter' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ServicesResetter.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ServicesResetterInterface' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ServicesResetterInterface.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AbstractSessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/AbstractSessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AddRequestFormatsListener' => $vendorDir . '/symfony/http-kernel/EventListener/AddRequestFormatsListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\CacheAttributeListener' => $vendorDir . '/symfony/http-kernel/EventListener/CacheAttributeListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DebugHandlersListener' => $vendorDir . '/symfony/http-kernel/EventListener/DebugHandlersListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DisallowRobotsIndexingListener' => $vendorDir . '/symfony/http-kernel/EventListener/DisallowRobotsIndexingListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DumpListener' => $vendorDir . '/symfony/http-kernel/EventListener/DumpListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ErrorListener' => $vendorDir . '/symfony/http-kernel/EventListener/ErrorListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\FragmentListener' => $vendorDir . '/symfony/http-kernel/EventListener/FragmentListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\IsSignatureValidAttributeListener' => $vendorDir . '/symfony/http-kernel/EventListener/IsSignatureValidAttributeListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleAwareListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleAwareListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ProfilerListener' => $vendorDir . '/symfony/http-kernel/EventListener/ProfilerListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ResponseListener' => $vendorDir . '/symfony/http-kernel/EventListener/ResponseListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\RouterListener' => $vendorDir . '/symfony/http-kernel/EventListener/RouterListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/SessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SurrogateListener' => $vendorDir . '/symfony/http-kernel/EventListener/SurrogateListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ValidateRequestListener' => $vendorDir . '/symfony/http-kernel/EventListener/ValidateRequestListener.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerArgumentsEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerArgumentsEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ExceptionEvent' => $vendorDir . '/symfony/http-kernel/Event/ExceptionEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\FinishRequestEvent' => $vendorDir . '/symfony/http-kernel/Event/FinishRequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\KernelEvent' => $vendorDir . '/symfony/http-kernel/Event/KernelEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\RequestEvent' => $vendorDir . '/symfony/http-kernel/Event/RequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ResponseEvent' => $vendorDir . '/symfony/http-kernel/Event/ResponseEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\TerminateEvent' => $vendorDir . '/symfony/http-kernel/Event/TerminateEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ViewEvent' => $vendorDir . '/symfony/http-kernel/Event/ViewEvent.php',
    'Symfony\\Component\\HttpKernel\\Exception\\AccessDeniedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/AccessDeniedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\BadRequestHttpException' => $vendorDir . '/symfony/http-kernel/Exception/BadRequestHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ConflictHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ConflictHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ControllerDoesNotReturnResponseException' => $vendorDir . '/symfony/http-kernel/Exception/ControllerDoesNotReturnResponseException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\GoneHttpException' => $vendorDir . '/symfony/http-kernel/Exception/GoneHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpException' => $vendorDir . '/symfony/http-kernel/Exception/HttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpExceptionInterface' => $vendorDir . '/symfony/http-kernel/Exception/HttpExceptionInterface.php',
    'Symfony\\Component\\HttpKernel\\Exception\\InvalidMetadataException' => $vendorDir . '/symfony/http-kernel/Exception/InvalidMetadataException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LengthRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LengthRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LockedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LockedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\MethodNotAllowedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/MethodNotAllowedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NearMissValueResolverException' => $vendorDir . '/symfony/http-kernel/Exception/NearMissValueResolverException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotAcceptableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotAcceptableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotFoundHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotFoundHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionFailedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionFailedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ResolverNotFoundException' => $vendorDir . '/symfony/http-kernel/Exception/ResolverNotFoundException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ServiceUnavailableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ServiceUnavailableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\TooManyRequestsHttpException' => $vendorDir . '/symfony/http-kernel/Exception/TooManyRequestsHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnauthorizedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnauthorizedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnexpectedSessionUsageException' => $vendorDir . '/symfony/http-kernel/Exception/UnexpectedSessionUsageException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnprocessableEntityHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnprocessableEntityHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnsupportedMediaTypeHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnsupportedMediaTypeHttpException.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\AbstractSurrogateFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/AbstractSurrogateFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\EsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/EsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentHandler' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentRendererInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentRendererInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGenerator' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGenerator.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGeneratorInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGeneratorInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\HIncludeFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/HIncludeFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\InlineFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/InlineFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\RoutableFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/RoutableFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\SsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/SsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\AbstractSurrogate' => $vendorDir . '/symfony/http-kernel/HttpCache/AbstractSurrogate.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\CacheWasLockedException' => $vendorDir . '/symfony/http-kernel/HttpCache/CacheWasLockedException.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Esi' => $vendorDir . '/symfony/http-kernel/HttpCache/Esi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\HttpCache' => $vendorDir . '/symfony/http-kernel/HttpCache/HttpCache.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategy' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategy.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategyInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategyInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Ssi' => $vendorDir . '/symfony/http-kernel/HttpCache/Ssi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Store' => $vendorDir . '/symfony/http-kernel/HttpCache/Store.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\StoreInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/StoreInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SubRequestHandler' => $vendorDir . '/symfony/http-kernel/HttpCache/SubRequestHandler.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SurrogateInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/SurrogateInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpClientKernel' => $vendorDir . '/symfony/http-kernel/HttpClientKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernel' => $vendorDir . '/symfony/http-kernel/HttpKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelBrowser' => $vendorDir . '/symfony/http-kernel/HttpKernelBrowser.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelInterface' => $vendorDir . '/symfony/http-kernel/HttpKernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Kernel' => $vendorDir . '/symfony/http-kernel/Kernel.php',
    'Symfony\\Component\\HttpKernel\\KernelEvents' => $vendorDir . '/symfony/http-kernel/KernelEvents.php',
    'Symfony\\Component\\HttpKernel\\KernelInterface' => $vendorDir . '/symfony/http-kernel/KernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerConfigurator' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerInterface' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\Logger' => $vendorDir . '/symfony/http-kernel/Log/Logger.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\FileProfilerStorage' => $vendorDir . '/symfony/http-kernel/Profiler/FileProfilerStorage.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profile' => $vendorDir . '/symfony/http-kernel/Profiler/Profile.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profiler' => $vendorDir . '/symfony/http-kernel/Profiler/Profiler.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\ProfilerStateChecker' => $vendorDir . '/symfony/http-kernel/Profiler/ProfilerStateChecker.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\ProfilerStorageInterface' => $vendorDir . '/symfony/http-kernel/Profiler/ProfilerStorageInterface.php',
    'Symfony\\Component\\HttpKernel\\RebootableInterface' => $vendorDir . '/symfony/http-kernel/RebootableInterface.php',
    'Symfony\\Component\\HttpKernel\\TerminableInterface' => $vendorDir . '/symfony/http-kernel/TerminableInterface.php',
    'Symfony\\Component\\Intl\\Countries' => $vendorDir . '/symfony/intl/Countries.php',
    'Symfony\\Component\\Intl\\Currencies' => $vendorDir . '/symfony/intl/Currencies.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Compiler\\BundleCompilerInterface' => $vendorDir . '/symfony/intl/Data/Bundle/Compiler/BundleCompilerInterface.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Compiler\\GenrbCompiler' => $vendorDir . '/symfony/intl/Data/Bundle/Compiler/GenrbCompiler.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\BufferedBundleReader' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/BufferedBundleReader.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\BundleEntryReader' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/BundleEntryReader.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\BundleEntryReaderInterface' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/BundleEntryReaderInterface.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\BundleReaderInterface' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/BundleReaderInterface.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\IntlBundleReader' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/IntlBundleReader.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\JsonBundleReader' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/JsonBundleReader.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Reader\\PhpBundleReader' => $vendorDir . '/symfony/intl/Data/Bundle/Reader/PhpBundleReader.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Writer\\BundleWriterInterface' => $vendorDir . '/symfony/intl/Data/Bundle/Writer/BundleWriterInterface.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Writer\\JsonBundleWriter' => $vendorDir . '/symfony/intl/Data/Bundle/Writer/JsonBundleWriter.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Writer\\PhpBundleWriter' => $vendorDir . '/symfony/intl/Data/Bundle/Writer/PhpBundleWriter.php',
    'Symfony\\Component\\Intl\\Data\\Bundle\\Writer\\TextBundleWriter' => $vendorDir . '/symfony/intl/Data/Bundle/Writer/TextBundleWriter.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\AbstractDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/AbstractDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\CurrencyDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/CurrencyDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\FallbackTrait' => $vendorDir . '/symfony/intl/Data/Generator/FallbackTrait.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\GeneratorConfig' => $vendorDir . '/symfony/intl/Data/Generator/GeneratorConfig.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\LanguageDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/LanguageDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\LocaleDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/LocaleDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\RegionDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/RegionDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\ScriptDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/ScriptDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Generator\\TimezoneDataGenerator' => $vendorDir . '/symfony/intl/Data/Generator/TimezoneDataGenerator.php',
    'Symfony\\Component\\Intl\\Data\\Util\\ArrayAccessibleResourceBundle' => $vendorDir . '/symfony/intl/Data/Util/ArrayAccessibleResourceBundle.php',
    'Symfony\\Component\\Intl\\Data\\Util\\LocaleScanner' => $vendorDir . '/symfony/intl/Data/Util/LocaleScanner.php',
    'Symfony\\Component\\Intl\\Data\\Util\\RecursiveArrayAccess' => $vendorDir . '/symfony/intl/Data/Util/RecursiveArrayAccess.php',
    'Symfony\\Component\\Intl\\Data\\Util\\RingBuffer' => $vendorDir . '/symfony/intl/Data/Util/RingBuffer.php',
    'Symfony\\Component\\Intl\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/intl/Exception/BadMethodCallException.php',
    'Symfony\\Component\\Intl\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/intl/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Intl\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/intl/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Intl\\Exception\\MissingResourceException' => $vendorDir . '/symfony/intl/Exception/MissingResourceException.php',
    'Symfony\\Component\\Intl\\Exception\\OutOfBoundsException' => $vendorDir . '/symfony/intl/Exception/OutOfBoundsException.php',
    'Symfony\\Component\\Intl\\Exception\\ResourceBundleNotFoundException' => $vendorDir . '/symfony/intl/Exception/ResourceBundleNotFoundException.php',
    'Symfony\\Component\\Intl\\Exception\\RuntimeException' => $vendorDir . '/symfony/intl/Exception/RuntimeException.php',
    'Symfony\\Component\\Intl\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/intl/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\Intl\\Intl' => $vendorDir . '/symfony/intl/Intl.php',
    'Symfony\\Component\\Intl\\Languages' => $vendorDir . '/symfony/intl/Languages.php',
    'Symfony\\Component\\Intl\\Locale' => $vendorDir . '/symfony/intl/Locale.php',
    'Symfony\\Component\\Intl\\Locales' => $vendorDir . '/symfony/intl/Locales.php',
    'Symfony\\Component\\Intl\\ResourceBundle' => $vendorDir . '/symfony/intl/ResourceBundle.php',
    'Symfony\\Component\\Intl\\Scripts' => $vendorDir . '/symfony/intl/Scripts.php',
    'Symfony\\Component\\Intl\\Timezones' => $vendorDir . '/symfony/intl/Timezones.php',
    'Symfony\\Component\\Intl\\Transliterator\\EmojiTransliterator' => $vendorDir . '/symfony/intl/Transliterator/EmojiTransliterator.php',
    'Symfony\\Component\\Intl\\Util\\GitRepository' => $vendorDir . '/symfony/intl/Util/GitRepository.php',
    'Symfony\\Component\\Intl\\Util\\GzipStreamWrapper' => $vendorDir . '/symfony/intl/Util/GzipStreamWrapper.php',
    'Symfony\\Component\\Intl\\Util\\IcuVersion' => $vendorDir . '/symfony/intl/Util/IcuVersion.php',
    'Symfony\\Component\\Intl\\Util\\IntlTestHelper' => $vendorDir . '/symfony/intl/Util/IntlTestHelper.php',
    'Symfony\\Component\\Intl\\Util\\Version' => $vendorDir . '/symfony/intl/Util/Version.php',
    'Symfony\\Component\\PasswordHasher\\Command\\UserPasswordHashCommand' => $vendorDir . '/symfony/password-hasher/Command/UserPasswordHashCommand.php',
    'Symfony\\Component\\PasswordHasher\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/password-hasher/Exception/ExceptionInterface.php',
    'Symfony\\Component\\PasswordHasher\\Exception\\InvalidPasswordException' => $vendorDir . '/symfony/password-hasher/Exception/InvalidPasswordException.php',
    'Symfony\\Component\\PasswordHasher\\Exception\\LogicException' => $vendorDir . '/symfony/password-hasher/Exception/LogicException.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\CheckPasswordLengthTrait' => $vendorDir . '/symfony/password-hasher/Hasher/CheckPasswordLengthTrait.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\MessageDigestPasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/MessageDigestPasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\MigratingPasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/MigratingPasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\NativePasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/NativePasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\PasswordHasherAwareInterface' => $vendorDir . '/symfony/password-hasher/Hasher/PasswordHasherAwareInterface.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\PasswordHasherFactory' => $vendorDir . '/symfony/password-hasher/Hasher/PasswordHasherFactory.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\PasswordHasherFactoryInterface' => $vendorDir . '/symfony/password-hasher/Hasher/PasswordHasherFactoryInterface.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\Pbkdf2PasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/Pbkdf2PasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\PlaintextPasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/PlaintextPasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\SodiumPasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/SodiumPasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\UserPasswordHasher' => $vendorDir . '/symfony/password-hasher/Hasher/UserPasswordHasher.php',
    'Symfony\\Component\\PasswordHasher\\Hasher\\UserPasswordHasherInterface' => $vendorDir . '/symfony/password-hasher/Hasher/UserPasswordHasherInterface.php',
    'Symfony\\Component\\PasswordHasher\\LegacyPasswordHasherInterface' => $vendorDir . '/symfony/password-hasher/LegacyPasswordHasherInterface.php',
    'Symfony\\Component\\PasswordHasher\\PasswordHasherInterface' => $vendorDir . '/symfony/password-hasher/PasswordHasherInterface.php',
    'Symfony\\Component\\Routing\\Alias' => $vendorDir . '/symfony/routing/Alias.php',
    'Symfony\\Component\\Routing\\Annotation\\Route' => $vendorDir . '/symfony/routing/Annotation/Route.php',
    'Symfony\\Component\\Routing\\Attribute\\DeprecatedAlias' => $vendorDir . '/symfony/routing/Attribute/DeprecatedAlias.php',
    'Symfony\\Component\\Routing\\Attribute\\Route' => $vendorDir . '/symfony/routing/Attribute/Route.php',
    'Symfony\\Component\\Routing\\CompiledRoute' => $vendorDir . '/symfony/routing/CompiledRoute.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\AddExpressionLanguageProvidersPass' => $vendorDir . '/symfony/routing/DependencyInjection/AddExpressionLanguageProvidersPass.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\RoutingControllerPass' => $vendorDir . '/symfony/routing/DependencyInjection/RoutingControllerPass.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\RoutingResolverPass' => $vendorDir . '/symfony/routing/DependencyInjection/RoutingResolverPass.php',
    'Symfony\\Component\\Routing\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/routing/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/routing/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidParameterException' => $vendorDir . '/symfony/routing/Exception/InvalidParameterException.php',
    'Symfony\\Component\\Routing\\Exception\\LogicException' => $vendorDir . '/symfony/routing/Exception/LogicException.php',
    'Symfony\\Component\\Routing\\Exception\\MethodNotAllowedException' => $vendorDir . '/symfony/routing/Exception/MethodNotAllowedException.php',
    'Symfony\\Component\\Routing\\Exception\\MissingMandatoryParametersException' => $vendorDir . '/symfony/routing/Exception/MissingMandatoryParametersException.php',
    'Symfony\\Component\\Routing\\Exception\\NoConfigurationException' => $vendorDir . '/symfony/routing/Exception/NoConfigurationException.php',
    'Symfony\\Component\\Routing\\Exception\\ResourceNotFoundException' => $vendorDir . '/symfony/routing/Exception/ResourceNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteCircularReferenceException' => $vendorDir . '/symfony/routing/Exception/RouteCircularReferenceException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteNotFoundException' => $vendorDir . '/symfony/routing/Exception/RouteNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RuntimeException' => $vendorDir . '/symfony/routing/Exception/RuntimeException.php',
    'Symfony\\Component\\Routing\\Generator\\CompiledUrlGenerator' => $vendorDir . '/symfony/routing/Generator/CompiledUrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\ConfigurableRequirementsInterface' => $vendorDir . '/symfony/routing/Generator/ConfigurableRequirementsInterface.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\CompiledUrlGeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/CompiledUrlGeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumperInterface' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumperInterface.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGenerator' => $vendorDir . '/symfony/routing/Generator/UrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGeneratorInterface' => $vendorDir . '/symfony/routing/Generator/UrlGeneratorInterface.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeClassLoader' => $vendorDir . '/symfony/routing/Loader/AttributeClassLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeDirectoryLoader' => $vendorDir . '/symfony/routing/Loader/AttributeDirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeFileLoader' => $vendorDir . '/symfony/routing/Loader/AttributeFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeServicesLoader' => $vendorDir . '/symfony/routing/Loader/AttributeServicesLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ClosureLoader' => $vendorDir . '/symfony/routing/Loader/ClosureLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\AliasConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/AliasConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\CollectionConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/CollectionConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\ImportConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/ImportConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RouteConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RouteConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RoutesReference' => $vendorDir . '/symfony/routing/Loader/Configurator/RoutesReference.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RoutingConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RoutingConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\AddTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/AddTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\HostTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/HostTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\LocalizedRouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/LocalizedRouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\PrefixTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/PrefixTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\RouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/RouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\ContainerLoader' => $vendorDir . '/symfony/routing/Loader/ContainerLoader.php',
    'Symfony\\Component\\Routing\\Loader\\DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/routing/Loader/GlobFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ObjectLoader' => $vendorDir . '/symfony/routing/Loader/ObjectLoader.php',
    'Symfony\\Component\\Routing\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/routing/Loader/PhpFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Psr4DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/Psr4DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/routing/Loader/XmlFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/routing/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Routing\\Matcher\\CompiledUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/CompiledUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherTrait' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherTrait.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumperInterface' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumperInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\StaticPrefixCollection' => $vendorDir . '/symfony/routing/Matcher/Dumper/StaticPrefixCollection.php',
    'Symfony\\Component\\Routing\\Matcher\\ExpressionLanguageProvider' => $vendorDir . '/symfony/routing/Matcher/ExpressionLanguageProvider.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\RequestMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RequestMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\TraceableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/TraceableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcher' => $vendorDir . '/symfony/routing/Matcher/UrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/UrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\RequestContext' => $vendorDir . '/symfony/routing/RequestContext.php',
    'Symfony\\Component\\Routing\\RequestContextAwareInterface' => $vendorDir . '/symfony/routing/RequestContextAwareInterface.php',
    'Symfony\\Component\\Routing\\Requirement\\EnumRequirement' => $vendorDir . '/symfony/routing/Requirement/EnumRequirement.php',
    'Symfony\\Component\\Routing\\Requirement\\Requirement' => $vendorDir . '/symfony/routing/Requirement/Requirement.php',
    'Symfony\\Component\\Routing\\Route' => $vendorDir . '/symfony/routing/Route.php',
    'Symfony\\Component\\Routing\\RouteCollection' => $vendorDir . '/symfony/routing/RouteCollection.php',
    'Symfony\\Component\\Routing\\RouteCompiler' => $vendorDir . '/symfony/routing/RouteCompiler.php',
    'Symfony\\Component\\Routing\\RouteCompilerInterface' => $vendorDir . '/symfony/routing/RouteCompilerInterface.php',
    'Symfony\\Component\\Routing\\Router' => $vendorDir . '/symfony/routing/Router.php',
    'Symfony\\Component\\Routing\\RouterInterface' => $vendorDir . '/symfony/routing/RouterInterface.php',
    'Symfony\\Component\\String\\AbstractString' => $vendorDir . '/symfony/string/AbstractString.php',
    'Symfony\\Component\\String\\AbstractUnicodeString' => $vendorDir . '/symfony/string/AbstractUnicodeString.php',
    'Symfony\\Component\\String\\ByteString' => $vendorDir . '/symfony/string/ByteString.php',
    'Symfony\\Component\\String\\CodePointString' => $vendorDir . '/symfony/string/CodePointString.php',
    'Symfony\\Component\\String\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/string/Exception/ExceptionInterface.php',
    'Symfony\\Component\\String\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/string/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\String\\Exception\\RuntimeException' => $vendorDir . '/symfony/string/Exception/RuntimeException.php',
    'Symfony\\Component\\String\\Inflector\\EnglishInflector' => $vendorDir . '/symfony/string/Inflector/EnglishInflector.php',
    'Symfony\\Component\\String\\Inflector\\FrenchInflector' => $vendorDir . '/symfony/string/Inflector/FrenchInflector.php',
    'Symfony\\Component\\String\\Inflector\\InflectorInterface' => $vendorDir . '/symfony/string/Inflector/InflectorInterface.php',
    'Symfony\\Component\\String\\Inflector\\SpanishInflector' => $vendorDir . '/symfony/string/Inflector/SpanishInflector.php',
    'Symfony\\Component\\String\\LazyString' => $vendorDir . '/symfony/string/LazyString.php',
    'Symfony\\Component\\String\\Slugger\\AsciiSlugger' => $vendorDir . '/symfony/string/Slugger/AsciiSlugger.php',
    'Symfony\\Component\\String\\Slugger\\SluggerInterface' => $vendorDir . '/symfony/string/Slugger/SluggerInterface.php',
    'Symfony\\Component\\String\\TruncateMode' => $vendorDir . '/symfony/string/TruncateMode.php',
    'Symfony\\Component\\String\\UnicodeString' => $vendorDir . '/symfony/string/UnicodeString.php',
    'Symfony\\Component\\Translation\\CatalogueMetadataAwareInterface' => $vendorDir . '/symfony/translation/CatalogueMetadataAwareInterface.php',
    'Symfony\\Component\\Translation\\Catalogue\\AbstractOperation' => $vendorDir . '/symfony/translation/Catalogue/AbstractOperation.php',
    'Symfony\\Component\\Translation\\Catalogue\\MergeOperation' => $vendorDir . '/symfony/translation/Catalogue/MergeOperation.php',
    'Symfony\\Component\\Translation\\Catalogue\\OperationInterface' => $vendorDir . '/symfony/translation/Catalogue/OperationInterface.php',
    'Symfony\\Component\\Translation\\Catalogue\\TargetOperation' => $vendorDir . '/symfony/translation/Catalogue/TargetOperation.php',
    'Symfony\\Component\\Translation\\Command\\TranslationLintCommand' => $vendorDir . '/symfony/translation/Command/TranslationLintCommand.php',
    'Symfony\\Component\\Translation\\Command\\TranslationPullCommand' => $vendorDir . '/symfony/translation/Command/TranslationPullCommand.php',
    'Symfony\\Component\\Translation\\Command\\TranslationPushCommand' => $vendorDir . '/symfony/translation/Command/TranslationPushCommand.php',
    'Symfony\\Component\\Translation\\Command\\TranslationTrait' => $vendorDir . '/symfony/translation/Command/TranslationTrait.php',
    'Symfony\\Component\\Translation\\Command\\XliffLintCommand' => $vendorDir . '/symfony/translation/Command/XliffLintCommand.php',
    'Symfony\\Component\\Translation\\DataCollectorTranslator' => $vendorDir . '/symfony/translation/DataCollectorTranslator.php',
    'Symfony\\Component\\Translation\\DataCollector\\TranslationDataCollector' => $vendorDir . '/symfony/translation/DataCollector/TranslationDataCollector.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\DataCollectorTranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/DataCollectorTranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\LoggingTranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/LoggingTranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslationDumperPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslationDumperPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslationExtractorPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslationExtractorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslatorPathsPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslatorPathsPass.php',
    'Symfony\\Component\\Translation\\Dumper\\CsvFileDumper' => $vendorDir . '/symfony/translation/Dumper/CsvFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\DumperInterface' => $vendorDir . '/symfony/translation/Dumper/DumperInterface.php',
    'Symfony\\Component\\Translation\\Dumper\\FileDumper' => $vendorDir . '/symfony/translation/Dumper/FileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\IcuResFileDumper' => $vendorDir . '/symfony/translation/Dumper/IcuResFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\IniFileDumper' => $vendorDir . '/symfony/translation/Dumper/IniFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\JsonFileDumper' => $vendorDir . '/symfony/translation/Dumper/JsonFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\MoFileDumper' => $vendorDir . '/symfony/translation/Dumper/MoFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\PhpFileDumper' => $vendorDir . '/symfony/translation/Dumper/PhpFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\PoFileDumper' => $vendorDir . '/symfony/translation/Dumper/PoFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\QtFileDumper' => $vendorDir . '/symfony/translation/Dumper/QtFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\XliffFileDumper' => $vendorDir . '/symfony/translation/Dumper/XliffFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\YamlFileDumper' => $vendorDir . '/symfony/translation/Dumper/YamlFileDumper.php',
    'Symfony\\Component\\Translation\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/translation/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Translation\\Exception\\IncompleteDsnException' => $vendorDir . '/symfony/translation/Exception/IncompleteDsnException.php',
    'Symfony\\Component\\Translation\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/translation/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Translation\\Exception\\InvalidResourceException' => $vendorDir . '/symfony/translation/Exception/InvalidResourceException.php',
    'Symfony\\Component\\Translation\\Exception\\LogicException' => $vendorDir . '/symfony/translation/Exception/LogicException.php',
    'Symfony\\Component\\Translation\\Exception\\MissingRequiredOptionException' => $vendorDir . '/symfony/translation/Exception/MissingRequiredOptionException.php',
    'Symfony\\Component\\Translation\\Exception\\NotFoundResourceException' => $vendorDir . '/symfony/translation/Exception/NotFoundResourceException.php',
    'Symfony\\Component\\Translation\\Exception\\ProviderException' => $vendorDir . '/symfony/translation/Exception/ProviderException.php',
    'Symfony\\Component\\Translation\\Exception\\ProviderExceptionInterface' => $vendorDir . '/symfony/translation/Exception/ProviderExceptionInterface.php',
    'Symfony\\Component\\Translation\\Exception\\RuntimeException' => $vendorDir . '/symfony/translation/Exception/RuntimeException.php',
    'Symfony\\Component\\Translation\\Exception\\UnsupportedSchemeException' => $vendorDir . '/symfony/translation/Exception/UnsupportedSchemeException.php',
    'Symfony\\Component\\Translation\\Extractor\\AbstractFileExtractor' => $vendorDir . '/symfony/translation/Extractor/AbstractFileExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\ChainExtractor' => $vendorDir . '/symfony/translation/Extractor/ChainExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\ExtractorInterface' => $vendorDir . '/symfony/translation/Extractor/ExtractorInterface.php',
    'Symfony\\Component\\Translation\\Extractor\\PhpAstExtractor' => $vendorDir . '/symfony/translation/Extractor/PhpAstExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\AbstractVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/AbstractVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\ConstraintVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/ConstraintVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\TransMethodVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/TransMethodVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\TranslatableMessageVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/TranslatableMessageVisitor.php',
    'Symfony\\Component\\Translation\\Formatter\\IntlFormatter' => $vendorDir . '/symfony/translation/Formatter/IntlFormatter.php',
    'Symfony\\Component\\Translation\\Formatter\\IntlFormatterInterface' => $vendorDir . '/symfony/translation/Formatter/IntlFormatterInterface.php',
    'Symfony\\Component\\Translation\\Formatter\\MessageFormatter' => $vendorDir . '/symfony/translation/Formatter/MessageFormatter.php',
    'Symfony\\Component\\Translation\\Formatter\\MessageFormatterInterface' => $vendorDir . '/symfony/translation/Formatter/MessageFormatterInterface.php',
    'Symfony\\Component\\Translation\\IdentityTranslator' => $vendorDir . '/symfony/translation/IdentityTranslator.php',
    'Symfony\\Component\\Translation\\Loader\\ArrayLoader' => $vendorDir . '/symfony/translation/Loader/ArrayLoader.php',
    'Symfony\\Component\\Translation\\Loader\\CsvFileLoader' => $vendorDir . '/symfony/translation/Loader/CsvFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\FileLoader' => $vendorDir . '/symfony/translation/Loader/FileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IcuDatFileLoader' => $vendorDir . '/symfony/translation/Loader/IcuDatFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IcuResFileLoader' => $vendorDir . '/symfony/translation/Loader/IcuResFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IniFileLoader' => $vendorDir . '/symfony/translation/Loader/IniFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\JsonFileLoader' => $vendorDir . '/symfony/translation/Loader/JsonFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\LoaderInterface' => $vendorDir . '/symfony/translation/Loader/LoaderInterface.php',
    'Symfony\\Component\\Translation\\Loader\\MoFileLoader' => $vendorDir . '/symfony/translation/Loader/MoFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/translation/Loader/PhpFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\PoFileLoader' => $vendorDir . '/symfony/translation/Loader/PoFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\QtFileLoader' => $vendorDir . '/symfony/translation/Loader/QtFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\XliffFileLoader' => $vendorDir . '/symfony/translation/Loader/XliffFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/translation/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Translation\\LocaleSwitcher' => $vendorDir . '/symfony/translation/LocaleSwitcher.php',
    'Symfony\\Component\\Translation\\LoggingTranslator' => $vendorDir . '/symfony/translation/LoggingTranslator.php',
    'Symfony\\Component\\Translation\\MessageCatalogue' => $vendorDir . '/symfony/translation/MessageCatalogue.php',
    'Symfony\\Component\\Translation\\MessageCatalogueInterface' => $vendorDir . '/symfony/translation/MessageCatalogueInterface.php',
    'Symfony\\Component\\Translation\\MetadataAwareInterface' => $vendorDir . '/symfony/translation/MetadataAwareInterface.php',
    'Symfony\\Component\\Translation\\Provider\\AbstractProviderFactory' => $vendorDir . '/symfony/translation/Provider/AbstractProviderFactory.php',
    'Symfony\\Component\\Translation\\Provider\\Dsn' => $vendorDir . '/symfony/translation/Provider/Dsn.php',
    'Symfony\\Component\\Translation\\Provider\\FilteringProvider' => $vendorDir . '/symfony/translation/Provider/FilteringProvider.php',
    'Symfony\\Component\\Translation\\Provider\\NullProvider' => $vendorDir . '/symfony/translation/Provider/NullProvider.php',
    'Symfony\\Component\\Translation\\Provider\\NullProviderFactory' => $vendorDir . '/symfony/translation/Provider/NullProviderFactory.php',
    'Symfony\\Component\\Translation\\Provider\\ProviderFactoryInterface' => $vendorDir . '/symfony/translation/Provider/ProviderFactoryInterface.php',
    'Symfony\\Component\\Translation\\Provider\\ProviderInterface' => $vendorDir . '/symfony/translation/Provider/ProviderInterface.php',
    'Symfony\\Component\\Translation\\Provider\\TranslationProviderCollection' => $vendorDir . '/symfony/translation/Provider/TranslationProviderCollection.php',
    'Symfony\\Component\\Translation\\Provider\\TranslationProviderCollectionFactory' => $vendorDir . '/symfony/translation/Provider/TranslationProviderCollectionFactory.php',
    'Symfony\\Component\\Translation\\PseudoLocalizationTranslator' => $vendorDir . '/symfony/translation/PseudoLocalizationTranslator.php',
    'Symfony\\Component\\Translation\\Reader\\TranslationReader' => $vendorDir . '/symfony/translation/Reader/TranslationReader.php',
    'Symfony\\Component\\Translation\\Reader\\TranslationReaderInterface' => $vendorDir . '/symfony/translation/Reader/TranslationReaderInterface.php',
    'Symfony\\Component\\Translation\\StaticMessage' => $vendorDir . '/symfony/translation/StaticMessage.php',
    'Symfony\\Component\\Translation\\Test\\AbstractProviderFactoryTestCase' => $vendorDir . '/symfony/translation/Test/AbstractProviderFactoryTestCase.php',
    'Symfony\\Component\\Translation\\Test\\IncompleteDsnTestTrait' => $vendorDir . '/symfony/translation/Test/IncompleteDsnTestTrait.php',
    'Symfony\\Component\\Translation\\Test\\ProviderFactoryTestCase' => $vendorDir . '/symfony/translation/Test/ProviderFactoryTestCase.php',
    'Symfony\\Component\\Translation\\Test\\ProviderTestCase' => $vendorDir . '/symfony/translation/Test/ProviderTestCase.php',
    'Symfony\\Component\\Translation\\TranslatableMessage' => $vendorDir . '/symfony/translation/TranslatableMessage.php',
    'Symfony\\Component\\Translation\\Translator' => $vendorDir . '/symfony/translation/Translator.php',
    'Symfony\\Component\\Translation\\TranslatorBag' => $vendorDir . '/symfony/translation/TranslatorBag.php',
    'Symfony\\Component\\Translation\\TranslatorBagInterface' => $vendorDir . '/symfony/translation/TranslatorBagInterface.php',
    'Symfony\\Component\\Translation\\Util\\ArrayConverter' => $vendorDir . '/symfony/translation/Util/ArrayConverter.php',
    'Symfony\\Component\\Translation\\Util\\XliffUtils' => $vendorDir . '/symfony/translation/Util/XliffUtils.php',
    'Symfony\\Component\\Translation\\Writer\\TranslationWriter' => $vendorDir . '/symfony/translation/Writer/TranslationWriter.php',
    'Symfony\\Component\\Translation\\Writer\\TranslationWriterInterface' => $vendorDir . '/symfony/translation/Writer/TranslationWriterInterface.php',
    'Symfony\\Component\\VarDumper\\Caster\\AddressInfoCaster' => $vendorDir . '/symfony/var-dumper/Caster/AddressInfoCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\AmqpCaster' => $vendorDir . '/symfony/var-dumper/Caster/AmqpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ArgsStub' => $vendorDir . '/symfony/var-dumper/Caster/ArgsStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\Caster' => $vendorDir . '/symfony/var-dumper/Caster/Caster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ClassStub' => $vendorDir . '/symfony/var-dumper/Caster/ClassStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ConstStub' => $vendorDir . '/symfony/var-dumper/Caster/ConstStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CurlCaster' => $vendorDir . '/symfony/var-dumper/Caster/CurlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutArrayStub' => $vendorDir . '/symfony/var-dumper/Caster/CutArrayStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutStub' => $vendorDir . '/symfony/var-dumper/Caster/CutStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\DOMCaster' => $vendorDir . '/symfony/var-dumper/Caster/DOMCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DateCaster' => $vendorDir . '/symfony/var-dumper/Caster/DateCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DoctrineCaster' => $vendorDir . '/symfony/var-dumper/Caster/DoctrineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsCaster' => $vendorDir . '/symfony/var-dumper/Caster/DsCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsPairStub' => $vendorDir . '/symfony/var-dumper/Caster/DsPairStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\EnumStub' => $vendorDir . '/symfony/var-dumper/Caster/EnumStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ExceptionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ExceptionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FFICaster' => $vendorDir . '/symfony/var-dumper/Caster/FFICaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FiberCaster' => $vendorDir . '/symfony/var-dumper/Caster/FiberCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FrameStub' => $vendorDir . '/symfony/var-dumper/Caster/FrameStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\GdCaster' => $vendorDir . '/symfony/var-dumper/Caster/GdCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\GmpCaster' => $vendorDir . '/symfony/var-dumper/Caster/GmpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImagineCaster' => $vendorDir . '/symfony/var-dumper/Caster/ImagineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImgStub' => $vendorDir . '/symfony/var-dumper/Caster/ImgStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\IntlCaster' => $vendorDir . '/symfony/var-dumper/Caster/IntlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\LinkStub' => $vendorDir . '/symfony/var-dumper/Caster/LinkStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\MemcachedCaster' => $vendorDir . '/symfony/var-dumper/Caster/MemcachedCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\MysqliCaster' => $vendorDir . '/symfony/var-dumper/Caster/MysqliCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\OpenSSLCaster' => $vendorDir . '/symfony/var-dumper/Caster/OpenSSLCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PdoCaster' => $vendorDir . '/symfony/var-dumper/Caster/PdoCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PgSqlCaster' => $vendorDir . '/symfony/var-dumper/Caster/PgSqlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ProxyManagerCaster' => $vendorDir . '/symfony/var-dumper/Caster/ProxyManagerCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RdKafkaCaster' => $vendorDir . '/symfony/var-dumper/Caster/RdKafkaCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RedisCaster' => $vendorDir . '/symfony/var-dumper/Caster/RedisCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ReflectionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ReflectionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/ResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ScalarStub' => $vendorDir . '/symfony/var-dumper/Caster/ScalarStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\SocketCaster' => $vendorDir . '/symfony/var-dumper/Caster/SocketCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\SplCaster' => $vendorDir . '/symfony/var-dumper/Caster/SplCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\SqliteCaster' => $vendorDir . '/symfony/var-dumper/Caster/SqliteCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\StubCaster' => $vendorDir . '/symfony/var-dumper/Caster/StubCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\SymfonyCaster' => $vendorDir . '/symfony/var-dumper/Caster/SymfonyCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\TraceStub' => $vendorDir . '/symfony/var-dumper/Caster/TraceStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UninitializedStub' => $vendorDir . '/symfony/var-dumper/Caster/UninitializedStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UuidCaster' => $vendorDir . '/symfony/var-dumper/Caster/UuidCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\VirtualStub' => $vendorDir . '/symfony/var-dumper/Caster/VirtualStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlReaderCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlReaderCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Cloner\\AbstractCloner' => $vendorDir . '/symfony/var-dumper/Cloner/AbstractCloner.php',
    'Symfony\\Component\\VarDumper\\Cloner\\ClonerInterface' => $vendorDir . '/symfony/var-dumper/Cloner/ClonerInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Cursor' => $vendorDir . '/symfony/var-dumper/Cloner/Cursor.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Data' => $vendorDir . '/symfony/var-dumper/Cloner/Data.php',
    'Symfony\\Component\\VarDumper\\Cloner\\DumperInterface' => $vendorDir . '/symfony/var-dumper/Cloner/DumperInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Stub' => $vendorDir . '/symfony/var-dumper/Cloner/Stub.php',
    'Symfony\\Component\\VarDumper\\Cloner\\VarCloner' => $vendorDir . '/symfony/var-dumper/Cloner/VarCloner.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\CliDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/CliDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\DumpDescriptorInterface' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/DumpDescriptorInterface.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\HtmlDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/HtmlDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\ServerDumpCommand' => $vendorDir . '/symfony/var-dumper/Command/ServerDumpCommand.php',
    'Symfony\\Component\\VarDumper\\Dumper\\AbstractDumper' => $vendorDir . '/symfony/var-dumper/Dumper/AbstractDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\CliDumper' => $vendorDir . '/symfony/var-dumper/Dumper/CliDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\CliContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/CliContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\ContextProviderInterface' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/ContextProviderInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\RequestContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/RequestContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\SourceContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/SourceContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextualizedDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ContextualizedDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\DataDumperInterface' => $vendorDir . '/symfony/var-dumper/Dumper/DataDumperInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\HtmlDumper' => $vendorDir . '/symfony/var-dumper/Dumper/HtmlDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ServerDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ServerDumper.php',
    'Symfony\\Component\\VarDumper\\Exception\\ThrowingCasterException' => $vendorDir . '/symfony/var-dumper/Exception/ThrowingCasterException.php',
    'Symfony\\Component\\VarDumper\\Server\\Connection' => $vendorDir . '/symfony/var-dumper/Server/Connection.php',
    'Symfony\\Component\\VarDumper\\Server\\DumpServer' => $vendorDir . '/symfony/var-dumper/Server/DumpServer.php',
    'Symfony\\Component\\VarDumper\\Test\\VarDumperTestTrait' => $vendorDir . '/symfony/var-dumper/Test/VarDumperTestTrait.php',
    'Symfony\\Component\\VarDumper\\VarDumper' => $vendorDir . '/symfony/var-dumper/VarDumper.php',
    'Symfony\\Component\\VarExporter\\Exception\\ClassNotFoundException' => $vendorDir . '/symfony/var-exporter/Exception/ClassNotFoundException.php',
    'Symfony\\Component\\VarExporter\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/var-exporter/Exception/ExceptionInterface.php',
    'Symfony\\Component\\VarExporter\\Exception\\LogicException' => $vendorDir . '/symfony/var-exporter/Exception/LogicException.php',
    'Symfony\\Component\\VarExporter\\Exception\\NotInstantiableTypeException' => $vendorDir . '/symfony/var-exporter/Exception/NotInstantiableTypeException.php',
    'Symfony\\Component\\VarExporter\\Hydrator' => $vendorDir . '/symfony/var-exporter/Hydrator.php',
    'Symfony\\Component\\VarExporter\\Instantiator' => $vendorDir . '/symfony/var-exporter/Instantiator.php',
    'Symfony\\Component\\VarExporter\\Internal\\Exporter' => $vendorDir . '/symfony/var-exporter/Internal/Exporter.php',
    'Symfony\\Component\\VarExporter\\Internal\\Hydrator' => $vendorDir . '/symfony/var-exporter/Internal/Hydrator.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyDecoratorTrait' => $vendorDir . '/symfony/var-exporter/Internal/LazyDecoratorTrait.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectRegistry' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectRegistry.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectState' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectState.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectTrait' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectTrait.php',
    'Symfony\\Component\\VarExporter\\Internal\\NamedClosure' => $vendorDir . '/symfony/var-exporter/Internal/NamedClosure.php',
    'Symfony\\Component\\VarExporter\\Internal\\Reference' => $vendorDir . '/symfony/var-exporter/Internal/Reference.php',
    'Symfony\\Component\\VarExporter\\Internal\\Registry' => $vendorDir . '/symfony/var-exporter/Internal/Registry.php',
    'Symfony\\Component\\VarExporter\\Internal\\Values' => $vendorDir . '/symfony/var-exporter/Internal/Values.php',
    'Symfony\\Component\\VarExporter\\LazyGhostTrait' => $vendorDir . '/symfony/var-exporter/LazyGhostTrait.php',
    'Symfony\\Component\\VarExporter\\LazyObjectInterface' => $vendorDir . '/symfony/var-exporter/LazyObjectInterface.php',
    'Symfony\\Component\\VarExporter\\LazyProxyTrait' => $vendorDir . '/symfony/var-exporter/LazyProxyTrait.php',
    'Symfony\\Component\\VarExporter\\ProxyHelper' => $vendorDir . '/symfony/var-exporter/ProxyHelper.php',
    'Symfony\\Component\\VarExporter\\VarExporter' => $vendorDir . '/symfony/var-exporter/VarExporter.php',
    'Symfony\\Component\\Yaml\\Command\\LintCommand' => $vendorDir . '/symfony/yaml/Command/LintCommand.php',
    'Symfony\\Component\\Yaml\\Dumper' => $vendorDir . '/symfony/yaml/Dumper.php',
    'Symfony\\Component\\Yaml\\Escaper' => $vendorDir . '/symfony/yaml/Escaper.php',
    'Symfony\\Component\\Yaml\\Exception\\DumpException' => $vendorDir . '/symfony/yaml/Exception/DumpException.php',
    'Symfony\\Component\\Yaml\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/yaml/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Yaml\\Exception\\ParseException' => $vendorDir . '/symfony/yaml/Exception/ParseException.php',
    'Symfony\\Component\\Yaml\\Exception\\RuntimeException' => $vendorDir . '/symfony/yaml/Exception/RuntimeException.php',
    'Symfony\\Component\\Yaml\\Inline' => $vendorDir . '/symfony/yaml/Inline.php',
    'Symfony\\Component\\Yaml\\Parser' => $vendorDir . '/symfony/yaml/Parser.php',
    'Symfony\\Component\\Yaml\\ParserState' => $vendorDir . '/symfony/yaml/ParserState.php',
    'Symfony\\Component\\Yaml\\Tag\\TaggedValue' => $vendorDir . '/symfony/yaml/Tag/TaggedValue.php',
    'Symfony\\Component\\Yaml\\Unescaper' => $vendorDir . '/symfony/yaml/Unescaper.php',
    'Symfony\\Component\\Yaml\\Yaml' => $vendorDir . '/symfony/yaml/Yaml.php',
    'Symfony\\Contracts\\Cache\\CacheInterface' => $vendorDir . '/symfony/cache-contracts/CacheInterface.php',
    'Symfony\\Contracts\\Cache\\CacheTrait' => $vendorDir . '/symfony/cache-contracts/CacheTrait.php',
    'Symfony\\Contracts\\Cache\\CallbackInterface' => $vendorDir . '/symfony/cache-contracts/CallbackInterface.php',
    'Symfony\\Contracts\\Cache\\ItemInterface' => $vendorDir . '/symfony/cache-contracts/ItemInterface.php',
    'Symfony\\Contracts\\Cache\\NamespacedPoolInterface' => $vendorDir . '/symfony/cache-contracts/NamespacedPoolInterface.php',
    'Symfony\\Contracts\\Cache\\TagAwareCacheInterface' => $vendorDir . '/symfony/cache-contracts/TagAwareCacheInterface.php',
    'Symfony\\Contracts\\EventDispatcher\\Event' => $vendorDir . '/symfony/event-dispatcher-contracts/Event.php',
    'Symfony\\Contracts\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher-contracts/EventDispatcherInterface.php',
    'Symfony\\Contracts\\HttpClient\\ChunkInterface' => $vendorDir . '/symfony/http-client-contracts/ChunkInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\ClientExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/ClientExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\DecodingExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/DecodingExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/ExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\HttpExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/HttpExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\RedirectionExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/RedirectionExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\ServerExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/ServerExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\TimeoutExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/TimeoutExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\Exception\\TransportExceptionInterface' => $vendorDir . '/symfony/http-client-contracts/Exception/TransportExceptionInterface.php',
    'Symfony\\Contracts\\HttpClient\\HttpClientInterface' => $vendorDir . '/symfony/http-client-contracts/HttpClientInterface.php',
    'Symfony\\Contracts\\HttpClient\\ResponseInterface' => $vendorDir . '/symfony/http-client-contracts/ResponseInterface.php',
    'Symfony\\Contracts\\HttpClient\\ResponseStreamInterface' => $vendorDir . '/symfony/http-client-contracts/ResponseStreamInterface.php',
    'Symfony\\Contracts\\Service\\Attribute\\Required' => $vendorDir . '/symfony/service-contracts/Attribute/Required.php',
    'Symfony\\Contracts\\Service\\Attribute\\SubscribedService' => $vendorDir . '/symfony/service-contracts/Attribute/SubscribedService.php',
    'Symfony\\Contracts\\Service\\ContainerAwareInterface' => $vendorDir . '/symfony/service-contracts/ContainerAwareInterface.php',
    'Symfony\\Contracts\\Service\\ContainerProviderInterface' => $vendorDir . '/symfony/service-contracts/ContainerProviderInterface.php',
    'Symfony\\Contracts\\Service\\ResetInterface' => $vendorDir . '/symfony/service-contracts/ResetInterface.php',
    'Symfony\\Contracts\\Service\\ServiceCollectionInterface' => $vendorDir . '/symfony/service-contracts/ServiceCollectionInterface.php',
    'Symfony\\Contracts\\Service\\ServiceLocatorTrait' => $vendorDir . '/symfony/service-contracts/ServiceLocatorTrait.php',
    'Symfony\\Contracts\\Service\\ServiceMethodsSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceMethodsSubscriberTrait.php',
    'Symfony\\Contracts\\Service\\ServiceProviderInterface' => $vendorDir . '/symfony/service-contracts/ServiceProviderInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberInterface' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberTrait.php',
    'Symfony\\Contracts\\Translation\\LocaleAwareInterface' => $vendorDir . '/symfony/translation-contracts/LocaleAwareInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatableInterface' => $vendorDir . '/symfony/translation-contracts/TranslatableInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorInterface' => $vendorDir . '/symfony/translation-contracts/TranslatorInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorTrait' => $vendorDir . '/symfony/translation-contracts/TranslatorTrait.php',
    'Symfony\\Polyfill\\Ctype\\Ctype' => $vendorDir . '/symfony/polyfill-ctype/Ctype.php',
    'Symfony\\Polyfill\\Intl\\Grapheme\\Grapheme' => $vendorDir . '/symfony/polyfill-intl-grapheme/Grapheme.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Collator' => $vendorDir . '/symfony/polyfill-intl-icu/Collator.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Currencies' => $vendorDir . '/symfony/polyfill-intl-icu/Currencies.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\AmPmTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/AmPmTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\DayOfWeekTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/DayOfWeekTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\DayOfYearTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/DayOfYearTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\DayTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/DayTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\FullTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/FullTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\Hour1200Transformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/Hour1200Transformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\Hour1201Transformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/Hour1201Transformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\Hour2400Transformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/Hour2400Transformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\Hour2401Transformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/Hour2401Transformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\HourTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/HourTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\MinuteTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/MinuteTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\MonthTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/MonthTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\QuarterTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/QuarterTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\SecondTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/SecondTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\TimezoneTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/TimezoneTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\Transformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/Transformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\DateFormat\\YearTransformer' => $vendorDir . '/symfony/polyfill-intl-icu/DateFormat/YearTransformer.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/ExceptionInterface.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\MethodArgumentNotImplementedException' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/MethodArgumentNotImplementedException.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\MethodArgumentValueNotImplementedException' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/MethodArgumentValueNotImplementedException.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\MethodNotImplementedException' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/MethodNotImplementedException.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\NotImplementedException' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/NotImplementedException.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Exception\\RuntimeException' => $vendorDir . '/symfony/polyfill-intl-icu/Exception/RuntimeException.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Icu' => $vendorDir . '/symfony/polyfill-intl-icu/Icu.php',
    'Symfony\\Polyfill\\Intl\\Icu\\IntlDateFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/IntlDateFormatter.php',
    'Symfony\\Polyfill\\Intl\\Icu\\IntlListFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/IntlListFormatter.php',
    'Symfony\\Polyfill\\Intl\\Icu\\Locale' => $vendorDir . '/symfony/polyfill-intl-icu/Locale.php',
    'Symfony\\Polyfill\\Intl\\Icu\\NumberFormatter' => $vendorDir . '/symfony/polyfill-intl-icu/NumberFormatter.php',
    'Symfony\\Polyfill\\Intl\\Normalizer\\Normalizer' => $vendorDir . '/symfony/polyfill-intl-normalizer/Normalizer.php',
    'Symfony\\Polyfill\\Mbstring\\Mbstring' => $vendorDir . '/symfony/polyfill-mbstring/Mbstring.php',
    'Symfony\\Polyfill\\Php80\\Php80' => $vendorDir . '/symfony/polyfill-php80/Php80.php',
    'Symfony\\Polyfill\\Php80\\PhpToken' => $vendorDir . '/symfony/polyfill-php80/PhpToken.php',
    'Symfony\\Polyfill\\Php83\\Php83' => $vendorDir . '/symfony/polyfill-php83/Php83.php',
    'Symfony\\Polyfill\\Php85\\Php85' => $vendorDir . '/symfony/polyfill-php85/Php85.php',
    'Twig\\AbstractTwigCallable' => $vendorDir . '/twig/twig/src/AbstractTwigCallable.php',
    'Twig\\Attribute\\AsTwigFilter' => $vendorDir . '/twig/twig/src/Attribute/AsTwigFilter.php',
    'Twig\\Attribute\\AsTwigFunction' => $vendorDir . '/twig/twig/src/Attribute/AsTwigFunction.php',
    'Twig\\Attribute\\AsTwigTest' => $vendorDir . '/twig/twig/src/Attribute/AsTwigTest.php',
    'Twig\\Attribute\\FirstClassTwigCallableReady' => $vendorDir . '/twig/twig/src/Attribute/FirstClassTwigCallableReady.php',
    'Twig\\Attribute\\YieldReady' => $vendorDir . '/twig/twig/src/Attribute/YieldReady.php',
    'Twig\\Cache\\CacheInterface' => $vendorDir . '/twig/twig/src/Cache/CacheInterface.php',
    'Twig\\Cache\\ChainCache' => $vendorDir . '/twig/twig/src/Cache/ChainCache.php',
    'Twig\\Cache\\FilesystemCache' => $vendorDir . '/twig/twig/src/Cache/FilesystemCache.php',
    'Twig\\Cache\\NullCache' => $vendorDir . '/twig/twig/src/Cache/NullCache.php',
    'Twig\\Cache\\ReadOnlyFilesystemCache' => $vendorDir . '/twig/twig/src/Cache/ReadOnlyFilesystemCache.php',
    'Twig\\Cache\\RemovableCacheInterface' => $vendorDir . '/twig/twig/src/Cache/RemovableCacheInterface.php',
    'Twig\\Compiler' => $vendorDir . '/twig/twig/src/Compiler.php',
    'Twig\\DeprecatedCallableInfo' => $vendorDir . '/twig/twig/src/DeprecatedCallableInfo.php',
    'Twig\\Environment' => $vendorDir . '/twig/twig/src/Environment.php',
    'Twig\\Error\\Error' => $vendorDir . '/twig/twig/src/Error/Error.php',
    'Twig\\Error\\LoaderError' => $vendorDir . '/twig/twig/src/Error/LoaderError.php',
    'Twig\\Error\\RuntimeError' => $vendorDir . '/twig/twig/src/Error/RuntimeError.php',
    'Twig\\Error\\SyntaxError' => $vendorDir . '/twig/twig/src/Error/SyntaxError.php',
    'Twig\\ExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser.php',
    'Twig\\ExpressionParser\\AbstractExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/AbstractExpressionParser.php',
    'Twig\\ExpressionParser\\ExpressionParserDescriptionInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserDescriptionInterface.php',
    'Twig\\ExpressionParser\\ExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserInterface.php',
    'Twig\\ExpressionParser\\ExpressionParserType' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserType.php',
    'Twig\\ExpressionParser\\ExpressionParsers' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParsers.php',
    'Twig\\ExpressionParser\\InfixAssociativity' => $vendorDir . '/twig/twig/src/ExpressionParser/InfixAssociativity.php',
    'Twig\\ExpressionParser\\InfixExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/InfixExpressionParserInterface.php',
    'Twig\\ExpressionParser\\Infix\\ArgumentsTrait' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ArgumentsTrait.php',
    'Twig\\ExpressionParser\\Infix\\ArrowExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ArrowExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\AssignmentExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/AssignmentExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\BinaryOperatorExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/BinaryOperatorExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\ConditionalTernaryExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ConditionalTernaryExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\DotExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/DotExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\FilterExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/FilterExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\FunctionExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/FunctionExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\IsExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/IsExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\IsNotExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/IsNotExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\SquareBracketExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/SquareBracketExpressionParser.php',
    'Twig\\ExpressionParser\\PrecedenceChange' => $vendorDir . '/twig/twig/src/ExpressionParser/PrecedenceChange.php',
    'Twig\\ExpressionParser\\PrefixExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/PrefixExpressionParserInterface.php',
    'Twig\\ExpressionParser\\Prefix\\GroupingExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/GroupingExpressionParser.php',
    'Twig\\ExpressionParser\\Prefix\\LiteralExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/LiteralExpressionParser.php',
    'Twig\\ExpressionParser\\Prefix\\UnaryOperatorExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/UnaryOperatorExpressionParser.php',
    'Twig\\ExtensionSet' => $vendorDir . '/twig/twig/src/ExtensionSet.php',
    'Twig\\Extension\\AbstractExtension' => $vendorDir . '/twig/twig/src/Extension/AbstractExtension.php',
    'Twig\\Extension\\AttributeExtension' => $vendorDir . '/twig/twig/src/Extension/AttributeExtension.php',
    'Twig\\Extension\\CoreExtension' => $vendorDir . '/twig/twig/src/Extension/CoreExtension.php',
    'Twig\\Extension\\DebugExtension' => $vendorDir . '/twig/twig/src/Extension/DebugExtension.php',
    'Twig\\Extension\\EscaperExtension' => $vendorDir . '/twig/twig/src/Extension/EscaperExtension.php',
    'Twig\\Extension\\ExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/ExtensionInterface.php',
    'Twig\\Extension\\GlobalsInterface' => $vendorDir . '/twig/twig/src/Extension/GlobalsInterface.php',
    'Twig\\Extension\\LastModifiedExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/LastModifiedExtensionInterface.php',
    'Twig\\Extension\\OptimizerExtension' => $vendorDir . '/twig/twig/src/Extension/OptimizerExtension.php',
    'Twig\\Extension\\ProfilerExtension' => $vendorDir . '/twig/twig/src/Extension/ProfilerExtension.php',
    'Twig\\Extension\\RuntimeExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/RuntimeExtensionInterface.php',
    'Twig\\Extension\\SandboxExtension' => $vendorDir . '/twig/twig/src/Extension/SandboxExtension.php',
    'Twig\\Extension\\StagingExtension' => $vendorDir . '/twig/twig/src/Extension/StagingExtension.php',
    'Twig\\Extension\\StringLoaderExtension' => $vendorDir . '/twig/twig/src/Extension/StringLoaderExtension.php',
    'Twig\\Extension\\YieldNotReadyExtension' => $vendorDir . '/twig/twig/src/Extension/YieldNotReadyExtension.php',
    'Twig\\Extra\\Intl\\IntlExtension' => $vendorDir . '/twig/intl-extra/IntlExtension.php',
    'Twig\\FileExtensionEscapingStrategy' => $vendorDir . '/twig/twig/src/FileExtensionEscapingStrategy.php',
    'Twig\\Lexer' => $vendorDir . '/twig/twig/src/Lexer.php',
    'Twig\\Loader\\ArrayLoader' => $vendorDir . '/twig/twig/src/Loader/ArrayLoader.php',
    'Twig\\Loader\\ChainLoader' => $vendorDir . '/twig/twig/src/Loader/ChainLoader.php',
    'Twig\\Loader\\FilesystemLoader' => $vendorDir . '/twig/twig/src/Loader/FilesystemLoader.php',
    'Twig\\Loader\\LoaderInterface' => $vendorDir . '/twig/twig/src/Loader/LoaderInterface.php',
    'Twig\\Markup' => $vendorDir . '/twig/twig/src/Markup.php',
    'Twig\\NodeTraverser' => $vendorDir . '/twig/twig/src/NodeTraverser.php',
    'Twig\\NodeVisitor\\AbstractNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/AbstractNodeVisitor.php',
    'Twig\\NodeVisitor\\CorrectnessNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/CorrectnessNodeVisitor.php',
    'Twig\\NodeVisitor\\EscaperNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/EscaperNodeVisitor.php',
    'Twig\\NodeVisitor\\NodeVisitorInterface' => $vendorDir . '/twig/twig/src/NodeVisitor/NodeVisitorInterface.php',
    'Twig\\NodeVisitor\\OptimizerNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/OptimizerNodeVisitor.php',
    'Twig\\NodeVisitor\\SafeAnalysisNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SafeAnalysisNodeVisitor.php',
    'Twig\\NodeVisitor\\SandboxNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SandboxNodeVisitor.php',
    'Twig\\NodeVisitor\\YieldNotReadyNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/YieldNotReadyNodeVisitor.php',
    'Twig\\Node\\AutoEscapeNode' => $vendorDir . '/twig/twig/src/Node/AutoEscapeNode.php',
    'Twig\\Node\\BlockNode' => $vendorDir . '/twig/twig/src/Node/BlockNode.php',
    'Twig\\Node\\BlockReferenceNode' => $vendorDir . '/twig/twig/src/Node/BlockReferenceNode.php',
    'Twig\\Node\\BodyNode' => $vendorDir . '/twig/twig/src/Node/BodyNode.php',
    'Twig\\Node\\CaptureNode' => $vendorDir . '/twig/twig/src/Node/CaptureNode.php',
    'Twig\\Node\\CheckSecurityCallNode' => $vendorDir . '/twig/twig/src/Node/CheckSecurityCallNode.php',
    'Twig\\Node\\CheckSecurityNode' => $vendorDir . '/twig/twig/src/Node/CheckSecurityNode.php',
    'Twig\\Node\\CheckToStringNode' => $vendorDir . '/twig/twig/src/Node/CheckToStringNode.php',
    'Twig\\Node\\CoercesChildrenToStringInterface' => $vendorDir . '/twig/twig/src/Node/CoercesChildrenToStringInterface.php',
    'Twig\\Node\\ConfigNode' => $vendorDir . '/twig/twig/src/Node/ConfigNode.php',
    'Twig\\Node\\DeprecatedNode' => $vendorDir . '/twig/twig/src/Node/DeprecatedNode.php',
    'Twig\\Node\\DoNode' => $vendorDir . '/twig/twig/src/Node/DoNode.php',
    'Twig\\Node\\EmbedNode' => $vendorDir . '/twig/twig/src/Node/EmbedNode.php',
    'Twig\\Node\\EmptyNode' => $vendorDir . '/twig/twig/src/Node/EmptyNode.php',
    'Twig\\Node\\Expression\\AbstractExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AbstractExpression.php',
    'Twig\\Node\\Expression\\ArrayExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ArrayExpression.php',
    'Twig\\Node\\Expression\\ArrowFunctionExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ArrowFunctionExpression.php',
    'Twig\\Node\\Expression\\AssignNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AssignNameExpression.php',
    'Twig\\Node\\Expression\\Binary\\AbstractBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AbstractBinary.php',
    'Twig\\Node\\Expression\\Binary\\AddBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AddBinary.php',
    'Twig\\Node\\Expression\\Binary\\AndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BinaryInterface' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BinaryInterface.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseAndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseAndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseOrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseOrBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseXorBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseXorBinary.php',
    'Twig\\Node\\Expression\\Binary\\ConcatBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ConcatBinary.php',
    'Twig\\Node\\Expression\\Binary\\DivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/DivBinary.php',
    'Twig\\Node\\Expression\\Binary\\ElvisBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ElvisBinary.php',
    'Twig\\Node\\Expression\\Binary\\EndsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EndsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\EqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\FloorDivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/FloorDivBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\HasEveryBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/HasEveryBinary.php',
    'Twig\\Node\\Expression\\Binary\\HasSomeBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/HasSomeBinary.php',
    'Twig\\Node\\Expression\\Binary\\InBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/InBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\MatchesBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MatchesBinary.php',
    'Twig\\Node\\Expression\\Binary\\ModBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ModBinary.php',
    'Twig\\Node\\Expression\\Binary\\MulBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MulBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotInBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotInBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotSameAsBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotSameAsBinary.php',
    'Twig\\Node\\Expression\\Binary\\NullCoalesceBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NullCoalesceBinary.php',
    'Twig\\Node\\Expression\\Binary\\ObjectDestructuringSetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ObjectDestructuringSetBinary.php',
    'Twig\\Node\\Expression\\Binary\\OrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/OrBinary.php',
    'Twig\\Node\\Expression\\Binary\\PowerBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/PowerBinary.php',
    'Twig\\Node\\Expression\\Binary\\RangeBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/RangeBinary.php',
    'Twig\\Node\\Expression\\Binary\\SameAsBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SameAsBinary.php',
    'Twig\\Node\\Expression\\Binary\\SequenceDestructuringSetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SequenceDestructuringSetBinary.php',
    'Twig\\Node\\Expression\\Binary\\SetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SetBinary.php',
    'Twig\\Node\\Expression\\Binary\\SpaceshipBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SpaceshipBinary.php',
    'Twig\\Node\\Expression\\Binary\\StartsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/StartsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\SubBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SubBinary.php',
    'Twig\\Node\\Expression\\Binary\\XorBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/XorBinary.php',
    'Twig\\Node\\Expression\\BlockReferenceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/BlockReferenceExpression.php',
    'Twig\\Node\\Expression\\CallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/CallExpression.php',
    'Twig\\Node\\Expression\\ConditionalExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConditionalExpression.php',
    'Twig\\Node\\Expression\\ConstantExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConstantExpression.php',
    'Twig\\Node\\Expression\\EmptyExpression' => $vendorDir . '/twig/twig/src/Node/Expression/EmptyExpression.php',
    'Twig\\Node\\Expression\\FilterExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FilterExpression.php',
    'Twig\\Node\\Expression\\Filter\\DefaultFilter' => $vendorDir . '/twig/twig/src/Node/Expression/Filter/DefaultFilter.php',
    'Twig\\Node\\Expression\\Filter\\RawFilter' => $vendorDir . '/twig/twig/src/Node/Expression/Filter/RawFilter.php',
    'Twig\\Node\\Expression\\FunctionExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionExpression.php',
    'Twig\\Node\\Expression\\FunctionNode\\EnumCasesFunction' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionNode/EnumCasesFunction.php',
    'Twig\\Node\\Expression\\FunctionNode\\EnumFunction' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionNode/EnumFunction.php',
    'Twig\\Node\\Expression\\GetAttrExpression' => $vendorDir . '/twig/twig/src/Node/Expression/GetAttrExpression.php',
    'Twig\\Node\\Expression\\InlinePrint' => $vendorDir . '/twig/twig/src/Node/Expression/InlinePrint.php',
    'Twig\\Node\\Expression\\ListExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ListExpression.php',
    'Twig\\Node\\Expression\\MacroReferenceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/MacroReferenceExpression.php',
    'Twig\\Node\\Expression\\MethodCallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/MethodCallExpression.php',
    'Twig\\Node\\Expression\\NameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NameExpression.php',
    'Twig\\Node\\Expression\\NullCoalesceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NullCoalesceExpression.php',
    'Twig\\Node\\Expression\\OperatorEscapeInterface' => $vendorDir . '/twig/twig/src/Node/Expression/OperatorEscapeInterface.php',
    'Twig\\Node\\Expression\\ParentExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ParentExpression.php',
    'Twig\\Node\\Expression\\ReturnArrayInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnArrayInterface.php',
    'Twig\\Node\\Expression\\ReturnBoolInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnBoolInterface.php',
    'Twig\\Node\\Expression\\ReturnNumberInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnNumberInterface.php',
    'Twig\\Node\\Expression\\ReturnPrimitiveTypeInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnPrimitiveTypeInterface.php',
    'Twig\\Node\\Expression\\ReturnStringInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnStringInterface.php',
    'Twig\\Node\\Expression\\SupportDefinedTestDeprecationTrait' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestDeprecationTrait.php',
    'Twig\\Node\\Expression\\SupportDefinedTestInterface' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestInterface.php',
    'Twig\\Node\\Expression\\SupportDefinedTestTrait' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestTrait.php',
    'Twig\\Node\\Expression\\TempNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TempNameExpression.php',
    'Twig\\Node\\Expression\\Ternary\\ConditionalTernary' => $vendorDir . '/twig/twig/src/Node/Expression/Ternary/ConditionalTernary.php',
    'Twig\\Node\\Expression\\TestExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TestExpression.php',
    'Twig\\Node\\Expression\\Test\\ConstantTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/ConstantTest.php',
    'Twig\\Node\\Expression\\Test\\DefinedTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DefinedTest.php',
    'Twig\\Node\\Expression\\Test\\DivisiblebyTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DivisiblebyTest.php',
    'Twig\\Node\\Expression\\Test\\EvenTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/EvenTest.php',
    'Twig\\Node\\Expression\\Test\\NullTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/NullTest.php',
    'Twig\\Node\\Expression\\Test\\OddTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/OddTest.php',
    'Twig\\Node\\Expression\\Test\\SameasTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/SameasTest.php',
    'Twig\\Node\\Expression\\Test\\TrueTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/TrueTest.php',
    'Twig\\Node\\Expression\\Unary\\AbstractUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/AbstractUnary.php',
    'Twig\\Node\\Expression\\Unary\\NegUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NegUnary.php',
    'Twig\\Node\\Expression\\Unary\\NotUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NotUnary.php',
    'Twig\\Node\\Expression\\Unary\\PosUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/PosUnary.php',
    'Twig\\Node\\Expression\\Unary\\SpreadUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/SpreadUnary.php',
    'Twig\\Node\\Expression\\Unary\\StringCastUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/StringCastUnary.php',
    'Twig\\Node\\Expression\\Unary\\UnaryInterface' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/UnaryInterface.php',
    'Twig\\Node\\Expression\\Variable\\AssignContextVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/AssignContextVariable.php',
    'Twig\\Node\\Expression\\Variable\\AssignTemplateVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/AssignTemplateVariable.php',
    'Twig\\Node\\Expression\\Variable\\ContextVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/ContextVariable.php',
    'Twig\\Node\\Expression\\Variable\\LocalVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/LocalVariable.php',
    'Twig\\Node\\Expression\\Variable\\TemplateVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/TemplateVariable.php',
    'Twig\\Node\\Expression\\VariadicExpression' => $vendorDir . '/twig/twig/src/Node/Expression/VariadicExpression.php',
    'Twig\\Node\\FlushNode' => $vendorDir . '/twig/twig/src/Node/FlushNode.php',
    'Twig\\Node\\ForElseNode' => $vendorDir . '/twig/twig/src/Node/ForElseNode.php',
    'Twig\\Node\\ForLoopNode' => $vendorDir . '/twig/twig/src/Node/ForLoopNode.php',
    'Twig\\Node\\ForNode' => $vendorDir . '/twig/twig/src/Node/ForNode.php',
    'Twig\\Node\\IfNode' => $vendorDir . '/twig/twig/src/Node/IfNode.php',
    'Twig\\Node\\ImportNode' => $vendorDir . '/twig/twig/src/Node/ImportNode.php',
    'Twig\\Node\\IncludeNode' => $vendorDir . '/twig/twig/src/Node/IncludeNode.php',
    'Twig\\Node\\MacroNode' => $vendorDir . '/twig/twig/src/Node/MacroNode.php',
    'Twig\\Node\\ModuleNode' => $vendorDir . '/twig/twig/src/Node/ModuleNode.php',
    'Twig\\Node\\NameDeprecation' => $vendorDir . '/twig/twig/src/Node/NameDeprecation.php',
    'Twig\\Node\\Node' => $vendorDir . '/twig/twig/src/Node/Node.php',
    'Twig\\Node\\NodeCaptureInterface' => $vendorDir . '/twig/twig/src/Node/NodeCaptureInterface.php',
    'Twig\\Node\\NodeOutputInterface' => $vendorDir . '/twig/twig/src/Node/NodeOutputInterface.php',
    'Twig\\Node\\Nodes' => $vendorDir . '/twig/twig/src/Node/Nodes.php',
    'Twig\\Node\\PrintNode' => $vendorDir . '/twig/twig/src/Node/PrintNode.php',
    'Twig\\Node\\SandboxNode' => $vendorDir . '/twig/twig/src/Node/SandboxNode.php',
    'Twig\\Node\\SetNode' => $vendorDir . '/twig/twig/src/Node/SetNode.php',
    'Twig\\Node\\TextNode' => $vendorDir . '/twig/twig/src/Node/TextNode.php',
    'Twig\\Node\\TypesNode' => $vendorDir . '/twig/twig/src/Node/TypesNode.php',
    'Twig\\Node\\WithNode' => $vendorDir . '/twig/twig/src/Node/WithNode.php',
    'Twig\\OperatorPrecedenceChange' => $vendorDir . '/twig/twig/src/OperatorPrecedenceChange.php',
    'Twig\\Parser' => $vendorDir . '/twig/twig/src/Parser.php',
    'Twig\\Profiler\\Dumper\\BaseDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BaseDumper.php',
    'Twig\\Profiler\\Dumper\\BlackfireDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BlackfireDumper.php',
    'Twig\\Profiler\\Dumper\\HtmlDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/HtmlDumper.php',
    'Twig\\Profiler\\Dumper\\TextDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/TextDumper.php',
    'Twig\\Profiler\\NodeVisitor\\ProfilerNodeVisitor' => $vendorDir . '/twig/twig/src/Profiler/NodeVisitor/ProfilerNodeVisitor.php',
    'Twig\\Profiler\\Node\\EnterProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/EnterProfileNode.php',
    'Twig\\Profiler\\Node\\LeaveProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/LeaveProfileNode.php',
    'Twig\\Profiler\\Profile' => $vendorDir . '/twig/twig/src/Profiler/Profile.php',
    'Twig\\RuntimeLoader\\ContainerRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/ContainerRuntimeLoader.php',
    'Twig\\RuntimeLoader\\FactoryRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/FactoryRuntimeLoader.php',
    'Twig\\RuntimeLoader\\RuntimeLoaderInterface' => $vendorDir . '/twig/twig/src/RuntimeLoader/RuntimeLoaderInterface.php',
    'Twig\\Runtime\\EscaperRuntime' => $vendorDir . '/twig/twig/src/Runtime/EscaperRuntime.php',
    'Twig\\Sandbox\\SecurityError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFilterError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFilterError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFunctionError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFunctionError.php',
    'Twig\\Sandbox\\SecurityNotAllowedMethodError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedMethodError.php',
    'Twig\\Sandbox\\SecurityNotAllowedPropertyError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedPropertyError.php',
    'Twig\\Sandbox\\SecurityNotAllowedTagError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedTagError.php',
    'Twig\\Sandbox\\SecurityNotAllowedTestError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedTestError.php',
    'Twig\\Sandbox\\SecurityPolicy' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicy.php',
    'Twig\\Sandbox\\SecurityPolicyInterface' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicyInterface.php',
    'Twig\\Sandbox\\SourcePolicyInterface' => $vendorDir . '/twig/twig/src/Sandbox/SourcePolicyInterface.php',
    'Twig\\Source' => $vendorDir . '/twig/twig/src/Source.php',
    'Twig\\Template' => $vendorDir . '/twig/twig/src/Template.php',
    'Twig\\TemplateWrapper' => $vendorDir . '/twig/twig/src/TemplateWrapper.php',
    'Twig\\Test\\IntegrationTestCase' => $vendorDir . '/twig/twig/src/Test/IntegrationTestCase.php',
    'Twig\\Test\\NodeTestCase' => $vendorDir . '/twig/twig/src/Test/NodeTestCase.php',
    'Twig\\Token' => $vendorDir . '/twig/twig/src/Token.php',
    'Twig\\TokenParser\\AbstractTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AbstractTokenParser.php',
    'Twig\\TokenParser\\ApplyTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ApplyTokenParser.php',
    'Twig\\TokenParser\\AutoEscapeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AutoEscapeTokenParser.php',
    'Twig\\TokenParser\\BlockTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/BlockTokenParser.php',
    'Twig\\TokenParser\\DeprecatedTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/DeprecatedTokenParser.php',
    'Twig\\TokenParser\\DoTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/DoTokenParser.php',
    'Twig\\TokenParser\\EmbedTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/EmbedTokenParser.php',
    'Twig\\TokenParser\\ExtendsTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ExtendsTokenParser.php',
    'Twig\\TokenParser\\FlushTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FlushTokenParser.php',
    'Twig\\TokenParser\\ForTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ForTokenParser.php',
    'Twig\\TokenParser\\FromTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FromTokenParser.php',
    'Twig\\TokenParser\\GuardTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/GuardTokenParser.php',
    'Twig\\TokenParser\\IfTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IfTokenParser.php',
    'Twig\\TokenParser\\ImportTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ImportTokenParser.php',
    'Twig\\TokenParser\\IncludeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IncludeTokenParser.php',
    'Twig\\TokenParser\\MacroTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/MacroTokenParser.php',
    'Twig\\TokenParser\\SandboxTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SandboxTokenParser.php',
    'Twig\\TokenParser\\SetTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SetTokenParser.php',
    'Twig\\TokenParser\\TokenParserInterface' => $vendorDir . '/twig/twig/src/TokenParser/TokenParserInterface.php',
    'Twig\\TokenParser\\TypesTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/TypesTokenParser.php',
    'Twig\\TokenParser\\UseTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/UseTokenParser.php',
    'Twig\\TokenParser\\WithTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/WithTokenParser.php',
    'Twig\\TokenStream' => $vendorDir . '/twig/twig/src/TokenStream.php',
    'Twig\\TwigCallableInterface' => $vendorDir . '/twig/twig/src/TwigCallableInterface.php',
    'Twig\\TwigFilter' => $vendorDir . '/twig/twig/src/TwigFilter.php',
    'Twig\\TwigFunction' => $vendorDir . '/twig/twig/src/TwigFunction.php',
    'Twig\\TwigTest' => $vendorDir . '/twig/twig/src/TwigTest.php',
    'Twig\\Util\\CallableArgumentsExtractor' => $vendorDir . '/twig/twig/src/Util/CallableArgumentsExtractor.php',
    'Twig\\Util\\CallableParameters' => $vendorDir . '/twig/twig/src/Util/CallableParameters.php',
    'Twig\\Util\\DeprecationCollector' => $vendorDir . '/twig/twig/src/Util/DeprecationCollector.php',
    'Twig\\Util\\ReflectionCallable' => $vendorDir . '/twig/twig/src/Util/ReflectionCallable.php',
    'Twig\\Util\\TemplateDirIterator' => $vendorDir . '/twig/twig/src/Util/TemplateDirIterator.php',
    'UnhandledMatchError' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/UnhandledMatchError.php',
    'ValueError' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/ValueError.php',
    'Webmozart\\Assert\\Assert' => $vendorDir . '/webmozart/assert/src/Assert.php',
    'Webmozart\\Assert\\HasAssert' => $vendorDir . '/webmozart/assert/src/HasAssert.php',
    'Webmozart\\Assert\\InvalidArgumentException' => $vendorDir . '/webmozart/assert/src/InvalidArgumentException.php',
    'Webmozart\\Assert\\Mixin' => $vendorDir . '/webmozart/assert/src/Mixin.php',
    'Webmozart\\Assert\\PsalmPlugin' => $vendorDir . '/webmozart/assert/src/PsalmPlugin.php',
    '©' => $vendorDir . '/symfony/cache/Traits/ValueWrapper.php',
);
